@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- *  $Id: commands.c,v 1.36 2007/08/01 14:47:53 strk Exp $
+ *  $Id: commands.c,v 1.39 2008/07/03 15:08:34 smorlat Exp $
  *
  *  Copyright (C) 2006  Sandro Santilli <strk@keybit.net>
  *  Copyright (C) 2004  Simon MORLAT <simon.morlat@linphone.org>
@@ -293,6 +293,11 @@ lpc_cmd_help(LinphoneCore *lc, char *arg)
 static int
 lpc_cmd_call(LinphoneCore *lc, char *args)
 {
+	if ( ! args || ! *args )
+	{
+		return 0;
+	}
+
 	if ( lc->call != NULL )
 	{
 		printf("Terminate current call first.\n");
@@ -749,11 +754,12 @@ linphonec_proxy_add(LinphoneCore *lc)
 			continue;
 		}
 
-		cfg=linphone_proxy_config_new(clean);
-		if (cfg==NULL)
+		cfg=linphone_proxy_config_new();
+		if (linphone_proxy_config_set_server_addr(cfg,clean)<0)
 		{
 			printf("Invalid sip address (sip:sip.domain.tld).\n");
 			free(input);
+			linphone_proxy_config_destroy(cfg);
 			continue;
 		}
 		free(input);
@@ -1015,7 +1021,7 @@ linphonec_friend_display(LinphoneFriend *fr)
 static int
 linphonec_friend_list(LinphoneCore *lc, char *pat)
 {
-	MSList *friend;
+	const MSList *friend;
 	int n;
 
 	if (pat) {
@@ -1040,7 +1046,7 @@ linphonec_friend_list(LinphoneCore *lc, char *pat)
 static int
 linphonec_friend_call(LinphoneCore *lc, unsigned int num)
 {
-	MSList *friend = linphone_core_get_friend_list(lc);
+	const MSList *friend = linphone_core_get_friend_list(lc);
 	unsigned int n;
 	char *addr;
 
@@ -1072,7 +1078,7 @@ linphonec_friend_add(LinphoneCore *lc, const char *name, const char *addr)
 static int
 linphonec_friend_delete(LinphoneCore *lc, int num)
 {
-	MSList *friend = linphone_core_get_friend_list(lc);
+	const MSList *friend = linphone_core_get_friend_list(lc);
 	unsigned int n;
 
 	for(n=0; friend!=NULL; friend=ms_list_next(friend), ++n )
@@ -1130,6 +1136,16 @@ lpc_find_command(const char *name)
 /****************************************************************************
  *
  * $Log: commands.c,v $
+ * Revision 1.39  2008/07/03 15:08:34  smorlat
+ * api cleanups, interface in progress.
+ *
+ * Revision 1.38  2008/06/17 20:38:59  smorlat
+ * added missing file.
+ *
+ * Revision 1.37  2008/04/09 09:26:00  smorlat
+ * merge various patches
+ * H264 support.
+ *
  * Revision 1.36  2007/08/01 14:47:53  strk
  *         * console/commands.c: Clean up commands 'nat', 'stun'
  *           and 'firewall' to be more intuitive.
