@@ -54,6 +54,14 @@ void linphone_auth_info_set_passwd(LinphoneAuthInfo *info, const char *passwd){
 	if (passwd!=NULL && (strlen(passwd)>0)) info->passwd=ms_strdup(passwd);
 }
 
+void linphone_auth_info_set_username(LinphoneAuthInfo *info, const char *username){
+	if (info->username){
+		ms_free(info->username);
+		info->username=NULL;
+	}
+	if (username && strlen(username)>0) info->username=ms_strdup(username);
+}
+
 void linphone_auth_info_destroy(LinphoneAuthInfo *obj){
 	if (obj->username!=NULL) ms_free(obj->username);
 	if (obj->userid!=NULL) ms_free(obj->userid);
@@ -122,6 +130,14 @@ static int auth_info_compare(const void *pinfo,const void *pref){
 	return -1;
 }
 
+static int auth_info_compare_only_realm(const void *pinfo,const void *pref){
+	LinphoneAuthInfo *info=(LinphoneAuthInfo*)pinfo;
+	LinphoneAuthInfo *ref=(LinphoneAuthInfo*)pref;
+	if (key_match(info->realm,ref->realm) ) return 0;
+	return -1;
+}
+
+
 LinphoneAuthInfo *linphone_core_auth_info_find(LinphoneCore *lc, const char *realm, const char *username)
 {
 	LinphoneAuthInfo ref;
@@ -129,7 +145,10 @@ LinphoneAuthInfo *linphone_core_auth_info_find(LinphoneCore *lc, const char *rea
 	ref.realm=(char*)realm;
 	ref.username=(char*)username;
 	elem=ms_list_find_custom(lc->auth_info,auth_info_compare,(void*)&ref);
-	if (elem==NULL) return NULL;
+	if (elem==NULL) {
+		elem=ms_list_find_custom(lc->auth_info,auth_info_compare_only_realm,(void*)&ref);
+		if (elem==NULL) return NULL;
+	}
 	return (LinphoneAuthInfo*)elem->data;
 }
 
@@ -158,6 +177,10 @@ void linphone_core_add_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info)
 	}
 }
 
+void linphone_core_abort_authentication(LinphoneCore *lc,  LinphoneAuthInfo *info){
+	if (lc->automatic_action>0) lc->automatic_action--;
+}
+
 void linphone_core_remove_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info){
 	int len=ms_list_size(lc->auth_info);
 	int newlen;
@@ -225,16 +248,14 @@ void linphone_authentication_ok(LinphoneCore *lc, eXosip_event_t *ev){
 
 void linphone_core_find_or_ask_for_auth_info(LinphoneCore *lc,const char *username,const char* realm, int tid)
 {
-	LinphoneAuthInfo *as;
-	if ((as=linphone_core_auth_info_find(lc,realm,username))==NULL || 
-		(as->works==FALSE && as->first_time==FALSE) ) {
-		if (lc->vtable.auth_info_requested!=NULL) {
+	LinphoneAuthInfo *as=linphone_core_auth_info_find(lc,realm,username);
+	if ( as==NULL || (as!=NULL && as->works==FALSE && as->first_time==FALSE)){
+		if (lc->vtable.auth_info_requested!=NULL){
 			lc->vtable.auth_info_requested(lc,realm,username);
 			lc->automatic_action++;/*suspends eXosip_automatic_action until the user supplies a password */
-		}else{
-			if (as) as->first_time=FALSE;
 		}
 	}
+	if (as) as->first_time=FALSE;
 }
 
 void linphone_process_authentication(LinphoneCore *lc, eXosip_event_t *ev)
@@ -245,7 +266,9 @@ void linphone_process_authentication(LinphoneCore *lc, eXosip_event_t *ev)
 	osip_message_t *resp=ev->response;
 	char *username;
 	
-	gstate_new_state(lc, GSTATE_REG_FAILED, "Authentication required");
+	if (strcmp(ev->request->sip_method,"REGISTER")==0) {
+		gstate_new_state(lc, GSTATE_REG_FAILED, "Authentication required");
+	}
 
 	username=osip_uri_get_username(resp->from->url);
 	prx_auth=(osip_proxy_authenticate_t*)osip_list_get(&resp->proxy_authenticates,0);
