@@ -55,7 +55,7 @@ static void linphone_connect_incoming(LinphoneCore *lc){
 int linphone_call_accepted(LinphoneCore *lc, eXosip_event_t *ev)
 {
 	LinphoneCall *call=lc->call;
-	sdp_message_t *sdp=eXosip_get_sdp_info(ev->response);
+	sdp_message_t *sdp;
 	const char *sdpanswer=NULL;
 	osip_message_t *msg=NULL;
 	int err;
@@ -69,6 +69,7 @@ int linphone_call_accepted(LinphoneCore *lc, eXosip_event_t *ev)
 		return 0; /*already accepted*/
 	}
 	linphone_call_init_media_params(call);
+	sdp=eXosip_get_sdp_info(ev->response);
 	if (!lc->sip_conf.sdp_200_ack){
 		err=0;
 		sdp_context_read_answer(call->sdpctx,sdp);
@@ -90,6 +91,7 @@ int linphone_call_accepted(LinphoneCore *lc, eXosip_event_t *ev)
 		ms_error("Incompatible SDP offer received in 200Ok, need to abort the call");
 		linphone_core_terminate_call(lc,NULL);
 	}
+	sdp_message_free(sdp);
 	return 0;
 }
 
@@ -247,7 +249,7 @@ static int linphone_answer_sdp(LinphoneCore *lc, eXosip_event_t *ev, sdp_message
 
 int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
 {
-	sdp_message_t *sdp=eXosip_get_sdp_info(ev->request);
+	sdp_message_t *sdp=NULL;
 	osip_from_t *from_url=ev->request->from;
 	char *barmesg;
 	char *from;
@@ -297,7 +299,7 @@ int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
 		goto end;
 	}
 	lc->call=linphone_call_new_incoming(lc,from,to,ev->cid,ev->did,ev->tid);
-	gstate_new_state(lc, GSTATE_CALL_IN_INVITE, NULL);
+	sdp=eXosip_get_sdp_info(ev->request);
 	if (sdp==NULL){
 		ms_message("No sdp body in invite, 200-ack scheme");
 		err=0;
@@ -309,6 +311,7 @@ int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
 		if (from_2char_without_params(from_url,&tmp)!=0){
 			tmp=ms_strdup("Unknown user");
 		}
+		gstate_new_state(lc, GSTATE_CALL_IN_INVITE, tmp);
 		barmesg=ortp_strdup_printf("%s %s",tmp,_("is contacting you."));
 		lc->vtable.show(lc);
 		lc->vtable.display_status(lc,barmesg);
@@ -336,6 +339,7 @@ int linphone_inc_new_call(LinphoneCore *lc, eXosip_event_t *ev)
 	end:
 	osip_free(from);
 	osip_free(to);
+	if (sdp) sdp_message_free(sdp);
 	return 0;
 }
 
@@ -344,6 +348,7 @@ void linphone_handle_ack(LinphoneCore *lc, eXosip_event_t *ev){
 	if (sdp){
 		sdp_context_read_answer(lc->call->sdpctx,sdp);
 		linphone_connect_incoming(lc);
+		sdp_message_free(sdp);
 	}
 }
 
@@ -370,7 +375,13 @@ void linphone_handle_reinvite(LinphoneCore *lc, eXosip_event_t *ev){
 		linphone_core_stop_media_streams(lc);
 		linphone_core_init_media_streams(lc);
 		eXosip_lock();
-		eXosip_call_build_answer(ev->tid,200,&msg);
+		if (eXosip_call_build_answer(ev->tid,200,&msg)<0){
+			ms_warning("Reinvite for closed call ?");
+                        eXosip_unlock();
+                        linphone_core_stop_media_streams(lc);
+			sdp_message_free(sdp);
+			return ;
+		}
 		answer=call->sdpctx->answerstr;	/* takes the sdp already computed*/
 		linphone_set_sdp(msg,answer);
 		eXosip_call_send_answer(ev->tid,200,msg);
@@ -381,6 +392,7 @@ void linphone_handle_reinvite(LinphoneCore *lc, eXosip_event_t *ev){
 		eXosip_call_send_answer(ev->tid,status,NULL);
 		eXosip_unlock();
 	}
+	sdp_message_free(sdp);
 }
 
 void linphone_do_automatic_redirect(LinphoneCore *lc, const char *contact){
@@ -431,7 +443,8 @@ int linphone_set_audio_offer(sdp_context_t *ctx)
 			sdp_payload_init(&payload);
 			payload.a_rtpmap=ortp_strdup_printf("%s/%i/1",codec->mime_type,codec->clock_rate);
 			payload.pt=rtp_profile_get_payload_number_from_rtpmap(lc->local_profile,payload.a_rtpmap);
-			payload.localport=lc->rtp_conf.audio_rtp_port;
+			payload.localport=call->audio_params.natd_port > 0 ? 
+						call->audio_params.natd_port : lc->rtp_conf.audio_rtp_port;
 			if (strcasecmp(codec->mime_type,"iLBC")==0){
 				/* prefer the 30 ms mode */
 				payload.a_fmtp="ptime=30";
@@ -451,6 +464,43 @@ int linphone_set_audio_offer(sdp_context_t *ctx)
 	return 0;
 }
 
+static int find_payload_type_number(RtpProfile *prof, PayloadType *pt){
+	int candidate=-1,i;
+	PayloadType *it;
+	for(i=0;i<127;++i){
+		it=rtp_profile_get_payload(prof,i);
+		if (it!=NULL && strcasecmp(pt->mime_type,it->mime_type)==0 
+			&& (pt->clock_rate==it->clock_rate || pt->clock_rate<=0) ){
+			if ( (pt->recv_fmtp && it->recv_fmtp && strcasecmp(pt->recv_fmtp,it->recv_fmtp)==0) ||
+				(pt->recv_fmtp==NULL && it->recv_fmtp==NULL) ){
+				/*exact match*/
+				return i;
+			}else candidate=i;
+		}
+	}
+	if (candidate==-1) ms_fatal("Should not happen.");
+	return candidate;
+}
+
+static int find_payload_type_number_best_match(RtpProfile *prof, const char *rtpmap, const char *fmtp){
+	int localpt=rtp_profile_get_payload_number_from_rtpmap(prof,rtpmap);
+	PayloadType *pt;
+	char value[10];
+	if (localpt<0) return -1;
+	pt=rtp_profile_get_payload(prof,localpt);
+	if (strcasecmp(pt->mime_type,"H264")==0){
+		/*hack for H264: need to answer with same packetization-mode*/
+		PayloadType tmp;
+		memset(&tmp,0,sizeof(tmp));
+		tmp.mime_type="H264";
+		tmp.clock_rate=pt->clock_rate;
+		if (fmtp && fmtp_get_value(fmtp,"packetization-mode",value,sizeof(value))){
+			tmp.recv_fmtp=(atoi(value)==1) ? "packetization-mode=1" : NULL;
+		}
+		localpt=find_payload_type_number(prof,&tmp);
+	}
+	return localpt;
+}
 
 int linphone_set_video_offer(sdp_context_t *ctx)
 {
@@ -469,8 +519,9 @@ int linphone_set_video_offer(sdp_context_t *ctx)
 			sdp_payload_init(&payload);
 			payload.line=1;
 			payload.a_rtpmap=ortp_strdup_printf("%s/%i",codec->mime_type,codec->clock_rate);
-			payload.localport=lc->rtp_conf.video_rtp_port;
-			payload.pt=rtp_profile_get_payload_number_from_rtpmap(lc->local_profile,payload.a_rtpmap);
+			payload.localport=call->video_params.natd_port>0 ? 
+					call->video_params.natd_port : lc->rtp_conf.video_rtp_port;
+			payload.pt=find_payload_type_number(lc->local_profile,codec);
 			payload.a_fmtp=codec->recv_fmtp;
 			if(firsttime){
 				firsttime=FALSE;
@@ -490,12 +541,12 @@ typedef enum {
 	SupportedAndValid  /* valid= the presence of this codec is enough to make a call */
 }SupportLevel;
 
-SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payload,RtpProfile *local_profile,RtpProfile *dialog_profile, bool_t answering)
+SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payload,RtpProfile *local_profile,RtpProfile *dialog_profile, bool_t answering, PayloadType **local_payload_type)
 {
 	int localpt;
 	SupportLevel ret;
 	if (payload->a_rtpmap!=NULL){
-		localpt=rtp_profile_get_payload_number_from_rtpmap(local_profile,payload->a_rtpmap);
+		localpt=find_payload_type_number_best_match(local_profile,payload->a_rtpmap,payload->a_fmtp);
 	}else{
 		localpt=payload->pt;
 		ms_warning("payload has no rtpmap.");
@@ -509,6 +560,7 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 			ms_warning("strange error !!");
 			return Unsupported;
 		}
+		*local_payload_type=rtppayload;
 		if (strcmp(rtppayload->mime_type,"telephone-event")!=0){
 			if (answering && !linphone_core_check_payload_type_usability(lc,rtppayload) ){
 				ms_warning("payload %s is not usable",rtppayload->mime_type);
@@ -535,6 +587,7 @@ SupportLevel linphone_payload_is_supported(LinphoneCore *lc, sdp_payload_t *payl
 				ubw=lc->up_audio_bw;
 			}
 			if (payload->b_as_bandwidth!=0){
+				ms_message("Remote bandwidth constraint: %i",payload->b_as_bandwidth);
 				/*obey to remote bandwidth constraint AND our own upbandwidth constraint*/
 				rtppayload->normal_bitrate=1000*get_min_bandwidth(
 					payload->b_as_bandwidth, ubw);
@@ -567,28 +620,44 @@ int linphone_accept_audio_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 	SupportLevel supported;
 	LinphoneCall *call=(LinphoneCall*)sdp_context_get_user_pointer(ctx);
 	LinphoneCore *lc=call->core;
-	
+	PayloadType *lpt=NULL;
+
+	params=&call->audio_params;
 	if (call->profile==NULL){
 		/* create a remote user agent profile */
 		call->profile=remote_profile=rtp_profile_new("remote");
 	}
 	remote_profile=call->profile;
 	/* see if this codec is supported in our local rtp profile*/
-	supported=linphone_payload_is_supported(lc,payload,lc->local_profile,remote_profile,TRUE);
+	supported=linphone_payload_is_supported(lc,payload,lc->local_profile,remote_profile,TRUE,&lpt);
 	if (supported==Unsupported) {
 		ms_message("Refusing audio codec %i (%s)",payload->pt,payload->a_rtpmap);
 		return -1;
 	}
-	if (supported==SupportedAndValid) {
-		params=&call->audio_params;
+	if (lc->sip_conf.only_one_codec && params->initialized){
+		return -1;
+	}
+	if (supported==SupportedAndValid) {		
 		if (params->initialized==0){
-			/* this is the first codec we may accept*/
-			params->localport=payload->localport=lc->rtp_conf.audio_rtp_port;
-			params->remoteport=payload->remoteport;
+			/* this is the first codec we accept, it is going to be used*/
+			params->localport=lc->rtp_conf.audio_rtp_port;
+			payload->localport=params->natd_port>0 ? 
+				params->natd_port : lc->rtp_conf.audio_rtp_port;
 			params->line=payload->line;
 			params->pt=payload->pt; /* remember the first payload accepted */
-			params->remoteaddr=payload->c_addr;
+			if (payload->relay_host!=NULL){
+				strncpy(params->remoteaddr,payload->relay_host,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->relay_port;
+				params->remotertcpport=payload->relay_port;
+				params->relay_session_id=payload->relay_session_id;
+			}else{
+				strncpy(params->remoteaddr,payload->c_addr,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->remoteport;
+				params->remotertcpport=payload->remoteport+1;
+			}
 			params->initialized=1;
+			/* we can now update the allocated bandwidth for audio, and then video*/
+			linphone_core_update_allocated_audio_bandwidth(lc,lpt);
 		}else{
 			/* refuse all other audio lines*/
 			if(params->line!=payload->line) return -1;
@@ -604,6 +673,7 @@ int linphone_accept_video_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 	RtpProfile *remote_profile;
 	StreamParams *params;
 	SupportLevel supported;
+	PayloadType *lpt=NULL;
 
 	if (!linphone_core_video_enabled(lc)) return -1;
 
@@ -611,22 +681,34 @@ int linphone_accept_video_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 		/* create a remote user agent profile */
 		call->profile=rtp_profile_new("remote");
 	}
+	params=&call->video_params;
 	remote_profile=call->profile;
 	/* see if this codec is supported in our local rtp profile*/
-	supported=linphone_payload_is_supported(lc,payload,lc->local_profile,remote_profile,TRUE);
+	supported=linphone_payload_is_supported(lc,payload,lc->local_profile,remote_profile,TRUE,&lpt);
 	if (supported==Unsupported) {
 		ms_message("Refusing video codec %i (%s)",payload->pt,payload->a_rtpmap);
 		return -1;
 	}
+	if (lc->sip_conf.only_one_codec && params->initialized){
+		return -1;
+	}
 	if (supported==SupportedAndValid){
-		params=&call->video_params;
 		if (params->initialized==0){
 			/* this is the first codec we may accept*/
-			params->localport=payload->localport=lc->rtp_conf.video_rtp_port;
-			params->remoteport=payload->remoteport;
+			params->localport=lc->rtp_conf.video_rtp_port;
+			payload->localport=params->natd_port>0 ? params->natd_port : lc->rtp_conf.video_rtp_port;
 			params->line=payload->line;
 			params->pt=payload->pt; /* remember the first payload accepted */
-			params->remoteaddr=payload->c_addr;
+			if (payload->relay_host!=NULL){
+				strncpy(params->remoteaddr,payload->relay_host,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->relay_port;
+				params->remotertcpport=payload->relay_port;
+				params->relay_session_id=payload->relay_session_id;
+			}else{
+				strncpy(params->remoteaddr,payload->c_addr,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->remoteport;
+				params->remotertcpport=params->remoteport+1;
+			}
 			params->initialized=1;
 		}else{
 			/* refuse all other video lines*/
@@ -642,9 +724,10 @@ int linphone_read_audio_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	LinphoneCore *lc=call->core;
 	StreamParams *params;
 	SupportLevel supported;
+	PayloadType *lpt=NULL;
 	
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
-	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE);
+	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE,&lpt);
 	if (supported==Unsupported) {
 		ms_warning("This remote sip phone did not answer properly to my sdp offer: rtpmap=%s",payload->a_rtpmap);
 		return 0;
@@ -652,13 +735,24 @@ int linphone_read_audio_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	if (supported==SupportedAndValid){
 		params=&call->audio_params;
 		if (params->initialized==0){
-			/* this is the first codec we may accept*/
+			/* this is the first codec we accept, this is the one that is going to be used (at least for sending
+			data.*/
 			params->localport=lc->rtp_conf.audio_rtp_port;
-			params->remoteport=payload->remoteport;
 			params->line=payload->line;
 			params->pt=payload->pt; /* remember the first payload accepted */
-			params->remoteaddr=payload->c_addr;
+			if (payload->relay_host!=NULL){
+				strncpy(params->remoteaddr,payload->relay_host,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->relay_port;
+				params->remotertcpport=payload->relay_port;
+				params->relay_session_id=payload->relay_session_id;
+			}else{
+				strncpy(params->remoteaddr,payload->c_addr,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->remoteport;
+				params->remotertcpport=payload->remoteport+1;
+			}
 			params->initialized=1;
+			/* we can now update the allocated bandwidth for audio, and then video*/
+			linphone_core_update_allocated_audio_bandwidth(lc,lpt);
 		}
 	}
 	return 0;
@@ -670,9 +764,10 @@ int linphone_read_video_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 	LinphoneCore *lc=call->core;
 	StreamParams *params;
 	SupportLevel supported;
+	PayloadType *lpt=NULL;
 	
 	/* paranoid check: see if this codec is supported in our local rtp profile*/
-	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE);
+	supported=linphone_payload_is_supported(lc, payload,lc->local_profile,call->profile,FALSE,&lpt);
 	if (supported==Unsupported) {
 		ms_warning("This remote sip phone did not answer properly to my sdp offer: rtpmap=%s",payload->a_rtpmap);
 		return 0;
@@ -682,10 +777,18 @@ int linphone_read_video_answer(sdp_context_t *ctx,sdp_payload_t *payload)
 		if (params->initialized==0){
 			/* this is the first codec we may accept*/
 			params->localport=lc->rtp_conf.video_rtp_port;
-			params->remoteport=payload->remoteport;
 			params->line=payload->line;
 			params->pt=payload->pt; /* remember the first payload accepted */
-			params->remoteaddr=payload->c_addr;
+			if (payload->relay_host!=NULL){
+				strncpy(params->remoteaddr,payload->relay_host,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->relay_port;
+				params->remotertcpport=payload->relay_port;
+				params->relay_session_id=payload->relay_session_id;
+			}else{
+				strncpy(params->remoteaddr,payload->c_addr,sizeof(params->remoteaddr)-1);
+				params->remoteport=payload->remoteport;
+				params->remotertcpport=payload->remoteport+1;
+			}
 			params->initialized=1;
 		}
 	}
@@ -798,22 +901,49 @@ void linphone_registration_success(LinphoneCore *lc,eXosip_event_t *ev){
 	msg=ms_strdup_printf(_("Registration on %s successful."),ru);
 	lc->vtable.display_status(lc,msg);
 	ms_free(msg);
+	osip_free(ru);
 	cfg=linphone_core_get_proxy_config_from_rid(lc,ev->rid);
 	ms_return_if_fail(cfg!=NULL);
 	cfg->auth_pending=FALSE;
 	cfg->registered=TRUE;
 	gstate_new_state(lc, GSTATE_REG_OK, NULL);
+	linphone_proxy_config_register_again_with_updated_contact(cfg,ev->request,ev->response);
+}
+
+static bool_t comes_from_local_if(osip_message_t *msg){
+	osip_via_t *via=NULL;
+	osip_message_get_via(msg,0,&via);
+	if (via){
+		const char *host;
+		host=osip_via_get_host(via);
+		if (strcmp(host,"127.0.0.1")==0 || strcmp(host,"::1")==0){
+			osip_generic_param_t *param=NULL;
+			osip_via_param_get_byname(via,"received",&param);
+			if (param==NULL) return TRUE;
+			if (param->gvalue && 
+				(strcmp(param->gvalue,"127.0.0.1")==0 || strcmp(param->gvalue,"::1")==0)){
+				return TRUE;
+			}
+		}
+	}
+	return FALSE;
 }
 
 static void linphone_other_request(LinphoneCore *lc, eXosip_event_t *ev){
+	ms_message("in linphone_other_request");
 	if (ev->request==NULL) return;
 	if (strcmp(ev->request->sip_method,"MESSAGE")==0){
 		linphone_core_text_received(lc,ev);
 		eXosip_message_send_answer(ev->tid,200,NULL);
 	}else if (strcmp(ev->request->sip_method,"OPTIONS")==0){
 		eXosip_options_send_answer(ev->tid,200,NULL);
-	}
-	else {
+	}else if (strcmp(ev->request->sip_method,"WAKEUP")==0
+		&& comes_from_local_if(ev->request)) {
+		eXosip_message_send_answer(ev->tid,200,NULL);
+		ms_message("Receiving WAKEUP request !");
+		if (lc->vtable.show)
+			lc->vtable.show(lc);
+	}else {
 		ms_message("Unsupported request received.");
 		/*answer with a 501 Not implemented*/
 		eXosip_message_send_answer(ev->tid,501,NULL);
@@ -874,6 +1004,12 @@ void linphone_core_process_event(LinphoneCore *lc,eXosip_event_t *ev)
 			ms_message("EXOSIP_CALL_MESSAGE_NEW");
 			linphone_call_message_new(lc,ev);
 			break;
+		case EXOSIP_CALL_MESSAGE_REQUESTFAILURE:
+			if (ev->did<0 && ev->response && 
+				(ev->response->status_code==407 || ev->response->status_code==401)){
+				 eXosip_default_action(ev);
+			}
+			break;
 		case EXOSIP_IN_SUBSCRIPTION_NEW:
 			ms_message("CALL_SUBSCRIPTION_NEW or UPDATE");
 			linphone_subscription_new(lc,ev);
@@ -901,13 +1037,20 @@ void linphone_core_process_event(LinphoneCore *lc,eXosip_event_t *ev)
 			linphone_registration_faillure(lc,ev);
 			break;
 		case EXOSIP_REGISTRATION_SUCCESS:
-			linphone_registration_success(lc,ev);
 			linphone_authentication_ok(lc,ev);
+			linphone_registration_success(lc,ev);
 			break;
 		case EXOSIP_MESSAGE_NEW:
 			linphone_other_request(lc,ev);
 			break;
+		case EXOSIP_MESSAGE_REQUESTFAILURE:
+			if (ev->response && (ev->response->status_code == 407 || ev->response->status_code == 401)){
+				/*the user is expected to have registered to the proxy, thus password is known*/
+				eXosip_default_action(ev);
+			}
+			break;
 		default:
+			ms_message("Unhandled exosip event !");
 			break;
 	}
 	eXosip_event_free(ev);
