@@ -28,6 +28,53 @@
 #include <osipparser2/osip_message.h>
 #include "lpconfig.h"
 
+const char *linphone_online_status_to_string(LinphoneOnlineStatus ss){
+	const char *str=NULL;
+	switch(ss){
+		case LINPHONE_STATUS_UNKNOWN:
+		str=_("Unknown");
+		break;
+		case LINPHONE_STATUS_ONLINE:
+		str=_("Online");
+		break;
+		case LINPHONE_STATUS_BUSY:
+		str=_("Busy");
+		break;
+		case LINPHONE_STATUS_BERIGHTBACK:
+		str=_("Be right back");
+		break;
+		case LINPHONE_STATUS_AWAY:
+		str=_("Away");
+		break;
+		case LINPHONE_STATUS_ONTHEPHONE:
+		str=_("On the phone");
+		break;
+		case LINPHONE_STATUS_OUTTOLUNCH:
+		str=_("Out to lunch");
+		break;
+		case LINPHONE_STATUS_NOT_DISTURB:
+		str=_("Do not disturb");
+		break;
+		case LINPHONE_STATUS_MOVED:
+		str=_("Moved");
+		break;
+		case LINPHONE_STATUS_ALT_SERVICE:
+		str=_("Using another messaging service");
+		break;
+		case LINPHONE_STATUS_OFFLINE:
+		str=_("Offline");
+		break;
+		case LINPHONE_STATUS_PENDING:
+		str=_("Pending");
+		break;
+		case LINPHONE_STATUS_CLOSED:
+		str=_("Closed");
+		break;
+		default:
+		str=_("Unknown-bug");
+	}
+	return str;
+}
 
 static int friend_data_compare(const void * a, const void * b, void * data){
 	osip_from_t *fa=((LinphoneFriend*)a)->url;
@@ -114,6 +161,7 @@ LinphoneFriend * linphone_friend_new(){
 	obj->nid=-1;
 	obj->sid=-1;
 	obj->pol=LinphoneSPAccept;
+	obj->status=LINPHONE_STATUS_OFFLINE;
 	obj->subscribe=TRUE;
 	return obj;	
 }
@@ -143,7 +191,21 @@ int linphone_friend_set_sip_addr(LinphoneFriend *lf, const char *addr){
 	return 0;
 }
 
-
+int linphone_friend_set_name(LinphoneFriend *lf, const char *name){
+	osip_from_t *fr=lf->url;
+	if (fr==NULL){
+		ms_error("linphone_friend_set_sip_addr() must be called before linphone_friend_set_name().");
+		return -1;
+	}
+	if (fr->displayname!=NULL){
+		osip_free(fr->displayname);
+		fr->displayname=NULL;
+	}
+	if (name && name[0]!='\0'){
+		fr->displayname=osip_strdup(name);
+	}
+	return 0;
+}
 
 int linphone_friend_send_subscribe(LinphoneFriend *fr, bool_t val){
 	fr->subscribe=val;
@@ -503,6 +565,17 @@ char * linphone_friend_get_url(LinphoneFriend *lf){
 	return ret;
 }
 
+bool_t linphone_friend_get_send_subscribe(const LinphoneFriend *lf){
+	return lf->subscribe;
+}
+
+LinphoneSubscribePolicy linphone_friend_get_inc_subscribe_policy(const LinphoneFriend *lf){
+	return lf->pol;
+}
+
+LinphoneOnlineStatus linphone_friend_get_status(const LinphoneFriend *lf){
+	return lf->status;
+}
 
 
 void linphone_friend_apply(LinphoneFriend *fr, LinphoneCore *lc){
@@ -550,7 +623,7 @@ void linphone_core_add_friend(LinphoneCore *lc, LinphoneFriend *lf)
 	ms_return_if_fail(lf->lc==NULL);
 	ms_return_if_fail(lf->url!=NULL);
 	linphone_friend_apply(lf,lc);
-	lc->friends=ms_list_insert_sorted(lc->friends,(void *)lf,friend_compare);
+	lc->friends=ms_list_append(lc->friends,lf);
 	return ;
 }
 
