@@ -25,7 +25,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "sdphandler.h"
 
 #include <ortp/telephonyevents.h>
-#include <ortp/stun.h>
 #include "exevents.h"
 
 
@@ -44,8 +43,6 @@ static const char *liblinphone_version=LIBLINPHONE_VERSION;
 
 #include "enum.h"
 
-void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val);
-bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc);
 void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result);
 static void apply_nat_settings(LinphoneCore *lc);
 static void toggle_video_preview(LinphoneCore *lc, bool_t val);
@@ -73,16 +70,17 @@ void lc_callback_obj_init(LCCallbackObj *obj,LinphoneCoreCbFunc func,void* ud)
 }
 
 int lc_callback_obj_invoke(LCCallbackObj *obj, LinphoneCore *lc){
-  if (obj->_func!=NULL) obj->_func(lc,obj->_user_data);
-  return 0;
+	if (obj->_func!=NULL) obj->_func(lc,obj->_user_data);
+	return 0;
 }
 
 static void  linphone_call_init_common(LinphoneCall *call, char *from, char *to){
-	sdp_context_set_user_pointer(call->sdpctx,(void*)call);
 	call->state=LCStateInit;
 	call->start_time=time(NULL);
 	call->log=linphone_call_log_new(call, from, to);
 	linphone_core_notify_all_friends(call->core,LINPHONE_STATUS_ONTHEPHONE);
+	if (linphone_core_get_firewall_policy(call->core)==LINPHONE_POLICY_USE_STUN) 
+		linphone_core_run_stun_tests(call->core,call);
 }
 
 void linphone_call_init_media_params(LinphoneCall *call){
@@ -90,6 +88,19 @@ void linphone_call_init_media_params(LinphoneCall *call){
 	memset(&call->video_params,0,sizeof(call->video_params));
 }
 
+static void discover_mtu(LinphoneCore *lc, const char *remote){
+	int mtu;
+	if (lc->net_conf.mtu==0	){
+		/*attempt to discover mtu*/
+		mtu=ms_discover_mtu(remote);
+		if (mtu>0){
+			ms_set_mtu(mtu);
+			ms_message("Discovered mtu is %i, RTP payload max size is %i",
+				mtu, ms_get_payload_max_size());
+		}
+	}
+}
+
 LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_from_t *from, const osip_to_t *to)
 {
 	LinphoneCall *call=ms_new0(LinphoneCall,1);
@@ -101,10 +112,14 @@ LinphoneCall * linphone_call_new_outgoing(struct _LinphoneCore *lc, const osip_f
 	call->tid=-1;
 	call->core=lc;
 	linphone_core_get_local_ip(lc,to->url->host,localip);
-	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,localip,from->url->username);
 	osip_from_to_str(from,&fromstr);
 	osip_to_to_str(to,&tostr);
 	linphone_call_init_common(call,fromstr,tostr);
+	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,
+		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : localip,
+		from->url->username,NULL);
+	sdp_context_set_user_pointer(call->sdpctx,(void*)call);
+	discover_mtu(lc,to->url->host);
 	return call;
 }
 
@@ -123,8 +138,12 @@ LinphoneCall * linphone_call_new_incoming(LinphoneCore *lc, const char *from, co
 	osip_from_init(&from_url);
 	osip_from_parse(from_url, from);
 	linphone_core_get_local_ip(lc,from_url->url->host,localip);
-	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,localip,me->url->username);
 	linphone_call_init_common(call, osip_strdup(from), osip_strdup(to));
+	call->sdpctx=sdp_handler_create_context(&linphone_sdphandler,
+		call->audio_params.natd_port>0 ? call->audio_params.natd_addr : localip,
+		me->url->username,NULL);
+	sdp_context_set_user_pointer(call->sdpctx,(void*)call);
+	discover_mtu(lc,from_url->url->host);
 	osip_from_free(me);
 	osip_from_free(from_url);
 	return call;
@@ -301,11 +320,7 @@ net_config_read (LinphoneCore *lc)
 	tmp=lp_config_get_int(lc->config,"net","nat_sdp_only",0);
 	lc->net_conf.nat_sdp_only=tmp;
 	tmp=lp_config_get_int(lc->config,"net","mtu",0);
-	if (tmp>0){
-		int pmax=tmp-40-8-12;
-		ms_message("MTU is %i, rtp payload max size will be %i",tmp,pmax);
-		ms_set_payload_max_size(pmax);
-	}
+	linphone_core_set_mtu(lc,tmp);
 }
 
 
@@ -451,13 +466,14 @@ void sip_config_read(LinphoneCore *lc)
 	}
 	/*for test*/
 	lc->sip_conf.sdp_200_ack=lp_config_get_int(lc->config,"sip","sdp_200_ack",0);
-	
+	lc->sip_conf.only_one_codec=lp_config_get_int(lc->config,"sip","only_one_codec",0);
 }
 
 void rtp_config_read(LinphoneCore *lc)
 {
 	int port;
 	int jitt_comp;
+	int nortp_timeout;
 	port=lp_config_get_int(lc->config,"rtp","audio_rtp_port",7078);
 	linphone_core_set_audio_port(lc,port);
 	
@@ -468,12 +484,14 @@ void rtp_config_read(LinphoneCore *lc)
 	jitt_comp=lp_config_get_int(lc->config,"rtp","audio_jitt_comp",60);
 	linphone_core_set_audio_jittcomp(lc,jitt_comp);		
 	jitt_comp=lp_config_get_int(lc->config,"rtp","video_jitt_comp",60);
+	nortp_timeout=lp_config_get_int(lc->config,"rtp","nortp_timeout",30);
+	linphone_core_set_nortp_timeout(lc,nortp_timeout);	
 }
 
 
 PayloadType * get_codec(LpConfig *config, char* type,int index){
 	char codeckey[50];
-	const char *mime;
+	const char *mime,*fmtp;
 	int rate,enabled;
 	PayloadType *pt;
 	
@@ -486,7 +504,8 @@ PayloadType * get_codec(LpConfig *config, char* type,int index){
 	
 	rate=lp_config_get_int(config,codeckey,"rate",8000);
 	pt->clock_rate=rate;
-	
+	fmtp=lp_config_get_string(config,codeckey,"recv_fmtp",NULL);
+	if (fmtp) pt->recv_fmtp=ms_strdup(fmtp);
 	enabled=lp_config_get_int(config,codeckey,"enabled",1);
 	if (enabled ) pt->flags|=PAYLOAD_TYPE_ENABLED;
 	//ms_message("Found codec %s/%i",pt->mime_type,pt->clock_rate);
@@ -519,9 +538,24 @@ void video_config_read(LinphoneCore *lc)
 	int capture, display;
 	int enabled;
 	const char *str;
-	
-	str=lp_config_get_string(lc->config,"video","device","/dev/video0");
-	linphone_core_set_video_device(lc,NULL,str);
+	int ndev;
+	const char **devices;
+	const MSList *elem;
+	int i;
+
+	/* retrieve all video devices */
+	elem=ms_web_cam_manager_get_list(ms_web_cam_manager_get());
+	ndev=ms_list_size(elem);
+	devices=ms_malloc((ndev+1)*sizeof(const char *));
+	for (i=0;elem!=NULL;elem=elem->next,i++){
+		devices[i]=ms_web_cam_get_string_id((MSWebCam *)elem->data);
+	}
+	devices[ndev]=NULL;
+	lc->video_conf.cams=devices;
+
+	str=lp_config_get_string(lc->config,"video","device",NULL);
+	if (str && str[0]==0) str=NULL;
+	linphone_core_set_video_device(lc,str);
 	
 	enabled=lp_config_get_int(lc->config,"video","enabled",1);
 	capture=lp_config_get_int(lc->config,"video","capture",enabled);
@@ -557,18 +591,19 @@ void linphone_core_set_download_bandwidth(LinphoneCore *lc, int bw){
 	if (bw==0){ /*infinite*/
 		lc->dw_audio_bw=-1;
 		lc->dw_video_bw=-1;
-		return;
-	}else if (bw>=512){
-		lc->dw_audio_bw=80;
-	}else if (bw>=128){
-		if (linphone_core_video_enabled(lc))
-			lc->dw_audio_bw=30;
-		else
+	}else {
+		if (bw>=256){
+			lc->dw_audio_bw=80;
+		}else if (bw>=128){
+			if (linphone_core_video_enabled(lc))
+				lc->dw_audio_bw=30;
+			else
+				lc->dw_audio_bw=bw;
+		}else{
 			lc->dw_audio_bw=bw;
-	}else{
-		lc->dw_audio_bw=bw;
+		}
+		lc->dw_video_bw=bw-lc->dw_audio_bw;
 	}
-	lc->dw_video_bw=bw-lc->dw_audio_bw;
 }
 
 void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw){
@@ -577,17 +612,15 @@ void linphone_core_set_upload_bandwidth(LinphoneCore *lc, int bw){
 		lc->up_audio_bw=-1;
 		lc->up_video_bw=-1;
 		return;
-	}else if (bw>=512){
-		lc->up_audio_bw=80;
 	}else if (bw>=128){
 		if (linphone_core_video_enabled(lc))
-			lc->up_audio_bw=30;
+			lc->up_audio_bw=lc->audio_bw;
 		else 
 			lc->up_audio_bw=bw;
 	}else{
 		lc->up_audio_bw=bw;
 	}
-	lc->up_video_bw=bw-lc->up_audio_bw;
+	lc->up_video_bw=bw-lc->up_audio_bw-10;/*-10: security margin*/
 }
 
 int linphone_core_get_download_bandwidth(const LinphoneCore *lc){
@@ -602,7 +635,12 @@ const char * linphone_core_get_version(void){
 	return liblinphone_version;
 }
 
-extern PayloadType payload_type_x_snow;
+#ifdef VIDEO_ENABLED
+
+static PayloadType * payload_type_h264_packetization_mode_1=NULL;
+static PayloadType * linphone_h263_1998=NULL;
+static PayloadType * linphone_mp4v_es=NULL;
+#endif
 
 void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, const char *config_path, void * userdata)
 {
@@ -624,9 +662,18 @@ void linphone_core_init (LinphoneCore * lc, const LinphoneCoreVTable *vtable, co
 	
 #ifdef VIDEO_ENABLED
 	rtp_profile_set_payload(&av_profile,97,&payload_type_theora);
-	rtp_profile_set_payload(&av_profile,98,&payload_type_h263_1998);
-	rtp_profile_set_payload(&av_profile,99,&payload_type_mp4v);
+
+	linphone_h263_1998=payload_type_clone(&payload_type_h263_1998);
+	payload_type_set_recv_fmtp(linphone_h263_1998,"CIF=1;QCIF=1");
+	rtp_profile_set_payload(&av_profile,98,linphone_h263_1998);
+
+	linphone_mp4v_es=payload_type_clone(&payload_type_mp4v);
+	payload_type_set_recv_fmtp(linphone_mp4v_es,"profile-level-id=3");
+	rtp_profile_set_payload(&av_profile,99,linphone_mp4v_es);
 	rtp_profile_set_payload(&av_profile,100,&payload_type_x_snow);
+	payload_type_h264_packetization_mode_1=payload_type_clone(&payload_type_h264);
+	payload_type_set_recv_fmtp(payload_type_h264_packetization_mode_1,"packetization-mode=1");
+	rtp_profile_set_payload(&av_profile,103,payload_type_h264_packetization_mode_1);
 	rtp_profile_set_payload(&av_profile,102,&payload_type_h264);
 #endif
 
@@ -692,35 +739,6 @@ int linphone_core_set_primary_contact(LinphoneCore *lc, const char *contact)
 	return 0;
 }
 
-static bool_t stun_get_localip(LinphoneCore *lc, char *result, int *port){
-	const char *server=linphone_core_get_stun_server(lc);
-	StunAddress4 addr;
-	StunAddress4 mapped;
-	StunAddress4 changed;
-	if (server!=NULL){
-		if (stunParseServerName((char*)server,&addr)){
-			if (lc->vtable.display_status!=NULL)
-				lc->vtable.display_status(lc,_("Stun lookup in progress..."));
-			if (stunTest(&addr,1,TRUE,NULL,&mapped,&changed)==0){
-				struct in_addr inaddr;
-				char *tmp;
-				inaddr.s_addr=ntohl(mapped.addr);
-				tmp=inet_ntoa(inaddr);
-				*port=ntohs(mapped.port);
-				strncpy(result,tmp,LINPHONE_IPADDR_SIZE);
-				if (lc->vtable.display_status!=NULL)
-					lc->vtable.display_status(lc,_("Stun lookup done..."));
-				ms_message("Stun server says we have address %s:i",result,*port);
-				return TRUE;
-			}else{
-				ms_warning("stun lookup failed.");
-			}
-		}else{
-			ms_warning("Fail to resolv or parse %s",server);
-		}
-	}
-	return FALSE;
-}
 
 /*result must be an array of chars at least LINPHONE_IPADDR_SIZE */
 void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result){
@@ -736,14 +754,17 @@ void linphone_core_get_local_ip(LinphoneCore *lc, const char *dest, char *result
 		if (lc->sip_conf.ipv6_enabled){
 			ms_warning("stun support is not implemented for ipv6");
 		}else{
+			/* we no more use stun for sip socket*/
+#if 0
 			int mport=0;
 			ms_message("doing stun lookup for local address...");
-			if (stun_get_localip(lc,result,&mport)){
+			if (stun_get_localip(lc,sock,linphone_core_get_sip_port(lc),result,&mport)){
 				if (!lc->net_conf.nat_sdp_only)
 					eXosip_masquerade_contact(result,mport);
 				return;
 			}
 			ms_warning("stun lookup failed, falling back to a local interface...");
+#endif
 		}
 		
 	}
@@ -833,7 +854,7 @@ int linphone_core_set_video_codecs(LinphoneCore *lc, MSList *codecs)
 	return 0;
 }
 
-MSList * linphone_core_get_friend_list(LinphoneCore *lc)
+const MSList * linphone_core_get_friend_list(LinphoneCore *lc)
 {
 	return lc->friends;
 }
@@ -852,6 +873,10 @@ int linphone_core_get_video_port(const LinphoneCore *lc){
 	return lc->rtp_conf.video_rtp_port;
 }
 
+int linphone_core_get_nortp_timeout(const LinphoneCore *lc){
+	return lc->rtp_conf.nortp_timeout;
+}
+
 void linphone_core_set_audio_jittcomp(LinphoneCore *lc, int value)
 {
 	lc->rtp_conf.audio_jitt_comp=value;
@@ -866,6 +891,10 @@ void linphone_core_set_video_port(LinphoneCore *lc, int port){
 	lc->rtp_conf.video_rtp_port=port;
 }
 
+void linphone_core_set_nortp_timeout(LinphoneCore *lc, int nortp_timeout){
+	lc->rtp_conf.nortp_timeout=nortp_timeout;
+}
+
 bool_t linphone_core_get_use_info_for_dtmf(LinphoneCore *lc)
 {
 	return lc->sip_conf.use_info;
@@ -943,9 +972,16 @@ static void display_bandwidth(RtpSession *as, RtpSession *vs){
 	(vs!=NULL) ? (rtp_session_compute_send_bandwidth(vs)*1e-3) : 0);
 }
 
+static void linphone_core_disconnected(LinphoneCore *lc){
+	lc->vtable.display_warning(lc,_("Remote end seems to have disconnected, the call is going to be closed."));
+	linphone_core_terminate_call(lc,NULL);
+}
+
 void linphone_core_iterate(LinphoneCore *lc)
 {
 	eXosip_event_t *ev;
+	bool_t disconnected=FALSE;
+	int disconnect_timeout = linphone_core_get_nortp_timeout(lc); 
 	if (lc->preview_finished){
 		lc->preview_finished=0;
 		ring_stop(lc->ringstream);
@@ -984,6 +1020,8 @@ void linphone_core_iterate(LinphoneCore *lc)
 			if (lc->videostream!=NULL)
 				video_stream_iterate(lc->videostream);
 #endif
+			if (lc->audiostream!=NULL && disconnect_timeout>0)
+				disconnected=!audio_stream_alive(lc->audiostream,disconnect_timeout);
 		}
 	}
 	if (linphone_core_video_preview_enabled(lc)){
@@ -996,6 +1034,8 @@ void linphone_core_iterate(LinphoneCore *lc)
 		if (lc->previewstream!=NULL)
 			toggle_video_preview(lc,FALSE);
 	}
+	if (disconnected)
+		linphone_core_disconnected(lc);
 }
 
 
@@ -1147,9 +1187,6 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 		gstate_new_state(lc, GSTATE_CALL_ERROR, NULL);
 		return -1;
 	}
-	barmsg=ortp_strdup_printf("%s %s", _("Contacting"), real_url);
-	lc->vtable.display_status(lc,barmsg);
-	ms_free(barmsg);
 	if (proxy!=NULL) {
 		from=linphone_proxy_config_get_identity(proxy);
 		
@@ -1171,6 +1208,9 @@ int linphone_core_invite(LinphoneCore *lc, const char *url)
 	osip_from_parse(parsed_url2,from);
 	
 	lc->call=linphone_call_new_outgoing(lc,parsed_url2,real_parsed_url);
+	barmsg=ortp_strdup_printf("%s %s", _("Contacting"), real_url);
+	lc->vtable.display_status(lc,barmsg);
+	ms_free(barmsg);
 	if (!lc->sip_conf.sdp_200_ack){
 		ctx=lc->call->sdpctx;
 		lc->call->profile=lc->local_profile;
@@ -1334,12 +1374,14 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				ms_warning("No card defined for capture !");
 				goto end;
 			}
-			
+			if (audio_params->relay_session_id!=NULL) 
+				audio_stream_set_relay_session_id(lc->audiostream,audio_params->relay_session_id);
 			audio_stream_start_now(
 				lc->audiostream,
 				call->profile,
 				audio_params->remoteaddr,
 				audio_params->remoteport,
+				audio_params->remotertcpport,
 				audio_params->pt,
 				jitt_comp,
 				playcard,
@@ -1351,6 +1393,7 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 				call->profile,
 				audio_params->remoteaddr,
 				audio_params->remoteport,
+				audio_params->remotertcpport,
 				audio_params->pt,
 				100,
 				lc->play_file,
@@ -1369,19 +1412,22 @@ void linphone_core_start_media_streams(LinphoneCore *lc, LinphoneCall *call){
 			StreamParams *video_params=&call->video_params;
 			
 			if (video_params->remoteport>0){
+				if (video_params->relay_session_id!=NULL) 
+					video_stream_set_relay_session_id(lc->videostream,video_params->relay_session_id);
 				if (lc->video_conf.display && lc->video_conf.capture)
 					video_stream_start(lc->videostream,
 					call->profile, video_params->remoteaddr, video_params->remoteport,
+					video_params->remotertcpport,
 					video_params->pt, jitt_comp, lc->video_conf.device);
 				else if (lc->video_conf.display)
 					video_stream_recv_only_start(lc->videostream,
 					call->profile, video_params->remoteaddr, video_params->remoteport,
-					video_params->pt, jitt_comp, lc->video_conf.device);
+					video_params->pt, jitt_comp);
 				else if (lc->video_conf.capture)
 					video_stream_send_only_start(lc->videostream,
-					call->profile, video_params->remoteaddr, video_params->remoteport, 
-					video_params->pt, jitt_comp, lc->video_conf.device);				
-
+					call->profile, video_params->remoteaddr, video_params->remoteport,
+					video_params->remotertcpport,
+					video_params->pt, jitt_comp, lc->video_conf.device);
 				video_stream_set_rtcp_information(lc->videostream, cname,tool);
 			}
 		}
@@ -1460,18 +1506,13 @@ int linphone_core_accept_call(LinphoneCore *lc, const char *url)
 			ms_error("fail to generate sdp offer !");
 			return -1;
 		}
-		ms_message("sdp message generated (sdpmesg=%p):\n%s",sdpmesg,sdpmesg);
 		linphone_set_sdp(msg,sdpmesg);
-		ms_message("sdp message attached to SIP answer");
 		linphone_core_init_media_streams(lc);
-		ms_message("init_media_streams done");
 	}else{
 		linphone_set_sdp(msg,sdpmesg);
 	}
 	eXosip_lock();
-	ms_message("eXosip_lock() done");
 	eXosip_call_send_answer(call->tid,200,msg);
-	ms_message("SIP answer sent.");
 	eXosip_unlock();
 	lc->vtable.display_status(lc,_("Connected."));
 	gstate_new_state(lc, GSTATE_CALL_IN_CONNECTED, NULL);
@@ -1505,6 +1546,10 @@ int linphone_core_terminate_call(LinphoneCore *lc, const char *url)
 	return 0;
 }
 
+bool_t linphone_core_in_call(const LinphoneCore *lc){
+	return lc->call!=NULL;
+}
+
 int linphone_core_send_publish(LinphoneCore *lc,
 			       LinphoneOnlineStatus presence_mode)
 {
@@ -1558,6 +1603,10 @@ void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,
 	
 }
 
+LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc){
+	return lc->presence_mode;
+}
+
 /* sound functions */
 int linphone_core_get_play_level(LinphoneCore *lc)
 {
@@ -1604,7 +1653,23 @@ static MSSndCard *get_card_from_string_id(const char *devid, unsigned int cap){
 			sndcard=NULL;
 		}
 	}
-	if (sndcard==NULL) sndcard=ms_snd_card_manager_get_default_card(ms_snd_card_manager_get());
+	if (sndcard==NULL) {
+		/* get a card that has read+write capabilities */
+		sndcard=ms_snd_card_manager_get_default_card(ms_snd_card_manager_get());
+		/* otherwise refine to the first card having the right capability*/
+		if (sndcard==NULL){
+			const MSList *elem=ms_snd_card_manager_get_list(ms_snd_card_manager_get());
+			for(;elem!=NULL;elem=elem->next){
+				sndcard=(MSSndCard*)elem->data;
+				if (ms_snd_card_get_capabilities(sndcard) & cap) break;
+			}
+		}
+		if (sndcard==NULL){/*looks like a bug! take the first one !*/
+			const MSList *elem=ms_snd_card_manager_get_list(ms_snd_card_manager_get());
+			sndcard=(MSSndCard*)elem->data;
+        }
+	}
+	if (sndcard==NULL) ms_error("Could not find a suitable soundcard !");
 	return sndcard;
 }
 
@@ -1643,6 +1708,10 @@ const char**  linphone_core_get_sound_devices(LinphoneCore *lc){
 	return lc->sound_conf.cards;
 }
 
+const char**  linphone_core_get_video_devices(const LinphoneCore *lc){
+	return lc->video_conf.cams;
+}
+
 char linphone_core_get_sound_source(LinphoneCore *lc)
 {
 	return lc->sound_conf.source;
@@ -1670,7 +1739,8 @@ void linphone_core_set_ring(LinphoneCore *lc,const char *path){
 	}
 	lc->sound_conf.local_ring=ms_strdup(path);
 }
-char *linphone_core_get_ring(LinphoneCore *lc){
+
+const char *linphone_core_get_ring(LinphoneCore *lc){
 	return lc->sound_conf.local_ring;
 }
 
@@ -1757,6 +1827,21 @@ const char * linphone_core_get_stun_server(const LinphoneCore *lc){
 	return lc->net_conf.stun_server;
 }
 
+const char * linphone_core_get_relay_addr(const LinphoneCore *lc){
+	return lc->net_conf.relay;
+}
+
+int linphone_core_set_relay_addr(LinphoneCore *lc, const char *addr){
+	if (lc->net_conf.relay!=NULL){
+		ms_free(lc->net_conf.relay);
+		lc->net_conf.relay=NULL;
+	}
+	if (addr){
+		lc->net_conf.relay=ms_strdup(addr);
+	}
+	return 0;
+}
+
 static void apply_nat_settings(LinphoneCore *lc){
 	char *wmsg;
 	char *tmp=NULL;
@@ -1887,21 +1972,35 @@ bool_t linphone_core_video_enabled(LinphoneCore *lc){
 	return (lc->video_conf.display || lc->video_conf.capture);
 }
 
+void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val){
+	lc->video_conf.show_local=val;
+}
 
 bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc){
 	return lc->video_conf.show_local;
 }
 
-int linphone_core_set_video_device(LinphoneCore *lc, const char *method, const char *device){
-	if (lc->video_conf.device!=NULL){
-		ms_free(lc->video_conf.device);
-		lc->video_conf.device=NULL;
+int linphone_core_set_video_device(LinphoneCore *lc, const char *id){
+	MSWebCam *olddev=lc->video_conf.device;
+	if (id!=NULL){
+		lc->video_conf.device=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),id);
+		if (lc->video_conf.device==NULL){
+			ms_warning("Could not found video device %s",id);
+		}
+	}
+	if (lc->video_conf.device==NULL)
+		lc->video_conf.device=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
+	if (olddev!=NULL && olddev!=lc->video_conf.device){
+		toggle_video_preview(lc,FALSE);/*restart the video local preview*/
 	}
-	if (device!=NULL)
-		lc->video_conf.device=ms_strdup(device);
 	return 0;
 }
 
+const char *linphone_core_get_video_device(const LinphoneCore *lc){
+	if (lc->video_conf.device) return ms_web_cam_get_string_id(lc->video_conf.device);
+	return NULL;
+}
+
 void linphone_core_use_files(LinphoneCore *lc, bool_t yesno){
 	lc->use_files=yesno;
 }
@@ -1935,6 +2034,22 @@ void *linphone_core_get_user_data(LinphoneCore *lc){
 	return lc->data;
 }
 
+int linphone_core_get_mtu(const LinphoneCore *lc){
+	return lc->net_conf.mtu;
+}
+
+void linphone_core_set_mtu(LinphoneCore *lc, int mtu){
+	lc->net_conf.mtu=mtu;
+	if (mtu>0){
+		if (mtu<500){
+			ms_error("MTU too small !");
+			mtu=500;
+		}
+		ms_set_mtu(mtu);
+		ms_message("MTU is supposed to be %i, rtp payload max size will be %i",mtu, ms_get_payload_max_size());
+	}else ms_set_mtu(0);//use mediastreamer2 default value
+}
+
 void net_config_uninit(LinphoneCore *lc)
 {
 	net_config_t *config=&lc->net_conf;
@@ -1946,6 +2061,7 @@ void net_config_uninit(LinphoneCore *lc)
 	if (config->nat_address!=NULL)
 		lp_config_set_string(lc->config,"net","nat_address",config->nat_address);
 	lp_config_set_int(lc->config,"net","firewall_policy",config->firewall_policy);
+	lp_config_set_int(lc->config,"net","mtu",config->mtu);
 }
 
 
@@ -2000,6 +2116,7 @@ void rtp_config_uninit(LinphoneCore *lc)
 	lp_config_set_int(lc->config,"rtp","video_rtp_port",config->video_rtp_port);
 	lp_config_set_int(lc->config,"rtp","audio_jitt_comp",config->audio_jitt_comp);
 	lp_config_set_int(lc->config,"rtp","video_jitt_comp",config->audio_jitt_comp);
+	lp_config_set_int(lc->config,"rtp","nortp_timeout",config->nortp_timeout);
 }
 
 void sound_config_uninit(LinphoneCore *lc)
@@ -2021,11 +2138,17 @@ void sound_config_uninit(LinphoneCore *lc)
 	lp_config_set_string(lc->config,"sound","local_ring",config->local_ring);
 	lp_config_set_string(lc->config,"sound","remote_ring",config->remote_ring);
 	lp_config_set_int(lc->config,"sound","echocancelation",config->ec);
+	if (config->local_ring) ms_free(config->local_ring);
 }
 
 void video_config_uninit(LinphoneCore *lc)
 {
 	video_config_t *config=&lc->video_conf;
+	const char *vd=linphone_core_get_video_device(lc);
+	if (vd && strstr(vd,"Static picture")!=NULL){
+		vd=NULL;
+	}
+	lp_config_set_string(lc->config,"video","device",vd);
 	lp_config_set_int(lc->config,"video","display",config->display);
 	lp_config_set_int(lc->config,"video","capture",config->capture);
 	lp_config_set_int(lc->config,"video","show_local",config->show_local);
@@ -2054,6 +2177,7 @@ void codecs_config_uninit(LinphoneCore *lc)
 		lp_config_set_string(lc->config,key,"mime",pt->mime_type);
 		lp_config_set_int(lc->config,key,"rate",pt->clock_rate);
 		lp_config_set_int(lc->config,key,"enabled",payload_type_enabled(pt));
+		lp_config_set_string(lc->config,key,"recv_fmtp",pt->recv_fmtp);
 		index++;
 	}
 }
@@ -2095,6 +2219,11 @@ void linphone_core_uninit(LinphoneCore *lc)
 	ui_config_uninit(lc);
 	lp_config_sync(lc->config);
 	lp_config_destroy(lc->config);
+
+#ifdef VIDEO_ENABLED
+	if (payload_type_h264_packetization_mode_1!=NULL)
+		payload_type_destroy(payload_type_h264_packetization_mode_1);
+#endif
 	
 	ortp_exit();
 	eXosip_quit();
