@@ -21,9 +21,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 
 #include <osipparser2/osip_message.h>
-#include <ortp/ortp.h>
-#include <ortp/payloadtype.h>
-#include <mediastreamer2/mscommon.h>
+#include "ortp/ortp.h"
+#include "ortp/payloadtype.h"
+#include "mediastreamer2/mscommon.h"
+
+#define LINPHONE_IPADDR_SIZE 64
+#define LINPHONE_HOSTNAME_SIZE 128
 
 #ifdef __cplusplus
 extern "C" {
@@ -33,7 +36,6 @@ struct _MSSndCard;
 struct _LinphoneCore;
 
 bool_t payload_type_enabled(struct _PayloadType *pt);
-bool_t payload_type_check_usable(struct _PayloadType *pt, const struct _LinphoneCore *lc);
 void payload_type_set_enable(struct _PayloadType *pt,int value);
 const char *payload_type_get_description(struct _PayloadType *pt);
 int payload_type_get_bitrate(PayloadType *pt);
@@ -56,6 +58,7 @@ typedef struct sip_config
 	bool_t loopback_only;
 	bool_t ipv6_enabled;
 	bool_t sdp_200_ack;
+	bool_t only_one_codec; /*in SDP answers*/
 } sip_config_t;
 
 typedef struct rtp_config
@@ -64,6 +67,7 @@ typedef struct rtp_config
 	int video_rtp_port;
 	int audio_jitt_comp;  /*jitter compensation*/
 	int video_jitt_comp;  /*jitter compensation*/
+	int nortp_timeout;
 }rtp_config_t;
 
 
@@ -72,9 +76,11 @@ typedef struct net_config
 {
 	char *nat_address;
 	char *stun_server;
+	char *relay;
 	int download_bw;
 	int upload_bw;
 	int firewall_policy;
+	int mtu;
 	bool_t nat_sdp_only;
 }net_config_t;
 
@@ -103,7 +109,8 @@ typedef struct codecs_config
 }codecs_config_t;
 
 typedef struct video_config{
-	char *device;
+	struct _MSWebCam *device;
+	const char **cams;
 	bool_t capture;
 	bool_t show_local;
 	bool_t display;
@@ -138,8 +145,12 @@ typedef struct _StreamParams
 	int line;
 	int localport;
 	int remoteport;
-	char *remoteaddr;
+	int remotertcpport;
 	int pt;
+	char *relay_session_id;
+	int natd_port;
+	char remoteaddr[LINPHONE_HOSTNAME_SIZE];
+	char natd_addr[LINPHONE_HOSTNAME_SIZE];
 } StreamParams;
 
 typedef enum _LCState{
@@ -204,6 +215,25 @@ typedef enum{
 	LinphoneSPAccept
 }LinphoneSubscribePolicy;
 
+typedef enum _LinphoneOnlineStatus{
+	LINPHONE_STATUS_UNKNOWN,
+	LINPHONE_STATUS_ONLINE,
+	LINPHONE_STATUS_BUSY,
+	LINPHONE_STATUS_BERIGHTBACK,
+	LINPHONE_STATUS_AWAY,
+	LINPHONE_STATUS_ONTHEPHONE,
+	LINPHONE_STATUS_OUTTOLUNCH,
+	LINPHONE_STATUS_NOT_DISTURB,
+	LINPHONE_STATUS_MOVED,
+	LINPHONE_STATUS_ALT_SERVICE,
+	LINPHONE_STATUS_OFFLINE,
+	LINPHONE_STATUS_PENDING,
+	LINPHONE_STATUS_CLOSED,
+	LINPHONE_STATUS_END
+}LinphoneOnlineStatus;
+
+const char *linphone_online_status_to_string(LinphoneOnlineStatus ss);
+
 typedef struct _LinphoneFriend{
 	osip_from_t *url;
 	int in_did;
@@ -211,6 +241,7 @@ typedef struct _LinphoneFriend{
 	int sid;
 	int nid;
 	LinphoneSubscribePolicy pol;
+	LinphoneOnlineStatus status;
 	struct _LinphoneProxyConfig *proxy;
 	struct _LinphoneCore *lc;
 	bool_t subscribe;
@@ -219,7 +250,8 @@ typedef struct _LinphoneFriend{
 
 LinphoneFriend * linphone_friend_new();
 LinphoneFriend *linphone_friend_new_with_addr(const char *addr);
-int linphone_friend_set_sip_addr(LinphoneFriend *fr, const char *name);
+int linphone_friend_set_sip_addr(LinphoneFriend *fr, const char *uri);
+int linphone_friend_set_name(LinphoneFriend *fr, const char *name);
 int linphone_friend_send_subscribe(LinphoneFriend *fr, bool_t val);
 int linphone_friend_set_inc_subscribe_policy(LinphoneFriend *fr, LinphoneSubscribePolicy pol);
 int linphone_friend_set_proxy(LinphoneFriend *fr, struct _LinphoneProxyConfig *cfg);
@@ -230,6 +262,10 @@ void linphone_friend_destroy(LinphoneFriend *lf);
 char *linphone_friend_get_name(LinphoneFriend *lf);
 char *linphone_friend_get_addr(LinphoneFriend *lf);
 char *linphone_friend_get_url(LinphoneFriend *lf);	/* name <sip address> */
+bool_t linphone_friend_get_send_subscribe(const LinphoneFriend *lf);
+LinphoneSubscribePolicy linphone_friend_get_inc_subscribe_policy(const LinphoneFriend *lf);
+LinphoneOnlineStatus linphone_friend_get_status(const LinphoneFriend *lf);
+
 #define linphone_friend_url(lf) ((lf)->url)
 
 void linphone_friend_write_to_config_file(struct _LpConfig *config, LinphoneFriend *lf, int index);
@@ -252,19 +288,22 @@ typedef struct _LinphoneProxyConfig
 	bool_t publish;
 } LinphoneProxyConfig;
 
-LinphoneProxyConfig *linphone_proxy_config_new(const char *server_addr);
+LinphoneProxyConfig *linphone_proxy_config_new(void);
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr);
 void linphone_proxy_config_set_identity(LinphoneProxyConfig *obj, const char *identity);
 void linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route);
 void linphone_proxy_config_expires(LinphoneProxyConfig *obj, const int expires);
-void linphone_proxy_config_enableregister(LinphoneProxyConfig *obj, bool_t val);
+void linphone_proxy_config_enable_register(LinphoneProxyConfig *obj, bool_t val);
+#define linphone_proxy_config_enableregister linphone_proxy_config_enable_register
 void linphone_proxy_config_edit(LinphoneProxyConfig *obj);
-void linphone_proxy_config_done(LinphoneProxyConfig *obj);
+int linphone_proxy_config_done(LinphoneProxyConfig *obj);
 void linphone_proxy_config_enable_publish(LinphoneProxyConfig *obj, bool_t val);
 #define linphone_proxy_config_get_route(obj)  ((obj)->reg_route)
 #define linphone_proxy_config_get_identity(obj)	((obj)->reg_identity)
 #define linphone_proxy_config_publish_enabled(obj) ((obj)->publish)
 #define linphone_proxy_config_get_addr(obj) ((obj)->reg_proxy)
+#define linphone_proxy_config_get_expires(obj)	((obj)->expires)
+#define linphone_proxy_config_register_enabled(obj) ((obj)->reg_sendregister)
 
 /* destruction is called automatically when removing the proxy config */
 void linphone_proxy_config_destroy(LinphoneProxyConfig *cfg);
@@ -283,8 +322,9 @@ typedef struct _LinphoneAuthInfo
 }LinphoneAuthInfo;
 
 LinphoneAuthInfo *linphone_auth_info_new(const char *username, const char *userid,
-				   										const char *passwd, const char *ha1,const char *realm);
+		const char *passwd, const char *ha1,const char *realm);
 void linphone_auth_info_set_passwd(LinphoneAuthInfo *info, const char *passwd);
+void linphone_auth_info_set_username(LinphoneAuthInfo *info, const char *username);
 /* you don't need those function*/
 void linphone_auth_info_destroy(LinphoneAuthInfo *info);
 LinphoneAuthInfo * linphone_auth_info_new_from_config_file(struct _LpConfig *config, int pos);
@@ -391,22 +431,7 @@ typedef struct _LCCallbackObj
   void * _user_data;
 }LCCallbackObj;
 
-typedef enum _LinphoneOnlineStatus{
-	LINPHONE_STATUS_UNKNOWN,
-	LINPHONE_STATUS_ONLINE,
-	LINPHONE_STATUS_BUSY,
-	LINPHONE_STATUS_BERIGHTBACK,
-	LINPHONE_STATUS_AWAY,
-	LINPHONE_STATUS_ONTHEPHONE,
-	LINPHONE_STATUS_OUTTOLUNCH,
-	LINPHONE_STATUS_NOT_DISTURB,
-	LINPHONE_STATUS_MOVED,
-	LINPHONE_STATUS_ALT_SERVICE,
-	LINPHONE_STATUS_OFFLINE,
-	LINPHONE_STATUS_PENDING,
-	LINPHONE_STATUS_CLOSED,
-	LINPHONE_STATUS_END
-}LinphoneOnlineStatus;
+
 
 typedef enum _LinphoneFirewallPolicy{
 	LINPHONE_POLICY_NO_FIREWALL,
@@ -457,6 +482,7 @@ typedef struct _LinphoneCore
 	int up_audio_bw;
 	int dw_video_bw;
 	int up_video_bw;
+	int audio_bw;
 	int automatic_action;
 	bool_t use_files;
 	bool_t apply_nat_settings;
@@ -467,7 +493,7 @@ typedef struct _LinphoneCore
 #endif
 } LinphoneCore;
 
-#define LINPHONE_IPADDR_SIZE 64
+
 
 /* THE main API */
 
@@ -492,6 +518,8 @@ int linphone_core_refer(LinphoneCore *lc, const char *url);
 
 bool_t linphone_core_inc_invite_pending(LinphoneCore*lc);
 
+bool_t linphone_core_in_call(const LinphoneCore *lc);
+
 int linphone_core_accept_call(LinphoneCore *lc, const char *url);
 
 int linphone_core_terminate_call(LinphoneCore *lc, const char *url);
@@ -534,9 +562,9 @@ const MSList *linphone_core_get_video_codecs(const LinphoneCore *lc);
 
 int linphone_core_set_video_codecs(LinphoneCore *lc, MSList *codecs);
 
-bool_t linphone_core_check_payload_type_usability(const LinphoneCore *lc, PayloadType *pt);
+bool_t linphone_core_check_payload_type_usability(LinphoneCore *lc, PayloadType *pt);
 
-void linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *config);
+int linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *config);
 
 void linphone_core_remove_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *config);
 
@@ -552,6 +580,8 @@ void linphone_core_add_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
 
 void linphone_core_remove_auth_info(LinphoneCore *lc, LinphoneAuthInfo *info);
 
+void linphone_core_abort_authentication(LinphoneCore *lc,  LinphoneAuthInfo *info);
+
 void linphone_core_clear_all_auth_info(LinphoneCore *lc);
 
 int linphone_core_get_audio_jittcomp(LinphoneCore *lc);
@@ -562,10 +592,14 @@ int linphone_core_get_audio_port(const LinphoneCore *lc);
 
 int linphone_core_get_video_port(const LinphoneCore *lc);
 
+int linphone_core_get_nortp_timeout(const LinphoneCore *lc);
+
 void linphone_core_set_audio_port(LinphoneCore *lc, int port);
 
 void linphone_core_set_video_port(LinphoneCore *lc, int port);
 
+void linphone_core_set_nortp_timeout(LinphoneCore *lc, int port);
+
 void linphone_core_set_use_info_for_dtmf(LinphoneCore *lc, bool_t use_info);
 
 bool_t linphone_core_get_use_info_for_dtmf(LinphoneCore *lc);
@@ -590,6 +624,10 @@ void linphone_core_set_firewall_policy(LinphoneCore *lc, LinphoneFirewallPolicy
 
 LinphoneFirewallPolicy linphone_core_get_firewall_policy(const LinphoneCore *lc);
 
+const char * linphone_core_get_relay_addr(const LinphoneCore *lc);
+
+int linphone_core_set_relay_addr(LinphoneCore *lc, const char *addr);
+
 /* sound functions */
 /* returns a null terminated static array of string describing the sound devices */ 
 const char**  linphone_core_get_sound_devices(LinphoneCore *lc);
@@ -608,7 +646,7 @@ int linphone_core_set_capture_device(LinphoneCore *lc, const char * devid);
 char linphone_core_get_sound_source(LinphoneCore *lc);
 void linphone_core_set_sound_source(LinphoneCore *lc, char source);
 void linphone_core_set_ring(LinphoneCore *lc, const char *path);
-char *linphone_core_get_ring(LinphoneCore *lc);
+const char *linphone_core_get_ring(LinphoneCore *lc);
 int linphone_core_preview_ring(LinphoneCore *lc, const char *ring,LinphoneCoreCbFunc func,void * userdata);
 typedef enum {RINGBACK_TYPE_FR,RINGBACK_TYPE_US} RingBackType;
 void linphone_core_set_ringback(LinphoneCore *lc,RingBackType type);
@@ -618,11 +656,13 @@ bool_t linphone_core_echo_cancelation_enabled(LinphoneCore *lc);
 
 void linphone_core_set_presence_info(LinphoneCore *lc,int minutes_away,const char *contact,LinphoneOnlineStatus os);
 
+LinphoneOnlineStatus linphone_core_get_presence_info(const LinphoneCore *lc);
+
 void linphone_core_add_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_remove_friend(LinphoneCore *lc, LinphoneFriend *fr);
 void linphone_core_reject_subscriber(LinphoneCore *lc, LinphoneFriend *lf);
 /* a list of LinphoneFriend */
-MSList * linphone_core_get_friend_list(LinphoneCore *lc);
+const MSList * linphone_core_get_friend_list(LinphoneCore *lc);
 /* notify all friends that have subscribed */
 void linphone_core_notify_all_friends(LinphoneCore *lc, LinphoneOnlineStatus os);
 
@@ -639,12 +679,19 @@ void linphone_core_set_play_file(LinphoneCore *lc, const char *file);
 void linphone_core_set_record_file(LinphoneCore *lc, const char *file);
 
 
-/*
-void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val);
-bool_t linphone_core_video_preview_enabled(LinphoneCore *lc);
-*/
 
-int linphone_core_set_video_device(LinphoneCore *lc, const char *method, const char *device);
+void linphone_core_enable_video_preview(LinphoneCore *lc, bool_t val);
+bool_t linphone_core_video_preview_enabled(const LinphoneCore *lc);
+
+
+
+/* returns a null terminated static array of string describing the webcams */ 
+const char**  linphone_core_get_video_devices(const LinphoneCore *lc);
+int linphone_core_set_video_device(LinphoneCore *lc, const char *id);
+const char *linphone_core_get_video_device(const LinphoneCore *lc);
+
+int linphone_core_get_mtu(const LinphoneCore *lc);
+void linphone_core_set_mtu(LinphoneCore *lc, int mtu);
 
 bool_t linphone_core_is_in_main_thread(LinphoneCore *lc);
 
@@ -654,6 +701,11 @@ void *linphone_core_get_user_data(LinphoneCore *lc);
 the config file with your own sections */
 struct _LpConfig *linphone_core_get_config(LinphoneCore *lc);
 
+/* attempts to wake up another linphone engine already running.
+The "show" callback is called for the other linphone, causing gui to show up.
+The method returns 0 if an already running linphone was found*/
+int linphone_core_wake_up_possible_already_running_instance(const char *config_file);
+
 void linphone_core_uninit(LinphoneCore *lc);
 void linphone_core_destroy(LinphoneCore *lc);
 
