@@ -33,7 +33,7 @@
 #endif
 
 #ifndef LINPHONE_VERSION
-#define LINPHONE_VERSION "2.1.0"
+#define LINPHONE_VERSION "3.0.0"
 #endif
 
 #ifndef LIBLINPHONE_VERSION 
@@ -54,8 +54,10 @@
 #define _(String) gettext(String)
 #endif
 #else
+#ifndef _
 #define _(something)	(something)
 #endif
+#endif
 
 void linphone_core_init_media_streams(LinphoneCore *lc);
 
@@ -98,7 +100,7 @@ static inline bool_t bandwidth_is_greater(int bw1, int bw2){
 }
 
 #define PAYLOAD_TYPE_ENABLED	PAYLOAD_TYPE_USER_FLAG_0
-
+void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer);
 void linphone_process_authentication(LinphoneCore* lc, eXosip_event_t *ev);
 void linphone_authentication_ok(LinphoneCore *lc, eXosip_event_t *ev);
 void linphone_subscription_new(LinphoneCore *lc, eXosip_event_t *ev);
@@ -116,4 +118,7 @@ MSList *find_friend(MSList *fl, const osip_from_t *friend, LinphoneFriend **lf);
 LinphoneFriend *linphone_find_friend_by_nid(MSList *l, int nid);
 LinphoneFriend *linphone_find_friend_by_sid(MSList *l, int sid);
 
+void linphone_core_update_allocated_audio_bandwidth(LinphoneCore *lc, const PayloadType *pt);
+void linphone_core_run_stun_tests(LinphoneCore *lc, LinphoneCall *call);
+
 #endif /* _PRIVATE_H */
