@@ -31,16 +31,10 @@ void linphone_proxy_config_init(LinphoneProxyConfig *obj){
 	obj->expires=3600;
 }
 
-LinphoneProxyConfig *linphone_proxy_config_new(const char *server_addr){
+LinphoneProxyConfig *linphone_proxy_config_new(){
 	LinphoneProxyConfig *obj=NULL;
-	int err;
 	obj=ms_new(LinphoneProxyConfig,1);
 	linphone_proxy_config_init(obj);
-	err=linphone_proxy_config_set_server_addr(obj,server_addr);
-	if (err<0){
-		linphone_proxy_config_destroy(obj);
-		obj=NULL;
-	}
 	return obj;
 }
 
@@ -58,22 +52,57 @@ static void linphone_proxy_config_register(LinphoneProxyConfig *obj){
 	eXosip_unlock();
 }
 
+void linphone_proxy_config_register_again_with_updated_contact(LinphoneProxyConfig *obj, osip_message_t *orig_request, osip_message_t *last_answer){
+	osip_message_t *msg;
+	const char *rport,*received;
+	osip_via_t *via=NULL;
+	osip_generic_param_t *param=NULL;
+	osip_contact_t *ctt=NULL;
+	osip_message_get_via(last_answer,0,&via);
+	if (!via) return;
+	osip_via_param_get_byname(via,"rport",&param);
+	if (param) rport=param->gvalue;
+	else return;
+	param=NULL;
+	osip_via_param_get_byname(via,"received",&param);
+	if (param) received=param->gvalue;
+	else return;
+	osip_message_get_contact(orig_request,0,&ctt);
+	if (strcmp(ctt->url->host,received)==0 && (ctt->url->port!=0 && strcmp(ctt->url->port,rport)==0)){
+		ms_message("Register has up to date contact, doing nothing.");
+		return;
+	}
+	eXosip_lock();
+	eXosip_register_build_register(obj->rid,obj->expires,&msg);
+	osip_message_get_contact(msg,0,&ctt);
+	if (ctt->url->host!=NULL){
+		osip_free(ctt->url->host);
+	}
+	ctt->url->host=osip_strdup(received);
+	if (ctt->url->port!=NULL){
+		osip_free(ctt->url->port);
+	}
+	ctt->url->port=osip_strdup(rport);
+	eXosip_register_send_register(obj->rid,msg);
+	eXosip_unlock();
+	ms_message("Resending new register with updated contact %s:%i",received,rport);
+}
 
 int linphone_proxy_config_set_server_addr(LinphoneProxyConfig *obj, const char *server_addr){
 	int err;
 	osip_from_t *url;
-	if (server_addr==NULL || strlen(server_addr)==0) return -1;
-	osip_from_init(&url);
-	err=osip_from_parse(url,server_addr);
-	if (err<0 || url->url->host==NULL){
-		ms_warning("Could not parse %s",server_addr);
+	if (obj->reg_proxy!=NULL) ms_free(obj->reg_proxy);
+	obj->reg_proxy=NULL;
+	if (server_addr!=NULL && strlen(server_addr)>0){
+		osip_from_init(&url);
+		err=osip_from_parse(url,server_addr);
+		if (err==0 && url->url->host!=NULL){
+			obj->reg_proxy=ms_strdup(server_addr);
+		}else{
+			ms_warning("Could not parse %s",server_addr);
+		}
 		osip_from_free(url);
-		return -1;
-	}else{
-		if (obj->reg_proxy!=NULL) ms_free(obj->reg_proxy);
-		obj->reg_proxy=ms_strdup(server_addr);
 	}
-	osip_from_free(url);
 	return 0;
 }
 
@@ -131,6 +160,22 @@ void linphone_proxy_config_set_route(LinphoneProxyConfig *obj, const char *route
 	}
 }
 
+bool_t linphone_proxy_config_check(LinphoneCore *lc, LinphoneProxyConfig *obj){
+	if (obj->reg_proxy==NULL){
+		if (lc->vtable.display_warning)
+			lc->vtable.display_warning(lc,_("The sip proxy address you entered is invalid, it must start with \"sip:\""
+						" followed by a hostname."));
+		return FALSE;
+	}
+	if (obj->reg_identity==NULL){
+		if (lc->vtable.display_warning)
+			lc->vtable.display_warning(lc,_("The sip identity you entered is invalid.\nIt should look like "
+					"sip:username@proxydomain, such as sip:alice@example.net"));
+		return FALSE;
+	}
+	return TRUE;
+}
+
 void linphone_proxy_config_enableregister(LinphoneProxyConfig *obj, bool_t val){
 	obj->reg_sendregister=val;
 }
@@ -165,9 +210,10 @@ void linphone_proxy_config_apply(LinphoneProxyConfig *obj,LinphoneCore *lc)
 	linphone_proxy_config_done(obj);
 }
 
-void linphone_proxy_config_done(LinphoneProxyConfig *obj)
+int linphone_proxy_config_done(LinphoneProxyConfig *obj)
 {
 	const char *id_str;
+	if (!linphone_proxy_config_check(obj->lc,obj)) return -1;
 	if (obj->reg_identity!=NULL) id_str=obj->reg_identity;
 	else id_str=linphone_core_get_primary_contact(obj->lc);
 	obj->frozen=FALSE;
@@ -178,6 +224,7 @@ void linphone_proxy_config_done(LinphoneProxyConfig *obj)
 		eXosip_register_send_register(obj->rid,msg);
 		if (ct!=NULL) osip_free(ct);
 	}
+	return 0;
 }
 
 void linphone_proxy_config_set_realm(LinphoneProxyConfig *cfg, const char *realm)
@@ -356,9 +403,11 @@ entity=\"%s\">\n%s",
   return 0;
 }
 
-void linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *cfg){
+int linphone_core_add_proxy_config(LinphoneCore *lc, LinphoneProxyConfig *cfg){
+	if (!linphone_proxy_config_check(lc,cfg)) return -1;
 	lc->sip_conf.proxies=ms_list_append(lc->sip_conf.proxies,(void *)cfg);
 	linphone_proxy_config_apply(cfg,lc);
+	return 0;
 }
 
 extern void linphone_friend_check_for_removed_proxy(LinphoneFriend *lf, LinphoneProxyConfig *cfg);
@@ -480,8 +529,8 @@ LinphoneProxyConfig *linphone_proxy_config_new_from_config_file(LpConfig *config
 	}
 	tmp=lp_config_get_string(config,key,"reg_proxy",NULL);
 	if (tmp==NULL) return NULL;
-	cfg=linphone_proxy_config_new(tmp);
-	if (cfg==NULL) return NULL;
+	cfg=linphone_proxy_config_new();
+	linphone_proxy_config_set_server_addr(cfg,tmp);
 		
 	tmp=lp_config_get_string(config,key,"reg_route",NULL);
 	if (tmp!=NULL) linphone_proxy_config_set_route(cfg,tmp);
