@@ -21,6 +21,7 @@
 #include <osipparser2/sdp_message.h>
 #include <eXosip2/eXosip.h>
 #include "linphonecore.h"
+#include "ortp/b64.h"
 
 #define keywordcmp(key,str)  strncmp(key,str,strlen(key))
 
@@ -36,6 +37,17 @@
 }while (0);
 
 
+static char *make_relay_session_id(const char *username, const char *relay){
+	/*ideally this should be a hash of the parameters with a random part*/
+	char tmp[128];
+	int s1=(int)random();
+	int s2=(int)random();
+	long long int res=((long long int)s1)<<32 | (long long int) s2;
+	void *src=&res;
+	b64_encode(src, sizeof(long long int), tmp, sizeof(tmp));
+	return osip_strdup(tmp);
+}
+
 char * int_2char(int a){
 	char *p=osip_malloc(16);
 	snprintf(p,16,"%i",a);
@@ -49,7 +61,7 @@ char *sdp_message_a_attr_value_get_with_pt(sdp_message_t *sdp,int pos,int pt,con
 	char *tmp;
 	sdp_attribute_t *attr;
 	for (i=0;(attr=sdp_message_attribute_get(sdp,pos,i))!=NULL;i++){
-		if (keywordcmp(field,attr->a_att_field)==0){
+		if (keywordcmp(field,attr->a_att_field)==0 && attr->a_att_value!=NULL){
 			int nb = sscanf(attr->a_att_value,"%i %n",&tmppt,&scanned);
 			/* the return value may depend on how %n is interpreted by the libc: see manpage*/
 			if (nb == 1 || nb==2 ){
@@ -64,6 +76,19 @@ char *sdp_message_a_attr_value_get_with_pt(sdp_message_t *sdp,int pos,int pt,con
 	return NULL;
 }
 
+/* return the value of attr "field" */
+char *sdp_message_a_attr_value_get(sdp_message_t *sdp,int pos,const char *field)
+{
+	int i;
+	sdp_attribute_t *attr;
+	for (i=0;(attr=sdp_message_attribute_get(sdp,pos,i))!=NULL;i++){
+		if (keywordcmp(field,attr->a_att_field)==0 && attr->a_att_value!=NULL){
+			return attr->a_att_value;
+		}
+	}
+	return NULL;
+}
+
 static int _sdp_message_get_a_ptime(sdp_message_t *sdp, int mline){
 	int i,ret;
 	sdp_attribute_t *attr;
@@ -86,12 +111,16 @@ sdp_payload_init (sdp_payload_t * payload)
 	return 0;
 }
 
-sdp_context_t *sdp_handler_create_context(sdp_handler_t *handler, const char *localip, const char *username){
+sdp_context_t *sdp_handler_create_context(sdp_handler_t *handler, const char *localip, const char *username, const char *relay){
 	sdp_context_t *ctx=osip_malloc(sizeof(sdp_context_t));
 	memset(ctx,0,sizeof(sdp_context_t));
 	if (localip!=NULL) ctx->localip=osip_strdup(localip);
 	ctx->username=osip_strdup(username);
 	ctx->handler=handler;
+	if (relay){
+		ctx->relay=osip_strdup(relay);
+		ctx->relay_session_id=make_relay_session_id(username,relay);
+	}
 	return ctx;
 }
 
@@ -144,6 +173,14 @@ sdp_context_generate_template (sdp_context_t * ctx)
 	return local;
 }
 
+static void add_relay_info(sdp_message_t *sdp, int mline, const char *relay, const char *relay_session_id){
+	
+	if (relay) sdp_message_a_attribute_add(sdp, mline,
+				     osip_strdup ("relay-addr"),osip_strdup(relay));
+	if (relay_session_id) sdp_message_a_attribute_add(sdp, mline,
+				     osip_strdup ("relay-session-id"), osip_strdup(relay_session_id));
+}
+
 /* to add payloads to the offer, must be called inside the write_offer callback */
 void
 sdp_context_add_payload (sdp_context_t * ctx, sdp_payload_t * payload, char *media)
@@ -166,6 +203,9 @@ sdp_context_add_payload (sdp_context_t * ctx, sdp_payload_t * payload, char *med
 		sdp_message_m_media_add (offer, osip_strdup (media),
 				 int_2char (payload->localport), NULL,
 				 osip_strdup (payload->proto));
+		if (ctx->relay){
+			add_relay_info(offer,payload->line,ctx->relay,ctx->relay_session_id);
+		}
 	}
 	sdp_message_m_payload_add (offer, payload->line, int_2char (payload->pt));
 	if (payload->a_rtpmap != NULL)
@@ -240,16 +280,39 @@ static void refuse_mline(sdp_message_t *answer,char *mtype,char *proto, int mlin
 	sdp_message_m_payload_add(answer,mline,int_2char(0));
 }
 
+static char * parse_relay_addr(char *addr, int *port)
+{
+	char *semicolon=NULL;
+	char *p;
+
+	*port=56789;
+	semicolon=strchr(addr,':');
+	for (p=addr+strlen(addr)-1;p>addr;p--){
+		if (*p==':') {
+			semicolon=p;
+			break;
+		}
+	}
+	if (semicolon){
+		*port=atoi(semicolon+1);
+		*semicolon='\0';
+	}
+	return addr;
+}
+
+
 char *
 sdp_context_get_answer ( sdp_context_t *ctx,sdp_message_t *remote)
 {
 	sdp_message_t *answer=NULL;
-	char *mtype=NULL, *tmp=NULL, *bw=NULL;
+	char *mtype=NULL, *tmp=NULL;
 	char *proto=NULL, *port=NULL, *pt=NULL;
 	int i, j, ncodec, m_lines_accepted = 0;
 	int err;
 	sdp_payload_t payload;
 	sdp_handler_t *sdph=ctx->handler;
+	sdp_bandwidth_t *sbw=NULL;
+	char *relay;
 
 	tmp = sdp_message_c_addr_get (remote, 0, 0);
 	if (tmp == NULL)
@@ -276,9 +339,16 @@ sdp_context_get_answer ( sdp_context_t *ctx,sdp_message_t *remote)
 		payload.c_addr = sdp_message_c_addr_get (remote, i, 0);
 		if (payload.c_addr == NULL)
 			payload.c_addr = sdp_message_c_addr_get (remote, -1, 0);
+		/*parse relay address if given*/
+		relay=sdp_message_a_attr_value_get(remote,i,"relay-addr");
+		if (relay){
+			payload.relay_host=parse_relay_addr(relay,&payload.relay_port);
+		}
+		payload.relay_session_id=sdp_message_a_attr_value_get(remote,i,"relay-session-id");
 		/* get application specific bandwidth, if any */
-		bw=sdp_message_b_bandwidth_get(remote, i, 0);
-		if (bw!=NULL) payload.b_as_bandwidth=atoi(bw);
+		for(j=0;(sbw=sdp_message_bandwidth_get(remote,i,j))!=NULL;j++){
+			if (strcasecmp(sbw->b_bwtype,"AS")==0) payload.b_as_bandwidth=atoi(sbw->b_bandwidth);
+		}
 		payload.a_ptime=_sdp_message_get_a_ptime(remote,i);
 		if (keywordcmp ("audio", mtype) == 0)
 		{
@@ -323,6 +393,10 @@ sdp_context_get_answer ( sdp_context_t *ctx,sdp_message_t *remote)
 								 NULL,
 								 osip_strdup
 								 (proto));
+							/* and accept the remote relay addr if we planned to use our own */
+							if (ctx->relay!=NULL && relay){
+								add_relay_info(answer,i,relay,payload.relay_session_id);
+							}
 						}
 						/* add the payload, rtpmap, fmtp */
 						sdp_message_m_payload_add (answer, i,
@@ -424,6 +498,10 @@ sdp_context_get_answer ( sdp_context_t *ctx,sdp_message_t *remote)
 								 (payload.localport), NULL,
 								 osip_strdup
 								 (proto));
+							/* and accept the remote relay addr if we planned to use our own */
+							if (ctx->relay!=NULL && relay){
+								add_relay_info(answer,i,relay,payload.relay_session_id);
+							}
 						}
 						/* add the payload, rtpmap, fmtp */
 						sdp_message_m_payload_add (answer, i,
@@ -503,11 +581,12 @@ void
 sdp_context_read_answer (sdp_context_t *ctx, sdp_message_t *remote)
 {
 	char *mtype;
-	char *proto, *port, *pt, *bw;
+	char *proto, *port, *pt;
 	int i, j,err;
-	sdp_payload_t payload;
+	char *relay;
+	sdp_payload_t payload,arg_payload;
 	sdp_handler_t *sdph=ctx->handler;
-
+	sdp_bandwidth_t *sbw=NULL;
 	/* for each m= line */
 	for (i = 0; !sdp_message_endof_media (remote, i); i++)
 	{
@@ -522,8 +601,15 @@ sdp_context_read_answer (sdp_context_t *ctx, sdp_message_t *remote)
 		payload.c_addr = sdp_message_c_addr_get (remote, i, 0);
 		if (payload.c_addr == NULL)
 			payload.c_addr = sdp_message_c_addr_get (remote, -1, 0);
-		bw=sdp_message_b_bandwidth_get(remote, i, 0);
-		if (bw!=NULL) payload.b_as_bandwidth=atoi(bw);
+		/*parse relay address if given*/
+		relay=sdp_message_a_attr_value_get(remote,i,"relay-addr");
+		if (relay){
+			payload.relay_host=parse_relay_addr(relay,&payload.relay_port);
+		}
+		payload.relay_session_id=sdp_message_a_attr_value_get(remote,i,"relay-session-id");
+		for(j=0;(sbw=sdp_message_bandwidth_get(remote,i,j))!=NULL;++j){
+			if (strcasecmp(sbw->b_bwtype,"AS")==0) payload.b_as_bandwidth=atoi(sbw->b_bandwidth);
+		}
 		payload.a_ptime=_sdp_message_get_a_ptime(remote,i);
 		if (keywordcmp ("audio", mtype) == 0)
 		{
@@ -547,8 +633,9 @@ sdp_context_read_answer (sdp_context_t *ctx, sdp_message_t *remote)
 						(remote, i, payload.pt,
 						 "fmtp");
 					/* ask the application if this codec is supported */
+					memcpy(&arg_payload,&payload,sizeof(payload));
 					err = sdph->get_audio_codecs (ctx,
-								      &payload);
+								      &arg_payload);
 				}
 			}
 		}
@@ -574,8 +661,9 @@ sdp_context_read_answer (sdp_context_t *ctx, sdp_message_t *remote)
 						(remote, i, payload.pt,
 						 "fmtp");
 					/* ask the application if this codec is supported */
+					memcpy(&arg_payload,&payload,sizeof(payload));
 					err = sdph->get_video_codecs (ctx,
-									 &payload);
+									 &arg_payload);
 				}
 			}
 		}
@@ -588,5 +676,7 @@ void sdp_context_free(sdp_context_t *ctx){
 	if (ctx->answer!=NULL) sdp_message_free(ctx->answer);
 	if (ctx->offerstr!=NULL) osip_free(ctx->offerstr);
 	if (ctx->answerstr!=NULL) osip_free(ctx->answerstr);
+	if (ctx->relay!=NULL) osip_free(ctx->relay);
+	if (ctx->relay_session_id!=NULL) osip_free(ctx->relay_session_id);
 	osip_free(ctx);
 }
