@@ -24,7 +24,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "msfilter.h"
 #include "msticker.h"
 #include "mssndcard.h"
+#include "mswebcam.h"
 #include "ortp/ortp.h"
+#include "ortp/event.h"
 
 struct _AudioStream
 {
@@ -38,6 +40,8 @@ struct _AudioStream
 	MSFilter *rtpsend;
 	MSFilter *dtmfgen;
 	MSFilter *ec;/*echo canceler*/
+	unsigned int last_packet_count;
+	time_t last_packet_time;
 };
 
 #ifdef __cplusplus
@@ -64,7 +68,7 @@ AudioStream *audio_stream_start (RtpProfile * prof, int locport, const char *rem
 AudioStream *audio_stream_start_with_sndcards(RtpProfile * prof, int locport, const char *remip4, int remport, int payload_type, int jitt_comp, MSSndCard *playcard, MSSndCard *captcard, bool_t echocancel);
 
 int audio_stream_start_with_files (AudioStream * stream, RtpProfile * prof,
-					    const char *remip, int remport,
+					    const char *remip, int remport, int rem_rtcp_port,
 					    int pt, int jitt_comp,
 					    const char * infile,  const char * outfile);
 
@@ -77,8 +81,10 @@ void audio_stream_set_rtcp_information(AudioStream *st, const char *cname, const
 this is useful to make sure that sockets are open before sending an invite; 
 or to start to stream only after receiving an ack.*/
 AudioStream *audio_stream_new(int locport, bool_t ipv6);
-int audio_stream_start_now(AudioStream * stream, RtpProfile * prof,  const char *remip, int remport, int payload_type, int jitt_comp,MSSndCard *playcard, MSSndCard *captcard, bool_t echo_cancel);
-
+int audio_stream_start_now(AudioStream * stream, RtpProfile * prof,  const char *remip, int remport, int rem_rtcp_port, int payload_type, int jitt_comp,MSSndCard *playcard, MSSndCard *captcard, bool_t echo_cancel);
+void audio_stream_set_relay_session_id(AudioStream *stream, const char *relay_session_id);
+/*returns true if we are still receiving some data from remote end in the last timeout seconds*/
+bool_t audio_stream_alive(AudioStream * stream, int timeout);
 
 /* stop the above process*/
 void audio_stream_stop (AudioStream * stream);
@@ -106,29 +112,36 @@ struct _VideoStream
 	MSFilter *source;
 	MSFilter *predec;
 	MSFilter *pixconv;
+	MSFilter *sizeconv;
 	MSFilter *tee;
 	MSFilter *output;
 	MSFilter *encoder;
 	MSFilter *decoder;
 	MSFilter *rtprecv;
 	MSFilter *rtpsend;
+	OrtpEvQueue *evq;
+	bool_t adapt_bitrate;
 };
 
 
 typedef struct _VideoStream VideoStream;
 VideoStream *video_stream_new(int locport, bool_t use_ipv6);
-int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp, const char *device);
+void video_stream_enable_adaptive_bitrate_control(VideoStream *s, bool_t yesno);
+int video_stream_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int rem_rtcp_port,
+		int payload, int jitt_comp, MSWebCam *device);
+void video_stream_set_relay_session_id(VideoStream *stream, const char *relay_session_id);
 void video_stream_set_rtcp_information(VideoStream *st, const char *cname, const char *tool);
 /*function to call periodically to handle various events */
 void video_stream_iterate(VideoStream *stream);
 void video_stream_send_vfu(VideoStream *stream);
 void video_stream_stop (VideoStream * stream);
 
-VideoStream * video_preview_start(const char *device);
+VideoStream * video_preview_start(MSWebCam *device);
 void video_preview_stop(VideoStream *stream);
 
-int video_stream_recv_only_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp, const char *device);
-int video_stream_send_only_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp, const char *device);
+int video_stream_recv_only_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp);
+int video_stream_send_only_start(VideoStream * stream, RtpProfile *profile, const char *remip, int remport,
+		int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *device);
 void video_stream_recv_only_stop(VideoStream *stream);
 void video_stream_send_only_stop(VideoStream *stream);
 
