@@ -114,7 +114,8 @@ void * ms_list_nth_data(const MSList *list, int index);
 int ms_list_position(const MSList *list, MSList *elem);
 int ms_list_index(const MSList *list, void *data);
 MSList *ms_list_insert_sorted(MSList *list, void *data, int (*compare_func)(const void *, const void*));
-
+MSList *ms_list_insert(MSList *list, MSList *before, void *data);
+MSList *ms_list_copy(const MSList *list);
 
 #undef MIN
 #define MIN(a,b)	((a)>(b) ? (b) : (a))
@@ -183,11 +184,23 @@ void ms_sleep(int seconds);
  * value works for both.
  * 
 **/
-
 int ms_get_payload_max_size();
 
 void ms_set_payload_max_size(int size);
 
+/**
+ * Returns the network Max Transmission Unit to reach destination_host.
+ * This will attempt to send one or more big packets to destination_host, to a random port.
+ * Those packets are filled with zeroes.
+**/
+int ms_discover_mtu(const char *destination_host);
+
+/**
+ * Set mediastreamer default mtu, used to compute the default RTP max payload size.
+ * This function will call ms_set_payload_max_size(mtu-[ipv6 header size]).
+**/
+void ms_set_mtu(int mtu);
+
 /** @} */
 
 #ifdef __cplusplus
