@@ -30,17 +30,26 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define MS_VIDEO_SIZE_4CIF_W 704
 #define MS_VIDEO_SIZE_4CIF_H 576
 
+#define MS_VIDEO_SIZE_QQVGA_W 160
+#define MS_VIDEO_SIZE_QQVGA_H 120
+
 #define MS_VIDEO_SIZE_QVGA_W 320
 #define MS_VIDEO_SIZE_QVGA_H 240
+
 #define MS_VIDEO_SIZE_VGA_W 640
 #define MS_VIDEO_SIZE_VGA_H 480
 
-#define MS_VIDEO_SIZE_MAX_W MS_VIDEO_SIZE_4CIF_W
-#define MS_VIDEO_SIZE_MAX_H MS_VIDEO_SIZE_4CIF_H
+#define MS_VIDEO_SIZE_720P_W 1280
+#define MS_VIDEO_SIZE_720P_H 720
 
 #define MS_VIDEO_SIZE_NS1_W 324
 #define MS_VIDEO_SIZE_NS1_H 248
 
+#define MS_VIDEO_SIZE_1024_W 1024
+#define MS_VIDEO_SIZE_1024_H 768
+
+#define MS_VIDEO_SIZE_MAX_W MS_VIDEO_SIZE_1024_W
+#define MS_VIDEO_SIZE_MAX_H MS_VIDEO_SIZE_1024_H
 
 typedef struct MSVideoSize{
 	int width,height;
@@ -54,18 +63,24 @@ typedef struct MSRect{
 #define MS_VIDEO_SIZE_QCIF (MSVideoSize){MS_VIDEO_SIZE_QCIF_W,MS_VIDEO_SIZE_QCIF_H}
 #define MS_VIDEO_SIZE_4CIF (MSVideoSize){MS_VIDEO_SIZE_4CIF_W,MS_VIDEO_SIZE_4CIF_H}
 
+#define MS_VIDEO_SIZE_QQVGA (MSVideoSize){MS_VIDEO_SIZE_QQVGA_W,MS_VIDEO_SIZE_QQVGA_H}
 #define MS_VIDEO_SIZE_QVGA (MSVideoSize){MS_VIDEO_SIZE_QVGA_W,MS_VIDEO_SIZE_QVGA_H}
 #define MS_VIDEO_SIZE_VGA (MSVideoSize){MS_VIDEO_SIZE_VGA_W,MS_VIDEO_SIZE_VGA_H}
 
+#define MS_VIDEO_SIZE_720P (MSVideoSize){MS_VIDEO_SIZE_720P_W, MS_VIDEO_SIZE_720P_H}
+
 #define MS_VIDEO_SIZE_NS1 (MSVideoSize){MS_VIDEO_SIZE_NS1_W,MS_VIDEO_SIZE_NS1_H}
 
+#define MS_VIDEO_SIZE_1024 (MSVideoSize){MS_VIDEO_SIZE_1024_W, MS_VIDEO_SIZE_1024_H}
+
 typedef enum{
 	MS_YUV420P,
 	MS_YUYV,
 	MS_RGB24,
 	MS_MJPEG,
 	MS_UYVY,
-	MS_YUY2   /* -> same as MS_YUYV */
+	MS_YUY2,   /* -> same as MS_YUYV */
+	MS_PIX_FMT_UNKNOWN
 }MSPixFmt;
 
 typedef struct _MSPicture{
@@ -82,12 +97,17 @@ extern "C"{
 
 int ms_pix_fmt_to_ffmpeg(MSPixFmt fmt);
 MSPixFmt ffmpeg_pix_fmt_to_ms(int fmt);
+MSPixFmt ms_fourcc_to_pix_fmt(uint32_t fourcc);
 void ms_ffmpeg_check_init(void);
 int yuv_buf_init_from_mblk(MSPicture *buf, mblk_t *m);
 void yuv_buf_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, int w, int h);
 mblk_t * yuv_buf_alloc(MSPicture *buf, int w, int h);
 void yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
 		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi);
+void yuv_buf_mirror(YuvBuf *buf);
+void rgb24_revert(uint8_t *buf, int w, int h, int linesize);
+void rgb24_copy_revert(uint8_t *dstbuf, int dstlsz,
+				const uint8_t *srcbuf, int srclsz, MSVideoSize roi);
 #ifdef __cplusplus
 }
 #endif
