@@ -25,6 +25,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/mssndcard.h"
 
+//#define THREADED_VERSION
+
 /*in case of troubles with a particular driver, try incrementing ALSA_PERIOD_SIZE
 to 512, 1024, 2048, 4096...
 then try incrementing the number of periods*/
@@ -67,25 +69,25 @@ static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo,
 	
 	/* Init hwparams with full configuration space */
 	if (snd_pcm_hw_params_any(pcm_handle, hwparams) < 0) {
-		ms_warning("alsa_set_params: Cannot configure this PCM device.\n");
+		ms_warning("alsa_set_params: Cannot configure this PCM device.");
 		return -1;
 	}
 	
 	if (snd_pcm_hw_params_set_access(pcm_handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED) < 0) {
-		ms_warning("alsa_set_params: Error setting access.\n");
+		ms_warning("alsa_set_params: Error setting access.");
 		return -1;
 	}
 	/* Set sample format */
 	format=SND_PCM_FORMAT_S16;
 	if (snd_pcm_hw_params_set_format(pcm_handle, hwparams, format) < 0) {
-		ms_warning("alsa_set_params: Error setting format.\n");
+		ms_warning("alsa_set_params: Error setting format.");
 		return -1;
 	}
 	/* Set number of channels */
 	if (stereo) channels=2;
 	else channels=1;
 	if (snd_pcm_hw_params_set_channels(pcm_handle, hwparams, channels) < 0) {
-		ms_warning("alsa_set_params: Error setting channels.\n");
+		ms_warning("alsa_set_params: Error setting channels.");
 		return -1;
 	}
 	/* Set sample rate. If the exact rate is not supported */
@@ -98,7 +100,7 @@ static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo,
 	}
 	if (dir != 0) {
 		ms_warning("alsa_set_params: The rate %d Hz is not supported by your hardware.\n "
-		"==> Using %d Hz instead.\n", rate, exact_uvalue);
+		"==> Using %d Hz instead.", rate, exact_uvalue);
 	}
 	/* choose greater period size when rate is high */
 	periodsize=periodsize*(rate/8000);	
@@ -109,24 +111,26 @@ static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo,
 	exact_ulvalue=periodsize;
 	dir=0;
 	if (snd_pcm_hw_params_set_period_size_near(pcm_handle, hwparams, &exact_ulvalue, &dir) < 0) {
-		ms_warning("alsa_set_params: Error setting period size.\n");
+		ms_warning("alsa_set_params: Error setting period size.");
 		return -1;
 	}
 	if (dir != 0) {
 		ms_warning("alsa_set_params: The period size %d is not supported by your hardware.\n "
-		"==> Using %d instead.\n", periodsize, (int)exact_ulvalue);
+		"==> Using %d instead.", periodsize, (int)exact_ulvalue);
 	}
+	ms_warning("alsa_set_params: periodsize:%d Using %d", periodsize, (int)exact_ulvalue);
 	periodsize=exact_ulvalue;
 	/* Set number of periods. Periods used to be called fragments. */ 
 	exact_uvalue=periods;
 	dir=0;
 	if (snd_pcm_hw_params_set_periods_near(pcm_handle, hwparams, &exact_uvalue, &dir) < 0) {
-		ms_warning("alsa_set_params: Error setting periods.\n");
+		ms_warning("alsa_set_params: Error setting periods.");
 		return -1;
 	}
+	ms_warning("alsa_set_params: period:%d Using %d", periods, exact_uvalue);
 	if (dir != 0) {
 		ms_warning("alsa_set_params: The number of periods %d is not supported by your hardware.\n "
-		"==> Using %d instead.\n", periods, exact_uvalue);
+		"==> Using %d instead.", periods, exact_uvalue);
 	}
 	/* Apply HW parameter settings to */
 	/* PCM device and prepare device  */
@@ -142,7 +146,7 @@ static int alsa_set_params(snd_pcm_t *pcm_handle, int rw, int bits, int stereo,
 			ms_warning("alsa_set_params: Error setting start threshold:%s",snd_strerror(err));
 		}
 		if ((err=snd_pcm_sw_params_set_stop_threshold(pcm_handle, swparams,periodsize*periods ))<0){
-			ms_warning("alsa_set_params: Error setting start threshold:%s",snd_strerror(err));
+			ms_warning("alsa_set_params: Error setting stop threshold:%s",snd_strerror(err));
 		}
 		if ((err=snd_pcm_sw_params(pcm_handle, swparams))<0){
 			ms_warning("alsa_set_params: Error setting SW params:%s",snd_strerror(err));
@@ -192,10 +196,18 @@ static snd_pcm_t * alsa_open_r(const char *pcmdev,int bits,int stereo,int rate)
 	ms_message("alsa_open_r: opening %s at %iHz, bits=%i, stereo=%i",pcmdev,rate,bits,stereo);
 
 
+#ifndef THREADED_VERSION
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_CAPTURE,SND_PCM_NONBLOCK) < 0) {
-		ms_warning("alsa_open_r: Error opening PCM device %s\n",pcmdev );
+		ms_warning("alsa_open_r: Error opening PCM device %s",pcmdev );
 		return NULL;
 	}
+#else
+	/* want blocking mode for threaded version */
+	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_CAPTURE,0) < 0) {
+		ms_warning("alsa_open_r: Error opening PCM device %s",pcmdev );
+		return NULL;
+	}
+#endif
 	if (alsa_set_params(pcm_handle,0,bits,stereo,rate)<0){
 		snd_pcm_close(pcm_handle);
 		return NULL;
@@ -212,7 +224,7 @@ static snd_pcm_t * alsa_open_w(const char *pcmdev,int bits,int stereo,int rate)
 	snd_pcm_t *pcm_handle;
 	
 	if (snd_pcm_open(&pcm_handle, pcmdev,SND_PCM_STREAM_PLAYBACK,SND_PCM_NONBLOCK) < 0) {
-		ms_warning("alsa_open_w: Error opening PCM device %s\n",pcmdev );
+		ms_warning("alsa_open_w: Error opening PCM device %s",pcmdev );
 		return NULL;
 	}
 	if (alsa_set_params(pcm_handle,1,bits,stereo,rate)<0){
@@ -222,7 +234,7 @@ static snd_pcm_t * alsa_open_w(const char *pcmdev,int bits,int stereo,int rate)
 	return pcm_handle;
 }
 
-static bool_t alsa_can_read(snd_pcm_t *dev, int frames)
+static int alsa_can_read(snd_pcm_t *dev, int frames)
 {
 	snd_pcm_sframes_t avail;
 	int err;
@@ -230,7 +242,7 @@ static bool_t alsa_can_read(snd_pcm_t *dev, int frames)
 	avail = snd_pcm_avail_update(dev);
 	ms_debug("*** %s %d %d", __FUNCTION__, (long)avail, frames);
 	if (avail < 0) {
-		ms_error("snd_pcm_avail_update: %s\n", snd_strerror(avail));	// most probably -EPIPE
+		ms_error("snd_pcm_avail_update: %s", snd_strerror(avail));	// most probably -EPIPE
 		/* overrun occured, snd_pcm_state() would return SND_PCM_STATE_XRUN
 		 FIXME: handle other error conditions*/
 		ms_error("*** alsa_can_read fixup, trying to recover");
@@ -238,16 +250,16 @@ static bool_t alsa_can_read(snd_pcm_t *dev, int frames)
 		err = snd_pcm_recover(dev, avail, 0);
 		if (err){ 
 			ms_error("snd_pcm_recover() failed with err %d: %s", err, snd_strerror(err));
-			return FALSE;
+			return -1;
 		}
 		err = snd_pcm_start(dev);
 		if (err){ 
-			ms_error("snd_pcm_start() failed with err %d: %s\n", err, snd_strerror(err)); 
-			return FALSE; 
+			ms_error("snd_pcm_start() failed with err %d: %s", err, snd_strerror(err)); 
+			return -1; 
 		}
-		ms_message("Recovery done\n");
+		ms_message("Recovery done");
 	}
-	return avail >= frames;
+	return avail;
 }
 
 static int alsa_read(snd_pcm_t *handle,unsigned char *buf,int nsamples)
@@ -255,6 +267,7 @@ static int alsa_read(snd_pcm_t *handle,unsigned char *buf,int nsamples)
 	int err;
 	err=snd_pcm_readi(handle,buf,nsamples);
 	if (err<0) {
+		ms_warning("alsa_read: snd_pcm_readi() returned %i",err);
 		if (err==-EPIPE){
 			snd_pcm_prepare(handle);
 			err=snd_pcm_readi(handle,buf,nsamples);
@@ -516,6 +529,7 @@ static void alsa_card_uninit(MSSndCard *obj){
 	AlsaData *ad=(AlsaData*)obj->data;
 	if (ad->pcmdev!=NULL) ms_free(ad->pcmdev);
 	if (ad->mixdev!=NULL) ms_free(ad->mixdev);
+	ms_free(ad);
 }
 
 static void alsa_card_detect(MSSndCardManager *m){
@@ -601,6 +615,14 @@ struct _AlsaReadData{
 	snd_pcm_t *handle;
 	int rate;
 	int nchannels;
+
+#ifdef THREADED_VERSION
+	ms_thread_t thread;
+	ms_mutex_t mutex;
+	MSBufferizer * bufferizer;
+	bool_t read_started;
+	bool_t write_started;
+#endif
 };
 
 typedef struct _AlsaReadData AlsaReadData;
@@ -612,35 +634,142 @@ void alsa_read_init(MSFilter *obj){
 	ad->rate=8000;
 	ad->nchannels=1;
 	obj->data=ad;
+
+#ifdef THREADED_VERSION
+	ad->read_started=FALSE;
+	ad->write_started=FALSE;
+	ad->bufferizer=ms_bufferizer_new();
+	ms_mutex_init(&ad->mutex,NULL);
+	ad->thread=0;
+#endif
 }
 
+#ifdef THREADED_VERSION
+
+static void * alsa_write_thread(void *p){
+	AlsaReadData *ad=(AlsaReadData*)p;
+	int samples=(160*ad->rate)/8000;
+	int err;
+	int count=0;
+	mblk_t *om=NULL;
+	struct timeval timeout;
+	if (ad->handle==NULL && ad->pcmdev!=NULL){
+		ad->handle=alsa_open_r(ad->pcmdev,16,ad->nchannels==2,ad->rate);
+	}
+	if (ad->handle==NULL) return NULL;
+
+	while (ad->read_started)
+	  {
+	    count = alsa_can_read(ad->handle,samples);
+	    if (count==24)
+	      { /* keep this value for this driver */ }
+	    else if (count<=0)
+	      {
+		count = samples;
+	      }
+	    else if (count>0)
+	      {
+		//ms_warning("%i count", count);
+		//count = samples;
+	      }
+
+	    int size=count*2;
+	    om=allocb(size,0);
+
+	    if ((err=alsa_read(ad->handle,om->b_wptr,count))<=0)
+	      {
+		ms_warning("nothing to read");
+		//ms_warning("Fail to read samples %i", count);
+		continue;
+	      }
+	    //ms_warning(" read %i", err);
+	    
+	    size=err*2;
+	    om->b_wptr+=size;
+
+	    ms_mutex_lock(&ad->mutex);
+	    ms_bufferizer_put(ad->bufferizer,om);
+	    ms_mutex_unlock(&ad->mutex);
+
+	    if (count==24)
+	      {
+		timeout.tv_sec = 0;
+		timeout.tv_usec = 2000;
+		select(0, 0, NULL, NULL, &timeout );
+	      }
+	    else
+	      {
+		/* select will be less active than locking on "read" */
+		timeout.tv_sec = 0;
+		timeout.tv_usec = 5000;
+		select(0, 0, NULL, NULL, &timeout );
+	      }
+	  }
+
+	if (ad->handle!=NULL) snd_pcm_close(ad->handle);
+	ad->handle=NULL;
+	return NULL;
+}
+
+static void alsa_start_r(AlsaReadData *d){
+	if (d->read_started==FALSE){
+		d->read_started=TRUE;
+		ms_thread_create(&d->thread,NULL,alsa_write_thread,d);
+	}else d->read_started=TRUE;
+}
+
+static void alsa_stop_r(AlsaReadData *d){
+	d->read_started=FALSE;
+	if (d->thread!=0)
+	  {
+	    ms_thread_join(d->thread,NULL);
+	    d->thread=0;
+	  }
+}
+#endif
+
+#ifdef THREADED_VERSION
+void alsa_read_preprocess(MSFilter *obj){
+	AlsaReadData *ad=(AlsaReadData*)obj->data;
+	alsa_start_r(ad);
+}
+#endif
+
 void alsa_read_postprocess(MSFilter *obj){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
+#ifdef THREADED_VERSION
+	alsa_stop_r(ad);
+#endif
 	if (ad->handle!=NULL) snd_pcm_close(ad->handle);
 	ad->handle=NULL;
 }
 
 void alsa_read_uninit(MSFilter *obj){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
+#ifdef THREADED_VERSION
+	alsa_stop_r(ad);
+#endif
 	if (ad->pcmdev!=NULL) ms_free(ad->pcmdev);
 	if (ad->handle!=NULL) snd_pcm_close(ad->handle);
+#ifdef THREADED_VERSION
+	ms_bufferizer_destroy(ad->bufferizer);
+	ms_mutex_destroy(&ad->mutex);
+#endif
 	ms_free(ad);
 }
 
+#ifndef THREADED_VERSION
 void alsa_read_process(MSFilter *obj){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
-#ifdef AMD_HACK
-	int samples=(160*ad->rate)/8000;
-#else
 	int samples=(128*ad->rate)/8000;
-#endif
 	int err;
 	mblk_t *om=NULL;
 	if (ad->handle==NULL && ad->pcmdev!=NULL){
 		ad->handle=alsa_open_r(ad->pcmdev,16,ad->nchannels==2,ad->rate);
 	}
 	if (ad->handle==NULL) return;
-	while (alsa_can_read(ad->handle,samples)){
+	while (alsa_can_read(ad->handle,samples)>=samples){
+	  
 		int size=samples*2;
 		om=allocb(size,0);
 		if ((err=alsa_read(ad->handle,om->b_wptr,samples))<=0) {
@@ -652,11 +781,28 @@ void alsa_read_process(MSFilter *obj){
 		om->b_wptr+=size;
 		/*ms_message("alsa_read_process: Outputing %i bytes",size);*/
 		ms_queue_put(obj->outputs[0],om);
-#ifdef AMD_HACK
-		break;
-#endif
 	}
 }
+#endif
+
+#ifdef THREADED_VERSION
+void alsa_read_process(MSFilter *obj){
+	AlsaReadData *ad=(AlsaReadData*)obj->data;
+	mblk_t *om=NULL;
+	int samples=(160*ad->rate)/8000;
+
+	ms_mutex_lock(&ad->mutex);
+	while (ms_bufferizer_get_avail(ad->bufferizer)>=samples*2){
+	  
+	  om=allocb(samples*2,0);
+	  ms_bufferizer_read(ad->bufferizer,om->b_wptr,samples*2);	  
+	  om->b_wptr+=samples*2;
+	  /*ms_message("alsa_read_process: Outputing %i bytes",size);*/
+	  ms_queue_put(obj->outputs[0],om);
+	}
+	ms_mutex_unlock(&ad->mutex);
+}
+#endif
 
 static int alsa_read_set_sample_rate(MSFilter *obj, void *param){
 	AlsaReadData *ad=(AlsaReadData*)obj->data;
@@ -684,6 +830,9 @@ MSFilterDesc alsa_read_desc={
 	.ninputs=0,
 	.noutputs=1,
 	.init=alsa_read_init,
+#ifdef THREADED_VERSION
+	.preprocess=alsa_read_preprocess,
+#endif
 	.process=alsa_read_process,
 	.postprocess=alsa_read_postprocess,
 	.uninit=alsa_read_uninit,
