@@ -102,7 +102,7 @@ int ice_sound_send_stun_request(RtpSession *session, struct CandidatePair *remot
 	if (round>500)
 		roll=2*roll;
 
-    if (round%roll==50)
+    if (round%roll==50 || round==10)
     {
         int pos;
 
