@@ -142,7 +142,7 @@ OSStatus writeACInputProc (
 		int readsize = ms_bufferizer_read(d->bufferizer,d->caSourceBuffer,*ioNumberDataPackets * packetSize);
 		ms_mutex_unlock(&d->mutex);
 		if(readsize != *ioNumberDataPackets * packetSize) {
-			ms_error("ms_bufferizer_read error request = %d result = %d", *ioNumberDataPackets * packetSize, readsize);
+		  /* ms_error("ms_bufferizer_read error request = %d result = %d", *ioNumberDataPackets * packetSize, readsize); */
 			memset(d->caSourceBuffer, 0, *ioNumberDataPackets * packetSize);
 			ioData->mBuffers[0].mDataByteSize = *ioNumberDataPackets * packetSize;		// tell the Audio Converter how much source data there is
 		} else {
@@ -358,9 +358,8 @@ static void ca_init(MSSndCard *card){
 	if (d->caInASBD.mFormatID == kAudioFormatLinearPCM && fAudioChannels == 1)
 		d->caInASBD.mFormatFlags &= ~kLinearPCMFormatFlagIsNonInterleaved;
 	d->caInASBD.mFormatFlags = kAudioFormatFlagIsFloat;
-#if __BIG_ENDIAN__
-	d->caInASBD.mFormatFlags |= kAudioFormatFlagIsBigEndian;
-#endif
+	if (htonl(0x1234) == 0x1234)
+	  d->caInASBD.mFormatFlags |= kAudioFormatFlagIsBigEndian;
 	d->caInASBD.mBitsPerChannel = sizeof(Float32) * 8;
 	d->caInASBD.mBytesPerFrame = d->caInASBD.mBitsPerChannel / 8;
 	d->caInASBD.mFramesPerPacket = 1;
@@ -477,7 +476,9 @@ static void ca_start_r(MSSndCard *card){
 		AudioStreamBasicDescription outASBD;
 		outASBD = d->caInASBD;
 		outASBD.mSampleRate = d->rate;
-		outASBD.mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked | kLinearPCMFormatFlagIsBigEndian;
+		outASBD.mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked;
+		if (htonl(0x1234) == 0x1234)
+		  outASBD.mFormatFlags |= kLinearPCMFormatFlagIsBigEndian;
 		outASBD.mBytesPerPacket = (d->bits / 8) * outASBD.mChannelsPerFrame;
 		outASBD.mBytesPerFrame = (d->bits / 8) * outASBD.mChannelsPerFrame;
 		outASBD.mFramesPerPacket = 1;
@@ -522,7 +523,9 @@ static void ca_start_w(MSSndCard *card){
 		inASBD.mSampleRate = d->rate;
 		inASBD.mFormatID = kAudioFormatLinearPCM;
 		// http://developer.apple.com/documentation/MusicAudio/Reference/CoreAudioDataTypesRef/Reference/reference.html
-		inASBD.mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked | kLinearPCMFormatFlagIsBigEndian;
+		inASBD.mFormatFlags = kAudioFormatFlagIsSignedInteger | kAudioFormatFlagIsPacked;
+		if (htonl(0x1234) == 0x1234)
+		  inASBD.mFormatFlags |= kLinearPCMFormatFlagIsBigEndian;
 		inASBD.mChannelsPerFrame = d->stereo ? 2 : 1;
 		inASBD.mBytesPerPacket = (d->bits / 8) * inASBD.mChannelsPerFrame;
 		inASBD.mBytesPerFrame = (d->bits / 8) * inASBD.mChannelsPerFrame;
