@@ -31,10 +31,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define CONF_GRAN_MAX 12 /* limit for 'too much data' */
 #endif
 
-#ifndef CONF_GRAN
-#define CONF_GRAN (160)
-#endif
-#define CONF_NSAMPLES CONF_GRAN*4 /* (CONF_GRAN/2) */
+//#ifndef CONF_GRAN
+//#define CONF_GRAN (160*4)
+//#endif
+#define CONF_NSAMPLES 160*4*4 /* (CONF_GRAN/2) */
 #ifndef CONF_MAX_PINS
 #define CONF_MAX_PINS 32
 #endif
@@ -75,7 +75,8 @@ typedef struct ConfState{
 static void channel_init(ConfState *s, Channel *chan, int pos){
 	ms_bufferizer_init(&chan->buff);
 #ifndef DISABLE_SPEEX
-	chan->speex_pp = speex_preprocess_state_init((s->conf_gran/2) *(s->samplerate/8000), s->samplerate);
+	//chan->speex_pp = speex_preprocess_state_init((s->conf_gran/2) *(s->samplerate/8000), s->samplerate);
+	chan->speex_pp = speex_preprocess_state_init(s->conf_gran/2, s->samplerate);
 	if (chan->speex_pp!=NULL) {
 		float f;
 		int val;
@@ -129,7 +130,7 @@ static void conf_init(MSFilter *f){
 	ConfState *s=(ConfState *)ms_new0(ConfState,1);
 	int i;
 	s->samplerate=8000;
-	s->conf_gran=CONF_GRAN;
+	s->conf_gran=((16 * s->samplerate) / 800) *2;
 	s->conf_nsamples=s->conf_gran/2;
     for (i=0;i<CONF_MAX_PINS;i++)
 		channel_init(s, &s->channels[i], i);
@@ -451,7 +452,7 @@ static int msconf_set_sr(MSFilter *f, void *arg){
 	int i;
 
 	s->samplerate = *(int*)arg;
-	s->conf_gran = (16 * s->samplerate) / 800;
+	s->conf_gran = ((16 * s->samplerate) / 800) *2;
 	s->conf_nsamples=s->conf_gran/2;
 	for (i=0;i<CONF_MAX_PINS;i++)
 		channel_uninit(&s->channels[i]);
