@@ -21,6 +21,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msticker.h"
 
 #include "ortp/telephonyevents.h"
+#if defined(__cplusplus)
+#define B64_NO_NAMESPACE
+#endif
+#include "ortp/b64.h"
+
 
 struct SenderData {
 	RtpSession *session;
@@ -30,6 +35,9 @@ struct SenderData {
 	uint32_t skip_until;
 	int rate;
 	char dtmf;
+	char relay_session_id[64];
+	int relay_session_id_size;
+	unsigned int last_rsi_time;
 	bool_t skip;
 	bool_t mute_mic;
 };
@@ -49,6 +57,8 @@ static void sender_init(MSFilter * f)
 	d->rate = 8000;
 	d->dtmf = 0;
 	d->mute_mic=FALSE;
+	d->relay_session_id_size=0;
+	d->last_rsi_time=0;
 	f->data = d;
 }
 
@@ -117,6 +127,13 @@ static int sender_unmute_mic(MSFilter * f, void *arg)
 	return 0;
 }
 
+static int sender_set_relay_session_id(MSFilter *f, void*arg){
+	SenderData *d = (SenderData *) f->data;
+	const char *tmp=(const char *)arg;
+	d->relay_session_id_size=b64_decode(tmp, strlen(tmp), (void*)d->relay_session_id, (unsigned int)sizeof(d->relay_session_id));
+	return 0;
+}
+
 /* the goal of that function is to return a absolute timestamp closest to real time, with respect of given packet_ts, which is a relative to an undefined origin*/
 static uint32_t get_cur_timestamp(MSFilter * f, uint32_t packet_ts)
 {
@@ -169,6 +186,13 @@ static void sender_process(MSFilter * f)
 		return;
 	}
 
+	if (d->relay_session_id_size>0 && 
+		( (f->ticker->time-d->last_rsi_time)>5000 || d->last_rsi_time==0) ) {
+		ms_message("relay session id sent in RTCP APP");
+		rtp_session_send_rtcp_APP(s,0,"RSID",(const uint8_t *)d->relay_session_id,d->relay_session_id_size);
+		d->last_rsi_time=f->ticker->time;
+	}
+
 	while ((im = ms_queue_get(f->inputs[0])) != NULL) {
 		mblk_t *header;
 
@@ -217,6 +241,7 @@ static MSFilterMethod sender_methods[] = {
 	{MS_RTP_SEND_SET_SESSION, sender_set_session},
 	{MS_RTP_SEND_SEND_DTMF, sender_send_dtmf},
 	{MS_RTP_SEND_SET_CANDIDATEPAIRS, sender_set_sdpcandidates},
+	{MS_RTP_SEND_SET_RELAY_SESSION_ID, sender_set_relay_session_id},
 	{0, NULL}
 };
 
@@ -346,14 +371,11 @@ static void receiver_process(MSFilter * f)
 
 	timestamp = (f->ticker->time * d->rate) / ((uint64_t)1000);
 	while ((m = rtp_session_recvm_with_ts(d->session, timestamp)) != NULL) {
-		mblk_t *payload = m->b_cont;
-
-		mblk_set_timestamp_info(payload, rtp_get_timestamp(m));
-		mblk_set_marker_info(payload, rtp_get_markbit(m));
-		mblk_set_payload_type(payload, rtp_get_payload_type(m));
-		
-		freeb(m);
-		ms_queue_put(f->outputs[0], payload);
+		mblk_set_timestamp_info(m, rtp_get_timestamp(m));
+		mblk_set_marker_info(m, rtp_get_markbit(m));
+		mblk_set_payload_type(m, rtp_get_payload_type(m));
+		rtp_get_payload(m,&m->b_rptr);
+		ms_queue_put(f->outputs[0], m);
 	}
 
 	/* check received STUN request */
