@@ -68,11 +68,8 @@ static void enc_uninit(MSFilter *f){
 static void enc_preprocess(MSFilter *f){
 	EncState *s=(EncState*)f->data;
 	const SpeexMode *mode=NULL;
-#ifdef WIN32
 	int _mode=0;
-#endif
 
-#ifdef WIN32
 	switch(s->rate){
 		case 8000:
 	        _mode = SPEEX_MODEID_NB;    /* rate = 8000Hz */
@@ -80,38 +77,16 @@ static void enc_preprocess(MSFilter *f){
 		case 16000:
 	        _mode = SPEEX_MODEID_WB;    /* rate = 16000Hz */
 			break;
-#if 0
 			/* should be supported in the future */
 		case 32000:
 	        _mode = SPEEX_MODEID_UWB;   /* rate = 32000Hz */
 			break;
-#endif
 		default:
 			ms_error("Unsupported rate for speex encoder (back to default rate=8000).");
 			s->rate=8000;
 	}
 	/* warning: speex_lib_get_mode() is not available on speex<1.1.12 */
 	mode = speex_lib_get_mode(_mode);
-#else
-	switch(s->rate){
-		case 8000:
-	        	mode = &speex_nb_mode;    /* rate = 8000Hz */
-			break;
-		case 16000:
-	        	mode = &speex_wb_mode;    /* rate = 16000Hz */
-			break;
-#if 0
-			/* should be supported in the future */
-		case 32000:
-	        _mode = SPEEX_MODEID_UWB;   /* rate = 32000Hz */
-			break;
-#endif
-		default:
-			ms_error("Unsupported rate for speex encoder (back to default rate=8000).");
-			s->rate=8000;
-			mode=&speex_nb_mode;
-	}
-#endif
 
 	if (mode==NULL)
 		return;
@@ -166,6 +141,28 @@ static void enc_preprocess(MSFilter *f){
 			}
 		}
 	}
+	else if (s->rate==16000){
+		if (s->mode!=0){
+			int q=0;
+			s->bitrate = -1; /* 28000; */
+			if (s->mode<=0 || s->mode>=8)
+				s->mode = 6; /* default mode */
+			/* no table exist for wide and ultra:
+			From libspeex/mode.c, those values seems to make sense... */
+			if (s->mode<=5)
+				q=5;
+			else if (s->mode==5)
+				q=6;
+			else if (s->mode==6)
+				q=8;
+			else if (s->mode>=7)
+				q=10;
+
+			if (speex_encoder_ctl(s->state,SPEEX_SET_QUALITY,&q)!=0){
+				ms_error("Could not set quality %i to speex encoder.",q);
+			}
+		}
+	}
 	else
 	{
 		if (s->mode!=0){
@@ -189,6 +186,7 @@ static void enc_preprocess(MSFilter *f){
 			}
 		}
 	}
+
 	if (s->maxbitrate>0){
 		/* convert from network bitrate to codec bitrate:*/
 		/* ((nbr/(50*8)) -20-12-8)*50*8*/
@@ -446,16 +444,24 @@ static void dec_uninit(MSFilter *f){
 static void dec_preprocess(MSFilter *f){
 	DecState *s=(DecState*)f->data;
 	const SpeexMode *mode=NULL;
+	int modeid;
 	switch(s->rate){
 		case 8000:
-			mode=speex_mode_list[SPEEX_MODEID_NB];
+	        modeid = SPEEX_MODEID_NB;    /* rate = 8000Hz */
 			break;
 		case 16000:
-			mode=speex_mode_list[SPEEX_MODEID_WB];
+	        modeid = SPEEX_MODEID_WB;    /* rate = 16000Hz */
+			break;
+			/* should be supported in the future */
+		case 32000:
+	        modeid = SPEEX_MODEID_UWB;   /* rate = 32000Hz */
 			break;
 		default:
-			ms_fatal("Unsupported rate for speex decoder.");
+			ms_error("Unsupported rate for speex decoder (back to default rate=8000).");
+			modeid=SPEEX_MODEID_NB;
 	}
+	/* warning: speex_lib_get_mode() is not available on speex<1.1.12 */
+	mode = speex_lib_get_mode(modeid);
 	s->state=speex_decoder_init(mode);
 	speex_mode_query(mode,SPEEX_MODE_FRAME_SIZE,&s->frsz);
 	if (s->penh==1)
