@@ -38,6 +38,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msv4l.h"
+#include "mediastreamer2/mswebcam.h"
 #include "nowebcam.h"
 
 /* From: Logitech QuickCam USB driver */
@@ -74,6 +75,8 @@ typedef struct V4lState{
 	bool_t run;
 	bool_t usemire;
 	bool_t v4lv2; /*we interface with a V4Lv2 driver */
+	bool_t force_v1;
+	bool_t auto_started;
 }V4lState;
 
 static void *v4l_thread(void *s);
@@ -95,6 +98,8 @@ static bool_t v4lv2_try_format(V4lState *s, int fmtid){
         if (ioctl (s->fd, VIDIOC_S_FMT, &fmt)<0){
 		return FALSE;
 	}
+	s->got_vsize.width=s->vsize.width;
+	s->got_vsize.height=s->vsize.height;
 	return TRUE;
 }
 
@@ -185,7 +190,12 @@ static int v4lv2_do_mmap(V4lState *s){
 			ms_error("Could not mmap: %s",strerror(errno));
 		}
 		msg=esballoc(start,buf.length,0,NULL);
-		msg->b_wptr+=buf.length;
+		/* adjust to real size of picture*/
+		if (s->pix_fmt==MS_RGB24)
+			msg->b_wptr+=s->vsize.width*s->vsize.height*3;
+		else 
+			msg->b_wptr+=(s->vsize.width*s->vsize.height*3)/2;
+
 		s->frames[i]=msg;
 	}
 	s->frame_max=req.count;
@@ -230,14 +240,14 @@ static mblk_t * v4lv2_grab_image(V4lState *s){
 				ms_warning("VIDIOC_DQBUF failed: %s",strerror(errno));
 			}
 		}else{
-			s->queued--;
-			/*decrement ref count of dequeued buffer */
-			ret=s->frames[buf.index];
-			ret->b_datap->db_ref--;
 			if (buf.index >= s->frame_max){
 				ms_error("buf.index>=s->max_frames !");
 				return NULL;
 			}
+			s->queued--;
+			/*decrement ref count of dequeued buffer */
+			ret=s->frames[buf.index];
+			ret->b_datap->db_ref--;
 			if (buf.bytesused<=30){
 				ms_warning("Ignoring empty buffer...");
 				return NULL;
@@ -282,6 +292,8 @@ static void v4l_init(MSFilter *f){
 	s->fps=15;
 	s->usemire=(getenv("DEBUG")!=NULL);
 	s->queued=0;
+	s->force_v1=FALSE;
+	s->auto_started=FALSE;
 	f->data=s;
 }
 
@@ -333,7 +345,7 @@ static int v4l_start(MSFilter *f, void *arg)
 		return -1;
 	}else{
 #ifdef HAVE_LINUX_VIDEODEV2_H
-		if (v4lv2_configure(s)<0) {/*might not be V4LV2 */
+		if (s->force_v1 || v4lv2_configure(s)<0) {/*might not be V4LV2 */
 #else
 		if (1){
 #endif
@@ -346,7 +358,7 @@ static int v4l_start(MSFilter *f, void *arg)
 				s->fd=-1;
 				return -1;
 			}
-			reuse_fd=TRUE;
+			if (!s->force_v1) reuse_fd=TRUE;
 
 			err=ioctl(s->fd, VIDIOCGCAP, &vidcap);
 			if (err==0)
@@ -398,6 +410,7 @@ static void v4l_stop_capture(V4lState *s){
 		s->run=FALSE;
 		ms_thread_join(s->thread,NULL);
 		ms_message("v4l thread has joined.");
+		flushq(&s->rq,0);
 	}
 }
 
@@ -493,6 +506,9 @@ static bool_t try_size(V4lState *s, MSVideoSize vsize){
 		ms_warning("Could not get window size: %s",strerror(errno));
 		return FALSE;
 	}
+	s->vsize.width=vsize.width;
+	s->vsize.height=vsize.height;
+
 	if (s->vsize.width!=win.width || s->vsize.height!=win.height){
 		ms_warning("Capture size is not what we expected: asked for %ix%i and get %ix%i",s->vsize.width,s->vsize.height, win.width, win.height);
 	}
@@ -572,6 +588,9 @@ static int v4l_configure(V4lState *s)
 		ms_message("Driver supports UYVY, using that format.");
 		s->pix_fmt=MS_UYVY;
 	}else{	
+	  s->vsize.width=MS_VIDEO_SIZE_CIF_W;
+	  s->vsize.height=MS_VIDEO_SIZE_CIF_H;
+		s->pix_fmt=MS_YUV420P;
 		ms_fatal("Unsupported video pixel format.");
 		return -1;
 	}
@@ -582,7 +601,11 @@ static int v4l_configure(V4lState *s)
 				if (!try_size(s,MS_VIDEO_SIZE_CIF)) {
 					if (!try_size(s,MS_VIDEO_SIZE_QCIF)) {
 						if (!try_size(s,MS_VIDEO_SIZE_QVGA)) {
-							return -1;
+						  if (!try_size(s,MS_VIDEO_SIZE_4CIF)) {
+						    if (!try_size(s,MS_VIDEO_SIZE_1024)) {
+						      return -1;
+						    }
+						  }
 						}
 					}
 				}
@@ -845,9 +868,20 @@ static void *v4l_thread(void *ptr){
 		else
 #endif
 			m=v4l_grab_image_mmap(s);
-		if (m) {
+
+		if (s->vsize.width!=s->got_vsize.width){
+			if (m){
+				/* mblock was allocated by crop or pad! */ 
+				ms_mutex_lock(&s->mutex);
+				putq(&s->rq,m);
+				ms_mutex_unlock(&s->mutex);
+			}else{
+				ms_error("grabbing failed !");
+			}
+		} else if (m!=NULL) {
+			mblk_t *dm=dupmsg(m);
 			ms_mutex_lock(&s->mutex);
-			putq(&s->rq,dupmsg(m));
+			putq(&s->rq,dm);
 			ms_mutex_unlock(&s->mutex);
 		}
 	}
@@ -877,7 +911,10 @@ static void v4l_process(MSFilter * obj){
 				om=dupmsg(v4l_make_mire(s));
 			}else {
 				mblk_t *tmpm=v4l_make_nowebcam(s);
-				if (tmpm) om=dupmsg(tmpm);
+				if (tmpm) {
+					om=dupmsg(tmpm);
+					mblk_set_precious_flag(om,1);
+				}
 			}
 		}
 		ms_mutex_unlock(&s->mutex);
@@ -894,12 +931,27 @@ static void v4l_process(MSFilter * obj){
 
 static void v4l_preprocess(MSFilter *f){
 	V4lState *s=(V4lState*)f->data;
-	v4l_start_capture(s);
+	if (s->fd==-1){
+		s->auto_started=TRUE;
+		v4l_start(f,NULL);
+		v4l_start_capture(s);
+	}
+	else
+	{
+		v4l_start_capture(s);
+	}
 }
 
 static void v4l_postprocess(MSFilter *f){
 	V4lState *s=(V4lState*)f->data;
-	v4l_stop_capture(s);
+	if (s->auto_started){
+		v4l_stop_capture(s);
+		v4l_stop(f,NULL);
+	}
+	else
+	{
+		v4l_stop_capture(s);
+	}
 }
 
 static int v4l_set_fps(MSFilter *f, void *arg){
@@ -911,7 +963,13 @@ static int v4l_set_fps(MSFilter *f, void *arg){
 
 static int v4l_get_pix_fmt(MSFilter *f,void *arg){
 	V4lState *s=(V4lState*)f->data;
-	*((MSPixFmt*)arg) = s->pix_fmt;
+	MSPixFmt res;
+	if (s->fd==-1){
+		v4l_start(f,NULL);
+		res=s->pix_fmt;
+		v4l_stop(f,NULL);
+	}else res=s->pix_fmt;
+	*((MSPixFmt*)arg)=res;
 	return 0;
 }
 
@@ -937,6 +995,13 @@ static MSFilterMethod methods[]={
 	{	0	,	NULL			}
 };
 
+static int v4l_set_devfile(MSFilter *f, void *arg){
+	V4lState *s=(V4lState*)f->data;
+	if (s->dev) ms_free(s->dev);
+	s->dev=ms_strdup((char*)arg);
+	return 0;
+}
+
 MSFilterDesc ms_v4l_desc={
 	.id=MS_V4L_ID,
 	.name="MSV4l",
@@ -954,4 +1019,53 @@ MSFilterDesc ms_v4l_desc={
 
 MS_FILTER_DESC_EXPORT(ms_v4l_desc)
 
+static MSFilter *v4l_create_reader(MSWebCam *obj){
+	MSFilter *f=ms_filter_new_from_desc(&ms_v4l_desc);
+	V4lState *s=(V4lState*)f->data;
+	v4l_set_devfile(f,obj->name);
+	s->force_v1=TRUE;
+	return f;
+}
+
+static void v4l_detect(MSWebCamManager *obj);
+
+static void v4l_cam_init(MSWebCam *cam){
+	
+}
+
+MSWebCamDesc v4l_desc={
+	"V4L",
+	&v4l_detect,
+	&v4l_cam_init,
+	&v4l_create_reader,
+	NULL
+};
+
+static void v4l_detect(MSWebCamManager *obj){
+	struct video_capability cap;
+	const char *devname="/dev/video0";
+	int fd=open(devname,O_RDWR);
+	if (fd!=-1){
+		if (ioctl (fd, VIDIOCGCAP, &cap)==0) {
+			/* is a V4Lv1 */
+			MSWebCam *cam=ms_web_cam_new(&v4l_desc);
+			cam->name=ms_strdup(devname);
+			ms_web_cam_manager_add_cam(obj,cam);
+		}
+		close(fd);
+	}
+	devname="/dev/video1";
+	fd=open(devname,O_RDWR);
+	if (fd!=-1){
+		if (ioctl (fd, VIDIOCGCAP, &cap)==0) {
+			/* is a V4Lv1 */
+			MSWebCam *cam=ms_web_cam_new(&v4l_desc);
+			cam->name=ms_strdup(devname);
+			ms_web_cam_manager_add_cam(obj,cam);
+		}
+		close(fd);
+	}
+}
+
+
 #endif
