@@ -195,9 +195,9 @@ pascal OSErr sgdata_callback(SGChannel c, Ptr p, long len, long *offset, long ch
 	}
 
       buf->b_wptr+=size;
-      ms_mutex_lock(&s->mutex);
+      //ms_mutex_lock(&s->mutex); /* called during SGIdle? */
       putq(&s->rq, buf);
-      ms_mutex_unlock(&s->mutex);
+      //ms_mutex_unlock(&s->mutex);
     }
   }
 
@@ -310,6 +310,8 @@ static int v4m_start(MSFilter *f, void *arg)
 	if (err!=0)
 	  {
 	    s->pix_fmt=MS_YUV420P;
+	    s->vsize.width=MS_VIDEO_SIZE_CIF_W;
+	    s->vsize.height=MS_VIDEO_SIZE_CIF_H;
 	    return 0;
 	  }
 
@@ -328,8 +330,11 @@ static void v4m_start_capture(V4lState *s){
 static int v4m_stop(MSFilter *f, void *arg){
 	V4lState *s=(V4lState*)f->data;
 	if (s->seqgrab!=NULL){
+	  ms_mutex_lock(&s->mutex);
 	  SGStop(s->seqgrab);
 	  v4m_close(s);
+	  flushq(&s->rq,0);
+	  ms_mutex_unlock(&s->mutex);
 	}
 	return 0;
 }
@@ -398,6 +403,9 @@ static void v4m_process(MSFilter * obj){
 		s->start_time=obj->ticker->time;
 		s->frame_count=0;
 	}
+
+	ms_mutex_lock(&s->mutex);
+
 	if (s->seqgrab!=NULL)
 	{
 	  SGIdle(s->seqgrab);
@@ -406,19 +414,22 @@ static void v4m_process(MSFilter * obj){
 	cur_frame=((obj->ticker->time-s->start_time)*s->fps/1000.0);
 	if (cur_frame>=s->frame_count){
 		mblk_t *om=NULL;
-		ms_mutex_lock(&s->mutex);
 		/*keep the most recent frame if several frames have been captured */
 		if (s->seqgrab!=NULL){
 			om=getq(&s->rq);
 		}else{
+		  if (s->pix_fmt==MS_YUV420P
+		      && s->vsize.width==MS_VIDEO_SIZE_CIF_W
+		      && s->vsize.height==MS_VIDEO_SIZE_CIF_H)
+		    {
 			if (s->usemire){
 				om=dupmsg(v4m_make_mire(s));
 			}else {
 				mblk_t *tmpm=v4m_make_nowebcam(s);
 				if (tmpm) om=dupmsg(tmpm);
 			}
+		    }
 		}
-		ms_mutex_unlock(&s->mutex);
 		if (om!=NULL){
 			timestamp=obj->ticker->time*90;/* rtp uses a 90000 Hz clockrate for video*/
 			mblk_set_timestamp_info(om,timestamp);
@@ -428,6 +439,8 @@ static void v4m_process(MSFilter * obj){
 			s->frame_count++;
 		}
 	}else flushq(&s->rq,0);
+
+	ms_mutex_unlock(&s->mutex);
 }
 
 static void v4m_preprocess(MSFilter *f){
