@@ -18,6 +18,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
 
 #include "mediastreamer2/mssndcard.h"
 #include "mediastreamer2/msfilter.h"
@@ -30,7 +33,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+
+#ifdef HAVE_ALLOCA_H /*FreeBSD does not have alloca.h*/
 #include <alloca.h>
+#endif
 
 MSFilter *ms_oss_read_new(MSSndCard *card);
 MSFilter *ms_oss_write_new(MSSndCard *card);
@@ -431,10 +437,13 @@ static void * oss_thread(void *p){
 
 static void oss_start_r(MSSndCard *card){
 	OssData *d=(OssData*)card->data;
+	ms_mutex_lock(&d->mutex);
 	if (d->read_started==FALSE && d->write_started==FALSE){
 		d->read_started=TRUE;
 		ms_thread_create(&d->thread,NULL,oss_thread,card);
 	}else d->read_started=TRUE;
+	flushq(&d->rq,0);
+	ms_mutex_unlock(&d->mutex);
 }
 
 static void oss_stop_r(MSSndCard *card){
@@ -445,14 +454,22 @@ static void oss_stop_r(MSSndCard *card){
 	}
 }
 
+static void _flush_buffer(MSBufferizer *obj){
+	flushq(&obj->q,0);
+	obj->size=0;
+}
+
 static void oss_start_w(MSSndCard *card){
 	OssData *d=(OssData*)card->data;
+	ms_mutex_lock(&d->mutex);
 	if (d->read_started==FALSE && d->write_started==FALSE){
 		d->write_started=TRUE;
 		ms_thread_create(&d->thread,NULL,oss_thread,card);
 	}else{
 		d->write_started=TRUE;
 	}
+	_flush_buffer(d->bufferizer);
+	ms_mutex_unlock(&d->mutex);
 }
 
 static void oss_stop_w(MSSndCard *card){
