@@ -159,8 +159,8 @@ static int pasnd_open(PASndData *device, int devnumber, int bits,int stereo, int
 			&device->waveoutdev,	/* stream */
 			NULL,                   /* no input */
 			&outputParameters,	//
-			8000,			// double sampleRate
-			160,			//unsigned long framesPerBuffer
+			rate,			// double sampleRate
+			160*(rate/8000),	//unsigned long framesPerBuffer
 			paClipOff,
 			SpeakerCallback,	//PortAudioCallback *callback
 			(void *) device);	//void *userData
@@ -182,8 +182,8 @@ static int pasnd_open(PASndData *device, int devnumber, int bits,int stereo, int
 			&device->waveindev,	//PortAudioStream** stream
 			&inputParameters,	/* input param*/
 			NULL,                   /* output param */
-			8000,			// double sampleRate
-			160,			//unsigned long framesPerBuffer
+			rate,			// double sampleRate
+			160*(rate/8000),	//unsigned long framesPerBuffer
 			paClipOff,
 			WaveInCallback,		//PortAudioCallback *callback
 			(void *) device);	//void *userData
