@@ -17,11 +17,14 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/swscale.h>
+#include "ffmpeg-priv.h"
 
 extern void ms_ffmpeg_check_init();
 
@@ -40,7 +43,7 @@ int ms_pix_fmt_to_ffmpeg(MSPixFmt fmt){
 		case MS_UYVY:
 			return PIX_FMT_UYVY422;
 		case MS_YUY2:
-			return PIX_FMT_YUYV422;   /* <- same as MS_UYVY */
+			return PIX_FMT_YUYV422;   /* <- same as MS_YUYV */
 		default:
 			ms_fatal("format not supported.");
 			return -1;
@@ -130,7 +133,7 @@ static void pixconv_process(MSFilter *f){
                 		NULL, NULL, NULL);
 			}
 			if (sws_scale(s->sws_ctx,inbuf.data,inbuf.linesize, 0,
-				0, s->outbuf.planes, s->outbuf.strides)!=0){
+				s->size.height, s->outbuf.planes, s->outbuf.strides)<0){
 				ms_error("MSPixConv: Error in sws_scale().");
 			}
 			freemsg(im);
