@@ -17,12 +17,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msvideo.h"
 
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/swscale.h>
+#include "ffmpeg-priv.h"
 
 typedef struct SizeConvState{
 	MSVideoSize target_vsize;
@@ -151,7 +154,7 @@ static void size_conv_process(MSFilter *f){
 				struct SwsContext *sws_ctx=get_resampler(s,inbuf.w,inbuf.h);
 				mblk_t *om=size_conv_alloc_mblk(s);
 				if (sws_scale(sws_ctx,inbuf.planes,inbuf.strides, 0,
-					0, s->outbuf.planes, s->outbuf.strides)!=0){
+					inbuf.h, s->outbuf.planes, s->outbuf.strides)<0){
 					ms_error("MSSizeConv: error in sws_scale().");
 				}
 				ms_queue_put(f->outputs[0],om);
