@@ -17,8 +17,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/swscale.h>
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
+#include "ffmpeg-priv.h"
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
@@ -77,14 +80,7 @@ static void dec_mpeg4_init(MSFilter *f){
 }
 
 static void dec_mjpeg_init(MSFilter *f){
-	DecState *s;
-	dec_init(f,CODEC_ID_JPEGLS);
-	s=(DecState*)f->data;
-	if (s->av_codec==NULL){
-		/*retry with mjpeg*/
-		ms_error("JPEGLS codec not found, retrying with MJPEG...");
-		dec_init(f,CODEC_ID_MJPEG);
-	}
+	dec_init(f,CODEC_ID_MJPEG);
 }
 
 static void dec_snow_init(MSFilter *f){
@@ -144,6 +140,21 @@ static void dec_postprocess(MSFilter *f){
 	}
 }
 
+static mblk_t * skip_rfc2190_header(mblk_t *inm){
+	if (msgdsize(inm) >= 4) {
+		uint8_t *ph = inm->b_rptr;
+		int F = (ph[0]>>7) & 0x1;
+		int P = (ph[0]>>6) & 0x1;
+		if (F == 0) inm->b_rptr += 4;  // mode A
+		else if (P == 0) inm->b_rptr += 8; // mode B
+		else inm->b_rptr += 12;   // mode C
+	} else {
+		freemsg(inm);
+		inm=NULL;
+	}
+	return inm;
+}
+
 static mblk_t * skip_rfc2429_header(mblk_t *inm){
 	if (msgdsize(inm) >= 2){
 		uint32_t *p = (uint32_t*)inm->b_rptr;
@@ -217,7 +228,7 @@ static mblk_t *get_as_yuvmsg(MSFilter *f, DecState *s, AVFrame *orig){
                 	NULL, NULL, NULL);
 	}
 	if (sws_scale(s->sws_ctx,orig->data,orig->linesize, 0,
-					0, s->outbuf.planes, s->outbuf.strides)!=0){
+					ctx->height, s->outbuf.planes, s->outbuf.strides)<0){
 		ms_error("%s: error in sws_scale().",f->desc->name);
 	}
 	return dupmsg(s->yuv_msg);
@@ -229,7 +240,8 @@ static void dec_process_frame(MSFilter *f, mblk_t *inm){
 	int got_picture;
 	/* get a picture from the input queue */
 	
-	if (s->codec==CODEC_ID_H263) inm=skip_rfc2429_header(inm);
+	if (f->desc->id==MS_H263_DEC_ID) inm=skip_rfc2429_header(inm);
+	else if (f->desc->id==MS_H263_OLD_DEC_ID) inm=skip_rfc2190_header(inm);
 	else if (s->codec==CODEC_ID_SNOW && s->input==NULL) inm=parse_snow_header(s,inm);
 	if (inm){
 		/* accumulate the video packet until we have the rtp markbit*/
@@ -295,6 +307,23 @@ MSFilterDesc ms_h263_dec_desc={
 	methods
 };
 
+MSFilterDesc ms_h263_old_dec_desc={
+	MS_H263_DEC_ID,
+	"MSH263OldDec",
+	"A H.263 decoder using ffmpeg library",
+	MS_FILTER_DECODER,
+	"H263",
+	1,
+	1,
+	dec_h263_init,
+	dec_preprocess,
+	dec_process,
+	dec_postprocess,
+	dec_uninit,
+	methods
+};
+
+
 MSFilterDesc ms_mpeg4_dec_desc={
 	MS_MPEG4_DEC_ID,
 	"MSMpeg4Dec",
@@ -361,6 +390,23 @@ MSFilterDesc ms_h263_dec_desc={
 	.methods= methods
 };
 
+MSFilterDesc ms_h263_old_dec_desc={
+	.id=MS_H263_OLD_DEC_ID,
+	.name="MSH263OldDec",
+	.text="A H.263 decoder using ffmpeg library",
+	.category=MS_FILTER_DECODER,
+	.enc_fmt="H263",
+	.ninputs=1,
+	.noutputs=1,
+	.init=dec_h263_init,
+	.preprocess=dec_preprocess,
+	.process=dec_process,
+	.postprocess=dec_postprocess,
+	.uninit=dec_uninit,
+	.methods= methods
+};
+
+
 MSFilterDesc ms_mpeg4_dec_desc={
 	.id=MS_MPEG4_DEC_ID,
 	.name="MSMpeg4Dec",
@@ -413,5 +459,6 @@ MSFilterDesc ms_snow_dec_desc={
 
 MS_FILTER_DESC_EXPORT(ms_mpeg4_dec_desc)
 MS_FILTER_DESC_EXPORT(ms_h263_dec_desc)
+MS_FILTER_DESC_EXPORT(ms_h263_old_dec_desc)
 MS_FILTER_DESC_EXPORT(ms_mjpeg_dec_desc)
 MS_FILTER_DESC_EXPORT(ms_snow_dec_desc)
