@@ -17,7 +17,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
+#ifdef HAVE_LIBAVCODEC_AVCODEC_H
+#include <libavcodec/avcodec.h>
+#else
 #include <ffmpeg/avcodec.h>
+#endif
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msticker.h"
@@ -84,38 +92,6 @@ static int enc_set_mtu(MSFilter *f,void *arg){
 	return 0;
 }
 
-static int enc_set_br(MSFilter *f, void *arg){
-	EncState *s=(EncState*)f->data;
-	bool_t snow=s->codec==CODEC_ID_SNOW;
-	s->maxbr=*(int*)arg;
-	if (s->maxbr>=512000){
-		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=17;
-	}else if (s->maxbr>=256000){
-		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
-		s->fps=10;
-		s->qmin=3;
-	}else if (s->maxbr>=128000){
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=10;
-		s->qmin=3;
-	}else if (s->maxbr>=64000){
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=snow ? 7 : 5;
-		s->qmin=snow ? 4 : 5;
-	}else{
-		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
-		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
-		s->fps=5;
-		s->qmin=5;
-	}
-	return 0;
-}
-
 static bool_t parse_video_fmtp(const char *fmtp, float *fps, MSVideoSize *vsize){
 	char *tmp=ms_strdup(fmtp);
 	char *semicolon;
@@ -186,6 +162,7 @@ static void enc_init(MSFilter *f, enum CodecID codec)
 	s->vsize.height=MS_VIDEO_SIZE_CIF_H;
 	s->qmin=2;
 	s->req_vfu=FALSE;
+	s->av_context.codec=NULL;
 }
 
 static void enc_h263_init(MSFilter *f){
@@ -204,14 +181,15 @@ static void prepare(EncState *s){
 	AVCodecContext *c=&s->av_context;
 	avcodec_get_context_defaults(c);
 	/* put codec parameters */
-	c->bit_rate=(float)s->maxbr*0.8;
-	c->bit_rate_tolerance=s->maxbr/5;
+	c->bit_rate=(float)s->maxbr*0.7;
+	c->bit_rate_tolerance=(float)c->bit_rate/(s->fps-1);
+
 	if (s->codec!=CODEC_ID_SNOW && s->maxbr<256000){
 		/*snow does not like 1st pass rate control*/
 		/*and rate control eats too much cpu with CIF high fps pictures*/
-		c->rc_max_rate=s->maxbr;
-		c->rc_min_rate=0;
-		c->rc_buffer_size=100000;
+		c->rc_max_rate=(float)s->maxbr*0.8;
+		c->rc_min_rate=c->bit_rate;
+		c->rc_buffer_size=c->rc_max_rate;
 	}else{
 		/*use qmin instead*/
 		c->qmin=s->qmin;
@@ -235,7 +213,9 @@ static void prepare_h263(EncState *s){
 	AVCodecContext *c=&s->av_context;
 	/* we don't use the rtp_callback but use rtp_mode that forces ffmpeg to insert
 	Start Codes as much as possible in the bitstream */
-	c->rtp_mode = 1;
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
+        c->rtp_mode = 1;
+#endif
 	c->rtp_payload_size = s->mtu/2;
 	if (s->profile==0){
 		s->codec=CODEC_ID_H263;
@@ -314,6 +294,56 @@ static void enc_postprocess(MSFilter *f){
 	}
 }
 
+static void add_rfc2190_header(mblk_t **packet, AVCodecContext *context){
+	mblk_t *header;
+	header = allocb(4, 0);
+	memset(header->b_wptr, 0, 4);
+	// assume video size is CIF or QCIF
+	if (context->width == 352 && context->height == 288) header->b_wptr[1] = 0x60;
+	else header->b_wptr[1] = 0x40;
+	if (context->coded_frame->pict_type != FF_I_TYPE) header->b_wptr[1] |= 0x10;
+	header->b_wptr += 4;
+	header->b_cont = *packet;
+	*packet = header;
+}
+
+static int get_gbsc(uint8_t *psc, uint8_t *end)
+{
+	int len = end-psc;
+	uint32_t buf;
+	int i, j, k;
+	k = len;
+	for (i = 2; i < len-4; i++) {
+		buf = *((uint32_t *)(psc+i));
+		for (j = 0; j < 8; j++) {
+			if (((buf >> j) & 0x00FCFFFF) == 0x00800000) {/*PSC*/
+				i += 2;
+				k=i;
+				break;
+			} else if (((buf >> j) & 0x0080FFFF) == 0x00800000) {/*GBSC*/
+				i += 2;
+				k = i;
+				break;
+			}
+		}
+	}
+	return k;
+}
+
+static void rfc2190_generate_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp){
+	mblk_t *packet=NULL;
+	
+	while (frame->b_rptr<frame->b_wptr){
+		packet=dupb(frame);
+		frame->b_rptr=packet->b_wptr=packet->b_rptr+get_gbsc(packet->b_rptr, MIN(packet->b_rptr+s->mtu,frame->b_wptr));
+		add_rfc2190_header(&packet, &s->av_context);
+		mblk_set_timestamp_info(packet,timestamp);
+		ms_queue_put(f->outputs[0],packet);
+	}
+	/* the marker bit is set on the last packet, if any.*/
+	mblk_set_marker_info(packet,TRUE);
+}
+
 static void mpeg4_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint32_t timestamp){
 	uint8_t *rptr;
 	mblk_t *packet=NULL;
@@ -331,7 +361,7 @@ static void mpeg4_fragment_and_send(MSFilter *f,EncState *s,mblk_t *frame, uint3
 	mblk_set_marker_info(packet,TRUE);
 }
 
-static void generate_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp, uint8_t *psc, uint8_t *end, bool_t last_packet){
+static void rfc4629_generate_follow_on_packets(MSFilter *f, EncState *s, mblk_t *frame, uint32_t timestamp, uint8_t *psc, uint8_t *end, bool_t last_packet){
 	mblk_t *packet;
 	int len=end-psc;
 	
@@ -400,17 +430,22 @@ static void split_and_send(MSFilter *f, EncState *s, mblk_t *frame){
 	}
 
 	ms_debug("processing frame of size %i",frame->b_wptr-frame->b_rptr);
-	lastpsc=frame->b_rptr;
-	while(1){
-		psc=get_psc(lastpsc+2,frame->b_wptr,s->mtu);
-		if (psc!=NULL){
-			generate_packets(f,s,frame,timestamp,lastpsc,psc,FALSE);
-			lastpsc=psc;
-		}else break;
+	if (f->desc->id==MS_H263_ENC_ID){
+		lastpsc=frame->b_rptr;
+		while(1){
+			psc=get_psc(lastpsc+2,frame->b_wptr,s->mtu);
+			if (psc!=NULL){
+				rfc4629_generate_follow_on_packets(f,s,frame,timestamp,lastpsc,psc,FALSE);
+				lastpsc=psc;
+			}else break;
+		}
+		/* send the end of frame */
+		rfc4629_generate_follow_on_packets(f,s,frame, timestamp,lastpsc,frame->b_wptr,TRUE);
+	}else if (f->desc->id==MS_H263_OLD_ENC_ID){
+		rfc2190_generate_packets(f,s,frame,timestamp);
+	}else{
+		ms_fatal("Ca va tres mal.");
 	}
-	
-	/* send the end of frame */
-	generate_packets(f,s,frame, timestamp,lastpsc,frame->b_wptr,TRUE);
 }
 
 static void process_frame(MSFilter *f, mblk_t *inm){
@@ -458,11 +493,64 @@ static void enc_process(MSFilter *f){
 		ms_queue_flush(f->inputs[0]);
 		return;
 	}
+	ms_filter_lock(f);
 	while((inm=ms_queue_get(f->inputs[0]))!=0){
 		process_frame(f,inm);
 	}
+	ms_filter_unlock(f);
 }
 
+
+static int enc_get_br(MSFilter *f, void *arg){
+	EncState *s=(EncState*)f->data;
+	*(int*)arg=s->maxbr;
+	return 0;
+}
+
+static int enc_set_br(MSFilter *f, void *arg){
+	EncState *s=(EncState*)f->data;
+	bool_t snow=s->codec==CODEC_ID_SNOW;
+	s->maxbr=*(int*)arg;
+	if (s->maxbr>=1024000 && s->codec!=CODEC_ID_H263P){
+		s->vsize.width = MS_VIDEO_SIZE_4CIF_W;
+		s->vsize.height = MS_VIDEO_SIZE_4CIF_H;
+		s->fps=17;
+	}else if (s->maxbr>=512000){
+		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
+		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
+		s->fps=17;
+	}else if (s->maxbr>=256000){
+		s->vsize.width=MS_VIDEO_SIZE_CIF_W;
+		s->vsize.height=MS_VIDEO_SIZE_CIF_H;
+		s->fps=10;
+		s->qmin=3;
+	}else if (s->maxbr>=128000){
+		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
+		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
+		s->fps=10;
+		s->qmin=3;
+	}else if (s->maxbr>=64000){
+		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
+		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
+		s->fps=snow ? 7 : 5;
+		s->qmin=snow ? 4 : 5;
+	}else{
+		s->vsize.width=MS_VIDEO_SIZE_QCIF_W;
+		s->vsize.height=MS_VIDEO_SIZE_QCIF_H;
+		s->fps=5;
+		s->qmin=5;
+	}
+	if (s->av_context.codec!=NULL){
+		/*apply new settings dynamically*/
+		ms_filter_lock(f);
+		enc_postprocess(f);
+		enc_preprocess(f);
+		ms_filter_lock(f);
+	}
+	return 0;
+}
+
+
 static MSFilterMethod methods[]={
 	{	MS_FILTER_SET_FPS	,	enc_set_fps	},
 	{	MS_FILTER_GET_FPS	,	enc_get_fps	},
@@ -470,6 +558,7 @@ static MSFilterMethod methods[]={
 	{	MS_FILTER_GET_VIDEO_SIZE ,	enc_get_vsize },
 	{	MS_FILTER_ADD_FMTP	,	enc_add_fmtp },
 	{	MS_FILTER_SET_BITRATE	,	enc_set_br	},
+	{	MS_FILTER_GET_BITRATE	,	enc_get_br	},
 	{	MS_FILTER_SET_MTU	,	enc_set_mtu	},
 	{	MS_FILTER_REQ_VFU	,	enc_req_vfu	},
 	{	0			,	NULL	}
@@ -493,6 +582,22 @@ MSFilterDesc ms_h263_enc_desc={
 	methods
 };
 
+MSFilterDesc ms_h263_old_enc_desc={
+	MS_H263_OLD_ENC_ID,
+	"MSH263OldEnc",
+	"A video H.263 encoder using ffmpeg library. It is compliant with old RFC2190 spec.",
+	MS_FILTER_ENCODER,
+	"H263",
+	1, /*MS_YUV420P is assumed on this input */
+	1,
+	enc_h263_init,
+	enc_preprocess,
+	enc_process,
+	enc_postprocess,
+	enc_uninit,
+	methods
+};
+
 MSFilterDesc ms_mpeg4_enc_desc={
 	MS_MPEG4_ENC_ID,
 	"MSMpeg4Enc",
@@ -543,6 +648,22 @@ MSFilterDesc ms_h263_enc_desc={
 	.methods=methods
 };
 
+MSFilterDesc ms_h263_old_enc_desc={
+	.id=MS_H263_OLD_ENC_ID,
+	.name="MSH263Enc",
+	.text="A video H.263 encoder using ffmpeg library, compliant with old RFC2190 spec.",
+	.category=MS_FILTER_ENCODER,
+	.enc_fmt="H263",
+	.ninputs=1, /*MS_YUV420P is assumed on this input */
+	.noutputs=1,
+	.init=enc_h263_init,
+	.preprocess=enc_preprocess,
+	.process=enc_process,
+	.postprocess=enc_postprocess,
+	.uninit=enc_uninit,
+	.methods=methods
+};
+
 MSFilterDesc ms_mpeg4_enc_desc={
 	.id=MS_MPEG4_ENC_ID,
 	.name="MSMpeg4Enc",
@@ -581,23 +702,8 @@ MSFilterDesc ms_snow_enc_desc={
 
 #endif
 
-#if defined(WIN32) || defined(_WIN32_WCE)
-__declspec(dllexport) PayloadType payload_type_x_snow
-#else
-PayloadType payload_type_x_snow
-#endif
-={
-	PAYLOAD_VIDEO,
-	90000,
-	0,
-	NULL,
-	0,
-	256000,
-	"x-snow"
-};
-
-
 MS_FILTER_DESC_EXPORT(ms_mpeg4_enc_desc)
 MS_FILTER_DESC_EXPORT(ms_h263_enc_desc)
+MS_FILTER_DESC_EXPORT(ms_h263_old_enc_desc)
 MS_FILTER_DESC_EXPORT(ms_snow_enc_desc)
 
