@@ -17,13 +17,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "mediastreamer-config.h"
+#endif
+
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msvideoout.h"
 
-#include <ffmpeg/avcodec.h>
-#include <ffmpeg/swscale.h>
-#include <ffmpeg/avutil.h>
+#include "ffmpeg-priv.h"
 
 static int video_out_handle_resize(MSFilter *f, void *arg);
 
@@ -44,6 +46,38 @@ static ms_mutex_t sdl_mutex;
 
 static SDL_Surface *sdl_screen=0;
 
+#ifdef HAVE_X11_XLIB_H
+
+#include <SDL/SDL_syswm.h>
+
+static void sdl_show_window(bool_t show){
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+	if ( SDL_GetWMInfo(&info) ) {
+		if ( info.subsystem == SDL_SYSWM_X11 ) {
+			Display *display;
+			Window window;
+		
+			info.info.x11.lock_func();
+			display = info.info.x11.display;
+			window = info.info.x11.wmwindow;
+			if (show)
+				XMapWindow(display,window);
+			else
+				XUnmapWindow(display,window);
+			info.info.x11.unlock_func();
+		}
+	}
+}
+
+#else
+
+static void sdl_show_window(bool_t show){
+	ms_warning("SDL window show/hide not implemented");
+}
+
+#endif
+
 static SDL_Overlay * sdl_create_window(int w, int h){
 	SDL_Overlay *lay;
 	sdl_screen = SDL_SetVideoMode(w,h, 0,SDL_SWSURFACE|SDL_RESIZABLE);
@@ -81,6 +115,7 @@ static bool_t sdl_display_init(MSDisplay *obj, MSPicture *fbuf){
 	}
 	if (obj->data!=NULL)
 		SDL_FreeYUVOverlay((SDL_Overlay*)obj->data);
+	
 	lay=sdl_create_window(fbuf->w, fbuf->h);
 	if (lay){
 		fbuf->planes[0]=lay->pixels[0];
@@ -90,6 +125,7 @@ static bool_t sdl_display_init(MSDisplay *obj, MSPicture *fbuf){
 		fbuf->strides[1]=lay->pitches[2];
 		fbuf->strides[2]=lay->pitches[1];
 		obj->data=lay;
+		sdl_show_window(TRUE);
 		return TRUE;
 	}
 	return FALSE;
@@ -165,6 +201,7 @@ static void sdl_display_uninit(MSDisplay *obj){
 		SDL_FreeSurface(sdl_screen);
 		sdl_screen=NULL;
 	}
+	sdl_show_window(FALSE);
 }
 
 MSDisplayDesc ms_sdl_display_desc={
@@ -345,7 +382,7 @@ static void yuv420p_to_rgb(MSPicture *src, uint8_t *rgb){
 			src->w,src->h,PIX_FMT_RGB24,
 			0, NULL, NULL, NULL);
 	if (sws_scale(sws,src->planes,src->strides, 0,
-           			0, &p, &rgb_stride)!=0){
+           			0, &p, &rgb_stride)<0){
 			ms_error("Error in 420->rgb sws_scale().");
 	}
 	sws_freeContext(sws);
@@ -497,54 +534,48 @@ typedef struct VideoOut
 
 static void set_corner(VideoOut *s, int corner)
 {
-  s->corner=corner;
-  s->local_pic.w=s->fbuf.w/SCALE_FACTOR;
-  s->local_pic.h=s->fbuf.h/SCALE_FACTOR;
-  if (corner==1)
-    {
-      /* top left corner */
+	s->corner=corner;
+	s->local_pic.w=s->fbuf.w/SCALE_FACTOR;
+	s->local_pic.h=s->fbuf.h/SCALE_FACTOR;
+	if (corner==1)
+	{
+	/* top left corner */
 	s->local_rect.x=0;
 	s->local_rect.y=0;
 	s->local_rect.w=s->local_pic.w;
 	s->local_rect.h=s->local_pic.h;
-    }
-  else if (corner==2)
-    {
-      /* top right corner */
+	}
+	else if (corner==2)
+	{
+	/* top right corner */
 	s->local_rect.x=s->fbuf.w-s->local_pic.w;
 	s->local_rect.y=0;
 	s->local_rect.w=s->local_pic.w;
 	s->local_rect.h=s->local_pic.h;
-    }
-  else if (corner==3)
-    {
-      /* bottom left corner */
+	}
+	else if (corner==3)
+	{
+	/* bottom left corner */
 	s->local_rect.x=0;
 	s->local_rect.y=s->fbuf.h-s->local_pic.h;
 	s->local_rect.w=s->local_pic.w;
 	s->local_rect.h=s->local_pic.h;
-    }
-  else
-    {
-      /* default: bottom right corner */
-      /* corner can be set to -1: to disable the self view... */
+	}
+	else
+	{
+	/* default: bottom right corner */
+	/* corner can be set to -1: to disable the self view... */
 	s->local_rect.x=s->fbuf.w-s->local_pic.w;
 	s->local_rect.y=s->fbuf.h-s->local_pic.h;
 	s->local_rect.w=s->local_pic.w;
 	s->local_rect.h=s->local_pic.h;
-    }
+	}
 }
 
 static void set_vsize(VideoOut *s, MSVideoSize *sz){
 	if (s->ratio.num!=0){
-#ifndef WIN32
 		sz->width=sz->width & (~0x1);
 		sz->height=sz->width*s->ratio.den/s->ratio.num;
-#else
-		int val=sz->width/s->ratio.num;
-		sz->width=val*s->ratio.num;
-		sz->height=val*s->ratio.den;
-#endif
 	}
 	s->fbuf.w=sz->width;
 	s->fbuf.h=sz->height;
@@ -582,6 +613,10 @@ static void video_out_uninit(MSFilter *f){
 		sws_freeContext(obj->sws2);
 		obj->sws2=NULL;
 	}
+	if (obj->local_msg!=NULL) {
+		freemsg(obj->local_msg);
+		obj->local_msg=NULL;
+	}
 	ms_free(obj);
 }
 
@@ -618,49 +653,52 @@ static void video_out_preprocess(MSFilter *f){
 static void video_out_postprocess(MSFilter *f){
 }
 
+
 static void video_out_process(MSFilter *f){
 	VideoOut *obj=(VideoOut*)f->data;
 	mblk_t *inm;
-	
+
+#ifdef WIN32
 	video_out_handle_resize(f, NULL);
+#endif
 
 	ms_filter_lock(f);
 	if (obj->display==NULL){
 		ms_filter_unlock(f);
-		ms_queue_flush(f->inputs[0]);
-		ms_queue_flush(f->inputs[1]);
+		if (f->inputs[0]!=NULL)
+			ms_queue_flush(f->inputs[0]);
+		if (f->inputs[1]!=NULL)
+			ms_queue_flush(f->inputs[1]);
 		return;
 	}
 	/*get most recent message and draw it*/
 	if (f->inputs[1]!=NULL && (inm=ms_queue_peek_last(f->inputs[1]))!=0) {
-
-	  if (obj->corner==-1)
-	    {
-	      if (obj->local_msg!=NULL) {
-		freemsg(obj->local_msg);
-		obj->local_msg=NULL;
-	      }
-	    }
-	  else
-	    {
-	      MSPicture src;
-	      if (yuv_buf_init_from_mblk(&src,inm)==0){
-		if (obj->sws2==NULL){
-		  obj->sws2=sws_getContext(src.w,src.h,PIX_FMT_YUV420P,
-					   obj->local_pic.w,obj->local_pic.h,PIX_FMT_YUV420P,
-					   SWS_FAST_BILINEAR, NULL, NULL, NULL);
+		if (obj->corner==-1){
+			if (obj->local_msg!=NULL) {
+				freemsg(obj->local_msg);
+				obj->local_msg=NULL;
+			}
+		}else{
+			MSPicture src;
+			if (yuv_buf_init_from_mblk(&src,inm)==0){
+			
+				if (obj->sws2==NULL){
+					obj->sws2=sws_getContext(src.w,src.h,PIX_FMT_YUV420P,
+								obj->local_pic.w,obj->local_pic.h,PIX_FMT_YUV420P,
+								SWS_FAST_BILINEAR, NULL, NULL, NULL);
+				}
+				if (obj->local_msg==NULL){
+					obj->local_msg=yuv_buf_alloc(&obj->local_pic,
+						obj->local_pic.w,obj->local_pic.h);
+				}
+				if (sws_scale(obj->sws2,src.planes,src.strides, 0,
+					src.h, obj->local_pic.planes, obj->local_pic.strides)<0){
+					ms_error("Error in sws_scale().");
+				}
+				if (!mblk_get_precious_flag(inm)) yuv_buf_mirror(&obj->local_pic);
+			}
 		}
-		if (obj->local_msg==NULL){
-		  obj->local_msg=yuv_buf_alloc(&obj->local_pic,
-					       obj->local_pic.w,obj->local_pic.h);
-		}
-		if (sws_scale(obj->sws2,src.planes,src.strides, 0,
-			      0, obj->local_pic.planes, obj->local_pic.strides)!=0){
-		  ms_error("Error in sws_scale().");
-		}
-	      }
-	    }
-	  ms_queue_flush(f->inputs[1]);
+		ms_queue_flush(f->inputs[1]);
 	}
 	
 	if (f->inputs[0]!=NULL && (inm=ms_queue_peek_last(f->inputs[0]))!=0) {
@@ -673,7 +711,7 @@ static void video_out_process(MSFilter *f){
 			}
 			ms_display_lock(obj->display);
 			if (sws_scale(obj->sws1,src.planes,src.strides, 0,
-            			0, obj->fbuf.planes, obj->fbuf.strides)!=0){
+            			src.h, obj->fbuf.planes, obj->fbuf.strides)<0){
 				ms_error("Error in sws_scale().");
 			}
 			ms_display_unlock(obj->display);
@@ -740,6 +778,22 @@ static int video_out_handle_resize(MSFilter *f, void *arg){
 static int video_out_set_corner(MSFilter *f,void *arg){
 	VideoOut *s=(VideoOut*)f->data;
 	set_corner(s, *(int*)arg);
+#if 1
+	ms_filter_lock(f);
+	ms_display_lock(s->display);
+	{
+	  int w=s->fbuf.w;
+	  int h=s->fbuf.h;
+	  int ysize=w*h;
+	  int usize=ysize/4;
+	  
+	  memset(s->fbuf.planes[0], 0, ysize);
+	  memset(s->fbuf.planes[1], 0, usize);
+	  memset(s->fbuf.planes[2], 0, usize);
+	}
+	ms_display_unlock(s->display);
+	ms_filter_unlock(f);
+#endif
 	return 0;
 }
 
