@@ -21,7 +21,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msrtp.h"
-#include "mediastreamer2/msv4l.h"
+
+
 #if (defined(WIN32) || defined(_WIN32_WCE))
 /* avoid double declaration of ms_win_display_desc */
 #define MS_VIDEO_OUT_HANDLE_RESIZING MS_FILTER_METHOD_NO_ARG(MS_VIDEO_OUT_ID,1)
@@ -40,8 +41,10 @@ extern RtpSession * create_duplex_rtpsession( int locport, bool_t ipv6);
 /* this code is not part of the library itself, it is part of the mediastream program */
 void video_stream_free (VideoStream * stream)
 {
-	if (stream->session!=NULL)
+	if (stream->session!=NULL){
+		rtp_session_unregister_event_queue(stream->session,stream->evq);
 		rtp_session_destroy(stream->session);
+	}
 	if (stream->rtprecv != NULL)
 		ms_filter_destroy (stream->rtprecv);
 	if (stream->rtpsend!=NULL) 
@@ -52,14 +55,16 @@ void video_stream_free (VideoStream * stream)
 		ms_filter_destroy (stream->output);
 	if (stream->decoder != NULL)
 		ms_filter_destroy (stream->decoder);
-	if (stream->encoder != NULL)
-		ms_filter_destroy (stream->encoder);
+	if (stream->sizeconv != NULL)
+		ms_filter_destroy (stream->sizeconv);
 	if (stream->pixconv!=NULL)
 		ms_filter_destroy(stream->pixconv);
 	if (stream->tee!=NULL)
 		ms_filter_destroy(stream->tee);
 	if (stream->ticker != NULL)
 		ms_ticker_destroy (stream->ticker);
+	if (stream->evq!=NULL)
+		ortp_ev_queue_destroy(stream->evq);
 	ms_free (stream);
 }
 
@@ -95,10 +100,62 @@ void video_stream_change_decoder(VideoStream *stream, int payload){
 	}
 }
 
+static void video_stream_adapt_bitrate(VideoStream *stream, int jitter, float lost){
+	if (stream->encoder!=NULL){
+		if (lost>10){
+			int bitrate=0;
+			int new_bitrate;
+			ms_warning("Remote reports bad receiving experience, trying to reduce bitrate of video encoder.");
+			
+			ms_filter_call_method(stream->encoder,MS_FILTER_GET_BITRATE,&bitrate);
+			if (bitrate==0){
+				ms_error("Video encoder does not implement MS_FILTER_GET_BITRATE.");
+				return;
+			}
+			if (bitrate>=20000){
+				new_bitrate=bitrate-10000;
+				ms_warning("Encoder bitrate reduced from %i to %i b/s.",bitrate,new_bitrate);
+				ms_filter_call_method(stream->encoder,MS_FILTER_SET_BITRATE,&new_bitrate);
+			}else{
+				ms_warning("Video encoder bitrate already at minimum.");
+			}
+			
+		}
+	}
+}
+
+static void video_steam_process_rtcp(VideoStream *stream, mblk_t *m){
+	do{
+		if (rtcp_is_SR(m)){
+			const report_block_t *rb;
+			ms_message("video_steam_process_rtcp: receiving RTCP SR");
+			rb=rtcp_SR_get_report_block(m,0);
+			if (rb){
+				unsigned int ij;
+				float flost;
+				ij=report_block_get_interarrival_jitter(rb);
+				flost=100.0*report_block_get_fraction_lost(rb)/256.0;
+				ms_message("interarrival jitter=%u , lost packets percentage since last report=%f ",ij,flost);
+				if (stream->adapt_bitrate) video_stream_adapt_bitrate(stream,ij,flost);
+			}
+		}
+	}while(rtcp_next_packet(m));
+}
+
 void video_stream_iterate(VideoStream *stream){
 	if (stream->output!=NULL)
 		ms_filter_call_method_noarg(stream->output,
 			MS_VIDEO_OUT_HANDLE_RESIZING);
+	if (stream->evq){
+		OrtpEvent *ev=ortp_ev_queue_get(stream->evq);
+		if (ev!=NULL){
+			if (ortp_event_get_type(ev)==ORTP_EVENT_RTCP_PACKET_RECEIVED){
+				OrtpEventData *evd=ortp_event_get_data(ev);
+				video_steam_process_rtcp(stream,evd->packet);
+			}
+			ortp_event_destroy(ev);
+		}
+	}
 }
 
 static void payload_type_changed(RtpSession *session, unsigned long data){
@@ -110,10 +167,23 @@ static void payload_type_changed(RtpSession *session, unsigned long data){
 VideoStream *video_stream_new(int locport, bool_t use_ipv6){
 	VideoStream *stream = (VideoStream *)ms_new0 (VideoStream, 1);
 	stream->session=create_duplex_rtpsession(locport,use_ipv6);
+	stream->evq=ortp_ev_queue_new();
+	stream->rtpsend=ms_filter_new(MS_RTP_SEND_ID);
+	rtp_session_register_event_queue(stream->session,stream->evq);
 	return stream;
 }
 
-int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,int payload, int jitt_comp,  const char *device)
+void video_stream_set_relay_session_id(VideoStream *stream, const char *id){
+	ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
+}
+
+
+void video_stream_enable_adaptive_bitrate_control(VideoStream *s, bool_t yesno){
+	s->adapt_bitrate=yesno;
+}
+
+int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,
+	int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *cam)
 {
 	PayloadType *pt;
 	RtpSession *rtps=stream->session;
@@ -138,27 +208,26 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	}
 	
 	rtp_session_set_profile(rtps,profile);
-	if (remport>0) rtp_session_set_remote_addr(rtps,remip,remport);
+	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
 	rtp_session_set_payload_type(rtps,payload);
 	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 
 	rtp_session_signal_connect(stream->session,"payload_type_changed",
 			(RtpCallback)payload_type_changed,(unsigned long)stream);
 
-	/* creates two rtp filters to recv send streams (remote part) */
 	rtp_session_set_recv_buf_size(stream->session,MAX_RTP_SIZE);
-	stream->rtpsend =ms_filter_new(MS_RTP_SEND_ID);
+
+	/* creates two rtp filters to recv send streams (remote part) */
 	if (remport>0) ms_filter_call_method(stream->rtpsend,MS_RTP_SEND_SET_SESSION,stream->session);
 	
 	stream->rtprecv = ms_filter_new (MS_RTP_RECV_ID);
 	ms_filter_call_method(stream->rtprecv,MS_RTP_RECV_SET_SESSION,stream->session);
 
 	/* creates the filters */
-	stream->source = ms_filter_new(MS_V4L_ID);
+	stream->source = ms_web_cam_create_reader(cam);
 	stream->tee = ms_filter_new(MS_TEE_ID);
 	stream->output=ms_filter_new(MS_VIDEO_OUT_ID);
-
-	
+	stream->sizeconv=ms_filter_new(MS_SIZE_CONV_ID);
 	
 	if (pt->normal_bitrate>0){
 		ms_message("Limiting bitrate of video encoder to %i bits/s",pt->normal_bitrate);
@@ -175,7 +244,7 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 	/* configure the filters */
 	ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
-	ms_filter_call_method_noarg(stream->source,MS_V4L_START);
+
 	/* get the output format for webcam reader */
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
 	if (format==MS_MJPEG){
@@ -184,13 +253,22 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 		stream->pixconv = ms_filter_new(MS_PIX_CONV_ID);
 		/*set it to the pixconv */
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
+
+		ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+	
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+		  
 	}
+
+	ms_filter_call_method(stream->encoder,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+
 	/*force the decoder to output YUV420P */
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->decoder,MS_FILTER_SET_PIX_FMT,&format);
-	/*ask the size-converter to always output CIF */
 
+
+	/*ask the video display to always output CIF */
 	vsize.height=MS_VIDEO_SIZE_CIF_H;
 	vsize.width=MS_VIDEO_SIZE_CIF_W;
 
@@ -202,7 +280,8 @@ int video_stream_start (VideoStream *stream, RtpProfile *profile, const char *re
 
 	/* and then connect all */
 	ms_filter_link (stream->source, 0, stream->pixconv, 0);
-	ms_filter_link (stream->pixconv, 0, stream->tee, 0);
+	ms_filter_link (stream->pixconv, 0, stream->sizeconv, 0);
+	ms_filter_link (stream->sizeconv, 0, stream->tee, 0);
 	ms_filter_link (stream->tee, 0 ,stream->encoder, 0 );
 	ms_filter_link (stream->encoder,0, stream->rtpsend,0);
 	
@@ -228,12 +307,12 @@ video_stream_stop (VideoStream * stream)
 {
 	if (stream->ticker){
 		ms_ticker_detach(stream->ticker,stream->source);
-		ms_filter_call_method_noarg(stream->source,MS_V4L_STOP);
 	
 		rtp_stats_display(rtp_session_get_stats(stream->session),"Video session's RTP statistics");
 		
 		ms_filter_unlink(stream->source,0,stream->pixconv,0);
-		ms_filter_unlink(stream->pixconv,0,stream->tee,0);
+		ms_filter_unlink (stream->pixconv, 0, stream->sizeconv, 0);
+		ms_filter_unlink (stream->sizeconv, 0, stream->tee, 0);
 		ms_filter_unlink(stream->tee,0,stream->encoder,0);
 		ms_filter_unlink(stream->encoder, 0, stream->rtpsend,0);
 		ms_filter_unlink(stream->rtprecv, 0, stream->decoder, 0);
@@ -253,7 +332,7 @@ void video_stream_set_rtcp_information(VideoStream *st, const char *cname, const
 
 
 
-VideoStream * video_preview_start(const char *device){
+VideoStream * video_preview_start(MSWebCam *device){
 	VideoStream *stream = (VideoStream *)ms_new0 (VideoStream, 1);
 	MSPixFmt format;
 	MSVideoSize vsize;
@@ -261,11 +340,11 @@ VideoStream * video_preview_start(const char *device){
 	vsize.height=MS_VIDEO_SIZE_CIF_H;
 
 	/* creates the filters */
-	stream->source = ms_filter_new(MS_V4L_ID);
+	stream->source = ms_web_cam_create_reader(device);
 	stream->output = ms_filter_new(MS_VIDEO_OUT_ID);
 
+
 	/* configure the filters */
-	ms_filter_call_method_noarg(stream->source,MS_V4L_START);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
 	ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
 	
@@ -276,12 +355,15 @@ VideoStream * video_preview_start(const char *device){
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
 		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	}
+
 	format=MS_YUV420P;
 	ms_filter_call_method(stream->output,MS_FILTER_SET_PIX_FMT,&format);
 	ms_filter_call_method(stream->output,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	/* and then connect all */
+
 	ms_filter_link(stream->source,0, stream->pixconv,0);
-	ms_filter_link(stream->pixconv,0,stream->output,0);
+	ms_filter_link(stream->pixconv, 0, stream->output, 0);
+
 	/* create the ticker */
 	stream->ticker = ms_ticker_new(); 
 	ms_ticker_attach (stream->ticker, stream->source);
@@ -290,7 +372,6 @@ VideoStream * video_preview_start(const char *device){
 
 void video_preview_stop(VideoStream *stream){
 	ms_ticker_detach(stream->ticker, stream->source);
-	ms_filter_call_method_noarg(stream->source,MS_V4L_STOP);
 	ms_filter_unlink(stream->source,0,stream->pixconv,0);
 	ms_filter_unlink(stream->pixconv,0,stream->output,0);
 	
@@ -298,7 +379,8 @@ void video_preview_stop(VideoStream *stream){
 }
 
 
-int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const char *remip, int remport, int payload, int jitt_comp, const char *device){
+int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const char *remip, int remport,
+	int rem_rtcp_port, int payload, int jitt_comp, MSWebCam *device){
 	PayloadType *pt;
 	MSPixFmt format;
 	MSVideoSize vsize;
@@ -309,7 +391,7 @@ int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const
 	vsize.height=MS_VIDEO_SIZE_CIF_H;
 
 	rtp_session_set_profile(rtps,profile);
-	if (remport>0) rtp_session_set_remote_addr(rtps,remip,remport);
+	if (remport>0) rtp_session_set_remote_addr_full(rtps,remip,remport,rem_rtcp_port);
 	rtp_session_set_payload_type(rtps,payload);
 	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 	
@@ -334,9 +416,8 @@ int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const
 	}
 
 	/* creates the filters */
-	stream->source = ms_filter_new(MS_V4L_ID);
-	stream->pixconv= ms_filter_new(MS_PIX_CONV_ID);
-	
+	stream->source = ms_web_cam_create_reader(device);
+	stream->sizeconv=ms_filter_new(MS_SIZE_CONV_ID);
 
 	/* configure the filters */
 	if (pt->send_fmtp)
@@ -347,19 +428,32 @@ int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const
 
 	ms_filter_call_method(stream->source,MS_FILTER_SET_FPS,&fps);
 	ms_filter_call_method(stream->source,MS_FILTER_SET_VIDEO_SIZE,&vsize);
-	ms_filter_call_method_noarg(stream->source,MS_V4L_START);
+	
 	/* get the output format for webcam reader */
 	ms_filter_call_method(stream->source,MS_FILTER_GET_PIX_FMT,&format);
 	/*set it to the pixconv */
-	ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
-	ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+
+	/* bug fix from AMD: What about MJPEG mode???*/
+	if (format==MS_MJPEG){
+		stream->pixconv=ms_filter_new(MS_MJPEG_DEC_ID);
+	}else{
+		stream->pixconv=ms_filter_new(MS_PIX_CONV_ID);
+		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_PIX_FMT,&format);
+
+		ms_filter_call_method(stream->source,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+		ms_filter_call_method(stream->pixconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
+	}
+
+	ms_filter_call_method(stream->encoder,MS_FILTER_GET_VIDEO_SIZE,&vsize);
+	ms_filter_call_method(stream->sizeconv,MS_FILTER_SET_VIDEO_SIZE,&vsize);
 	
 	ms_message("vsize=%ix%i, fps=%f, send format: %s, capture format: %d, bitrate: %d",
 			vsize.width,vsize.height,fps,pt->send_fmtp,format, pt->normal_bitrate);
 
 	/* and then connect all */
 	ms_filter_link (stream->source, 0, stream->pixconv, 0);
-	ms_filter_link (stream->pixconv, 0, stream->encoder, 0);
+	ms_filter_link (stream->pixconv, 0, stream->sizeconv, 0);
+	ms_filter_link (stream->sizeconv, 0, stream->encoder, 0);
 	ms_filter_link (stream->encoder,0, stream->rtpsend,0);
 
 	/* create the ticker */
@@ -372,15 +466,15 @@ int video_stream_send_only_start(VideoStream* stream, RtpProfile *profile, const
 void video_stream_send_only_stop(VideoStream *stream){
 	if (stream->ticker){
 		ms_ticker_detach (stream->ticker, stream->source);
-		ms_filter_call_method_noarg(stream->source,MS_V4L_STOP);
 		ms_filter_unlink(stream->source,0,stream->pixconv,0);
-		ms_filter_unlink(stream->pixconv,0,stream->encoder,0);
+		ms_filter_unlink (stream->pixconv, 0, stream->sizeconv, 0);
+		ms_filter_unlink (stream->sizeconv, 0, stream->encoder, 0);
 		ms_filter_unlink(stream->encoder,0,stream->rtpsend,0);
 	}
 	video_stream_free(stream);
 }
 
-int video_stream_recv_only_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,int payload, int jitt_comp,  const char *device){
+int video_stream_recv_only_start (VideoStream *stream, RtpProfile *profile, const char *remip, int remport,int payload, int jitt_comp){
 	PayloadType *pt;
 	MSPixFmt format;
 	MSVideoSize vsize;
