@@ -27,8 +27,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 #include <msacm.h>
 
+#if defined(_WIN32_WCE)
+//#define DISABLE_SPEEX
+//#define WCE_OPTICON_WORKAROUND 1000
+#endif
+#ifndef DISABLE_SPEEX
+#include <speex/speex_preprocess.h>
+#endif
+
 #define WINSND_NBUFS 10
-#define WINSND_NSAMPLES 160
+#define WINSND_OUT_NBUFS 20
+#define WINSND_NSAMPLES 320
 #define WINSND_MINIMUMBUFFER 5
 
 static MSFilter *ms_winsnd_read_new(MSSndCard *card);
@@ -157,19 +166,41 @@ static void add_or_update_card(MSSndCardManager *m, const char *name, int indev,
 
 static void winsndcard_detect(MSSndCardManager *m){
     MMRESULT mr = NOERROR;
+    unsigned int nOutDevices = waveOutGetNumDevs ();
     unsigned int nInDevices = waveInGetNumDevs ();
     unsigned int item;
 
+    if (nOutDevices>nInDevices)
+		nInDevices = nOutDevices;
+
     for (item = 0; item < nInDevices; item++){
 		
         WAVEINCAPS incaps;
         WAVEOUTCAPS outcaps;
         mr = waveInGetDevCaps (item, &incaps, sizeof (WAVEINCAPS));
         if (mr == MMSYSERR_NOERROR)
-    		add_or_update_card(m,incaps.szPname,item,-1,MS_SND_CARD_CAP_CAPTURE);
+		{
+#if defined(_WIN32_WCE)
+			char card[256];
+			snprintf(card, sizeof(card), "Input card %i", item);
+			add_or_update_card(m,card,item,-1,MS_SND_CARD_CAP_CAPTURE);
+			/* _tprintf(L"new card: %s", incaps.szPname); */
+#else
+			add_or_update_card(m,incaps.szPname,item,-1,MS_SND_CARD_CAP_CAPTURE);
+#endif
+		}
     	mr = waveOutGetDevCaps (item, &outcaps, sizeof (WAVEOUTCAPS));
         if (mr == MMSYSERR_NOERROR)
+		{
+#if defined(_WIN32_WCE)
+			char card[256];
+			snprintf(card, sizeof(card), "Output card %i", item);
+    		add_or_update_card(m,card,-1,item,MS_SND_CARD_CAP_PLAYBACK);
+			/* _tprintf(L"new card: %s", outcaps.szPname); */
+#else
     		add_or_update_card(m,outcaps.szPname,-1,item,MS_SND_CARD_CAP_PLAYBACK);
+#endif
+		}
     }
 }
 
@@ -179,7 +210,8 @@ typedef struct WinSnd{
 	HWAVEIN indev;
 	HWAVEOUT outdev;
 	WAVEFORMATEX wfx;
-	WAVEHDR hdrs[WINSND_NBUFS*2];
+	WAVEHDR hdrs_read[WINSND_NBUFS];
+	WAVEHDR hdrs_write[WINSND_OUT_NBUFS];
 	queue_t rq;
 	ms_mutex_t mutex;
 	unsigned int bytes_read;
@@ -191,6 +223,14 @@ typedef struct WinSnd{
 	int32_t stat_notplayed;
 
 	int32_t stat_minimumbuffer;
+
+	queue_t write_rq;
+#ifndef DISABLE_SPEEX
+	SpeexPreprocessState *pst;
+	int pst_frame_size;
+#endif
+	int ready;
+	int workaround; /* workaround for opticon audio device */
 }WinSnd;
 
 static void winsnd_apply_settings(WinSnd *d){
@@ -198,12 +238,15 @@ static void winsnd_apply_settings(WinSnd *d){
 	d->wfx.nAvgBytesPerSec=d->wfx.nSamplesPerSec*d->wfx.nBlockAlign;
 }
 
+
+#ifndef _TRUE_TIME
 static uint64_t winsnd_get_cur_time( void *data){
 	WinSnd *d=(WinSnd*)data;
 	uint64_t curtime=((uint64_t)d->bytes_read*1000)/(uint64_t)d->wfx.nAvgBytesPerSec;
 	/* ms_debug("winsnd_get_cur_time: bytes_read=%u return %lu\n",d->bytes_read,(unsigned long)curtime); */
 	return curtime;
 }
+#endif
 
 
 static void winsnd_init(MSFilter *f){
@@ -216,6 +259,13 @@ static void winsnd_init(MSFilter *f){
 	d->wfx.nSamplesPerSec = 8000;
 	d->wfx.wBitsPerSample = 16;
 	qinit(&d->rq);
+	qinit(&d->write_rq);
+#ifndef DISABLE_SPEEX
+	d->pst=NULL;
+	d->pst_frame_size=0;
+#endif
+	d->ready=0;
+	d->workaround=0;
 	ms_mutex_init(&d->mutex,NULL);
 	f->data=d;
 
@@ -228,6 +278,15 @@ static void winsnd_init(MSFilter *f){
 static void winsnd_uninit(MSFilter *f){
 	WinSnd *d=(WinSnd*)f->data;
 	flushq(&d->rq,0);
+	flushq(&d->write_rq,0);
+#ifndef DISABLE_SPEEX
+	if (d->pst!=NULL)
+	    speex_preprocess_state_destroy(d->pst);
+	d->pst=NULL;
+	d->pst_frame_size=0;
+#endif
+	d->ready=0;
+	d->workaround=0;
 	ms_mutex_destroy(&d->mutex);
 	ms_free(f->data);
 }
@@ -343,7 +402,7 @@ static void winsnd_read_preprocess(MSFilter *f){
 	bsize=WINSND_NSAMPLES*d->wfx.nAvgBytesPerSec/8000;
 	ms_debug("Using input buffers of %i bytes",bsize);
 	for(i=0;i<WINSND_NBUFS;++i){
-		WAVEHDR *hdr=&d->hdrs[i];
+		WAVEHDR *hdr=&d->hdrs_read[i];
 		add_input_buffer(d,hdr,bsize);
 	}
 	d->running=TRUE;
@@ -352,14 +411,18 @@ static void winsnd_read_preprocess(MSFilter *f){
 		ms_error("waveInStart() error");
 		return ;
 	}
+#ifndef _TRUE_TIME
 	ms_ticker_set_time_func(f->ticker,winsnd_get_cur_time,d);
+#endif
 }
 
 static void winsnd_read_postprocess(MSFilter *f){
 	WinSnd *d=(WinSnd*)f->data;
 	MMRESULT mr;
 	int i;
+#ifndef _TRUE_TIME
 	ms_ticker_set_time_func(f->ticker,NULL,NULL);
+#endif
 	d->running=FALSE;
 	mr=waveInStop(d->indev);
 	if (mr != MMSYSERR_NOERROR){
@@ -372,7 +435,7 @@ static void winsnd_read_postprocess(MSFilter *f){
 		return ;
 	}
 	for(i=0;i<WINSND_NBUFS;++i){
-		WAVEHDR *hdr=&d->hdrs[i];
+		WAVEHDR *hdr=&d->hdrs_read[i];
 		if (hdr->dwFlags & WHDR_PREPARED)
 		{
 			mr = waveInUnprepareHeader(d->indev,hdr,sizeof (*hdr));
@@ -401,7 +464,7 @@ static void winsnd_read_process(MSFilter *f){
 	}
 	ms_mutex_unlock(&d->mutex);
 	for(i=0;i<WINSND_NBUFS;++i){
-		WAVEHDR *hdr=&d->hdrs[i];
+		WAVEHDR *hdr=&d->hdrs_read[i];
 		if (hdr->dwUser==0) {
 			MMRESULT mr;
 			mr=waveInUnprepareHeader(d->indev,hdr,sizeof(*hdr));
@@ -467,8 +530,8 @@ static void winsnd_write_preprocess(MSFilter *f){
 			return ;
 		}
 	}
-	for(i=0;i<WINSND_NBUFS*2;++i){
-		WAVEHDR *hdr=&d->hdrs[i];
+	for(i=0;i<WINSND_OUT_NBUFS;++i){
+		WAVEHDR *hdr=&d->hdrs_write[i];
 		hdr->dwFlags=0;
 		hdr->dwUser=0;
 	}
@@ -484,8 +547,8 @@ static void winsnd_write_postprocess(MSFilter *f){
 		ms_error("waveOutReset() error");
 		return ;
 	}
-	for(i=0;i<WINSND_NBUFS*2;++i){
-		WAVEHDR *hdr=&d->hdrs[i];
+	for(i=0;i<WINSND_OUT_NBUFS;++i){
+		WAVEHDR *hdr=&d->hdrs_write[i];
 		mblk_t *old;
 		if (hdr->dwFlags & WHDR_DONE){
 			mr=waveOutUnprepareHeader(d->outdev,hdr,sizeof(*hdr));
@@ -502,7 +565,17 @@ static void winsnd_write_postprocess(MSFilter *f){
 		ms_error("waveOutClose() error");
 		return ;
 	}
-	ms_message("Shutting down sound device (playing: %i) (input-output: %i) (notplayed: %i)", d->nbufs_playing, d->stat_input - d->stat_output, d->stat_notplayed);
+	ms_message("Shutting down sound device (playing: %i) (d->write_rq.q_mcount=%i) (input-output: %i) (notplayed: %i)", d->nbufs_playing, d->write_rq.q_mcount, d->stat_input - d->stat_output, d->stat_notplayed);
+	flushq(&d->write_rq,0);
+	d->ready=0;
+	d->workaround=0;
+
+#ifndef DISABLE_SPEEX
+	if (d->pst!=NULL)
+	    speex_preprocess_state_destroy(d->pst);
+	d->pst=NULL;
+	d->pst_frame_size=0;
+#endif
 }
 
 static void winsnd_write_process(MSFilter *f){
@@ -511,23 +584,119 @@ static void winsnd_write_process(MSFilter *f){
 	MMRESULT mr;
 	int i;
 	int discarded=0;
+	int possible_size=0;
 
 	if (d->outdev==NULL) {
 		ms_queue_flush(f->inputs[0]);
 		return;
 	}
 
+	while((m=ms_queue_get(f->inputs[0]))!=NULL){
+		possible_size = msgdsize(m);
+#ifndef DISABLE_SPEEX
+		if (d->pst_frame_size==0)
+		{
+			d->pst_frame_size=possible_size;
+
+			d->pst = speex_preprocess_state_init(d->pst_frame_size/2, d->wfx.nSamplesPerSec);
+			if (d->pst!=NULL) {
+				float f;
+				i=1;
+				speex_preprocess_ctl(d->pst, SPEEX_PREPROCESS_SET_VAD, &i);
+				i=0;
+				speex_preprocess_ctl(d->pst, SPEEX_PREPROCESS_SET_DENOISE, &i);
+				i=0;
+				speex_preprocess_ctl(d->pst, SPEEX_PREPROCESS_SET_AGC, &i);
+				f=8000;
+				speex_preprocess_ctl(d->pst, SPEEX_PREPROCESS_SET_AGC_LEVEL, &f);
+				i=0;
+				speex_preprocess_ctl(d->pst, SPEEX_PREPROCESS_SET_DEREVERB, &i);
+			}
+		}
+#endif
+
+		putq(&d->write_rq,m);
+	}
+
 #ifdef AMD_HACK
 	/* too many sound card are crappy on windows... */
 	d->stat_minimumbuffer=15;
+  if (d->wfx.nSamplesPerSec>=32000) /* better results for high rates */
+  	d->stat_minimumbuffer=8;
 #endif
 
-	if (d->nbufs_playing==0)
-		ms_warning("no buffer currently playing (q.m_count=%i), (playing: %i)", f->inputs[0]->q.q_mcount,  d->nbufs_playing);
+  if (d->wfx.nSamplesPerSec>=32000) /* better results for high rates */
+  {
+	  if (d->nbufs_playing+d->write_rq.q_mcount<4)
+	  {
+		  d->ready=0;
+	  }
+  }
+  else
+  {
+	  if (d->nbufs_playing+d->write_rq.q_mcount<7)
+	  {
+		  d->ready=0;
+	  }
+  }
+#if defined(WCE_OPTICON_WORKAROUND)
+	if (d->workaround==0)
+	{
+		d->workaround=1;
+		Sleep(WCE_OPTICON_WORKAROUND);
+	}
+#endif
+
+	while((m=peekq(&d->write_rq))!=NULL){
+
+#ifndef DISABLE_SPEEX
+		int vad=1;
+		if (d->pst!=NULL && msgdsize(m)==d->pst_frame_size && d->pst_frame_size<=4096)
+		{
+			char tmp[4096];
+			memcpy(tmp, m->b_rptr, msgdsize(m));
+			vad = speex_preprocess(d->pst, (short*)tmp, NULL);
+
+			if (d->ready==0)
+			{
+				if (vad==0)
+				{
+					int missing;
+          missing = 10 - d->write_rq.q_mcount - d->nbufs_playing;
+          if (d->wfx.nSamplesPerSec>=32000) /* better results for high rates */
+            missing = 6 - d->write_rq.q_mcount - d->nbufs_playing;
+
+		      ms_message("WINSND trouble: inserting %i silence", missing);
+					while(missing>0)
+					{
+						old=dupb(m);
+						putq(&d->write_rq,old);
+						missing--;
+					}
+				}
+				d->ready=1;
+			}
+		}
+#else
+		if (d->ready==0)
+		{
+      int missing;
+			missing = 10 - d->write_rq.q_mcount - d->nbufs_playing;
+      if (d->wfx.nSamplesPerSec>=32000) /* better results for high rates */
+  			missing = 6 - d->write_rq.q_mcount - d->nbufs_playing;
+			ms_message("WINSND trouble: inserting %i silence", missing);
+			while(missing>0)
+			{
+				old=dupb(m);
+				putq(&d->write_rq,old);
+				missing--;
+			}
+			d->ready=1;
+		}
+#endif
 
-	while((m=ms_queue_get(f->inputs[0]))!=NULL){
 		for(i=0;i<d->stat_minimumbuffer;++i){
-			WAVEHDR *hdr=&d->hdrs[i];
+			WAVEHDR *hdr=&d->hdrs_write[i];
 			if (hdr->dwFlags & WHDR_DONE){
 				old=(mblk_t*)hdr->dwUser;
 				mr=waveOutUnprepareHeader(d->outdev,hdr,sizeof(*hdr));
@@ -545,20 +714,22 @@ static void winsnd_write_process(MSFilter *f){
 			    mr = waveOutPrepareHeader(d->outdev,hdr,sizeof(*hdr));
 			    if (mr != MMSYSERR_NOERROR){
 					ms_error("waveOutPrepareHeader() error");
+					getq(&d->write_rq);
 					freemsg(m);
 					discarded++;
 					d->stat_notplayed++;
 					break;
 				}
-				ms_debug("waveOutPrepareHeader() done");
 				mr=waveOutWrite(d->outdev,hdr,sizeof(*hdr));
 				if (mr != MMSYSERR_NOERROR){
 					ms_error("waveOutWrite() error");
+					getq(&d->write_rq);
 					freemsg(m);
 					discarded++;
 					d->stat_notplayed++;
 					break;
 				}else {
+					getq(&d->write_rq);
 					d->nbufs_playing++;
 					/* ms_debug("waveOutWrite() done"); */
 				}
@@ -566,32 +737,37 @@ static void winsnd_write_process(MSFilter *f){
 			}
 		}
 		if (i==d->stat_minimumbuffer){
-			ms_error("winsnd_write_process: All buffers are busy.");
+			//ms_error("winsnd_write_process: All buffers are busy.");
+#ifndef DISABLE_SPEEX
+			if (d->pst==NULL)
+			{
+				/* initial behavior (detection in process?) */
+				getq(&d->write_rq);
+				freemsg(m);
+		    discarded++;
+		    d->stat_notplayed++;
+			}
+			else
+			{
+				if (vad==0)
+				{
+					getq(&d->write_rq);
+					freemsg(m);
+          ms_message("WINSND trouble: silence removed");
+			    discarded++;
+			    d->stat_notplayed++;
+				}
+			}
+#else
+			getq(&d->write_rq);
 			freemsg(m);
 			discarded++;
 			d->stat_notplayed++;
+#endif
+
+			break;
 		}
 	}
-
-	if ( d->stat_output > 150
-		&& ((d->stat_notplayed*100)/d->stat_output) >10
-		&& d->stat_minimumbuffer<15)
-	{
-		d->stat_minimumbuffer+=6;
-		ms_message("Increasing sound card number of write buffer to %i", d->stat_minimumbuffer);
-		d->stat_notplayed = 0; /* reset number... */
-	}
-	else if ( d->stat_output > 150
-		&& ((d->stat_notplayed*100)/d->stat_output) >1
-		&& d->stat_minimumbuffer<18)
-	{
-		d->stat_minimumbuffer+=3;
-		ms_message("Increasing sound card number of write buffer to %i", d->stat_minimumbuffer);
-		d->stat_notplayed = 0; /* reset number... */
-	}
-
-	if (discarded>0)
-		ms_warning("Extra data for sound card removed (%i buf), (playing: %i) (input-output: %i)", discarded, d->nbufs_playing, d->stat_input - d->stat_output);
 }
 
 static int set_rate(MSFilter *f, void *arg){
