@@ -24,6 +24,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msvideo.h"
 #include "mediastreamer2/msticker.h"
 #include "mediastreamer2/msv4l.h"
+#include "mediastreamer2/mswebcam.h"
 
 #include "nowebcam.h"
 #include <ffmpeg/avcodec.h>
@@ -36,8 +37,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <initguid.h>
 #include "dxfilter.h"
 #include <qedit.h>
-#include <atlbase.h>
-#include <atlcom.h>
+//#include <atlbase.h>
+//#include <atlcom.h>
 
 HRESULT AddGraphToRot(IUnknown *pUnkGraph, DWORD *pdwRegister);
 void RemoveGraphFromRot(DWORD pdwRegister);
@@ -47,18 +48,25 @@ typedef struct V4wState{
 	char dev[512];
 	int devidx;
 
-	CComPtr<IGraphBuilder> m_pGraph;
-	CComPtr<ICaptureGraphBuilder2> m_pBuilder;
-	CComPtr<IMediaControl> m_pControl;
+	//CComPtr<IGraphBuilder> m_pGraph;
+	//CComPtr<ICaptureGraphBuilder2> m_pBuilder;
+	//CComPtr<IMediaControl> m_pControl;
+	IGraphBuilder *m_pGraph;
+	ICaptureGraphBuilder2 *m_pBuilder;
+	IMediaControl *m_pControl;
 	CDXFilter *m_pDXFilter;
-	CComPtr<IBaseFilter> m_pIDXFilter;	
-	CComPtr<IBaseFilter> m_pNullRenderer;
-	CComPtr<IBaseFilter> m_pDeviceFilter;
+	//CComPtr<IBaseFilter> m_pIDXFilter;	
+	//CComPtr<IBaseFilter> m_pNullRenderer;
+	//CComPtr<IBaseFilter> m_pDeviceFilter;
+	IBaseFilter *m_pIDXFilter;	
+	IBaseFilter *m_pNullRenderer;
+	IBaseFilter *m_pDeviceFilter;
 	DWORD rotregvalue;
 
 	MSVideoSize vsize;
 	int pix_fmt;
 	mblk_t *mire[10];
+	char nowebcamimage[256];
 	queue_t rq;
 	ms_mutex_t mutex;
 	int frame_ind;
@@ -142,7 +150,28 @@ HRESULT ( Callback)(IMediaSample* pSample, REFERENCE_TIME* sTime, REFERENCE_TIME
 	return S_OK;
 }
 
-int try_format(V4wState *s, int format)
+HRESULT GetPinCategory(IPin *pPin, GUID *pPinCategory)
+{
+    HRESULT hr;
+    IKsPropertySet *pKs;
+    hr = pPin->QueryInterface(IID_IKsPropertySet, (void **)&pKs);
+    if (FAILED(hr))
+    {
+        // The pin does not support IKsPropertySet.
+        return hr;
+    }
+    // Try to retrieve the pin category.
+    DWORD cbReturned;
+    hr = pKs->Get(AMPROPSETID_Pin, AMPROPERTY_PIN_CATEGORY, NULL, 0, 
+        pPinCategory, sizeof(GUID), &cbReturned);
+
+    // If this succeeded, pPinCategory now contains the category GUID.
+
+    pKs->Release();
+    return hr;
+}
+
+int try_format(V4wState *s, int format, GUID *pPinCategory)
 {
     HRESULT hr=S_OK;
     IEnumPins *pEnum=0;
@@ -207,8 +236,14 @@ int try_format(V4wState *s, int format)
         if(pindir != PINDIR_INPUT)
 		{
 			IEnumMediaTypes *ppEnum;
-		    ULONG ulFound2;
-			hr = pPin->EnumMediaTypes(&ppEnum);
+	    ULONG ulFound2;
+
+      GetPinCategory(pPin, pPinCategory);
+      if (*pPinCategory!=PIN_CATEGORY_CAPTURE
+        && *pPinCategory!=PIN_CATEGORY_PREVIEW)
+        continue;
+
+      hr = pPin->EnumMediaTypes(&ppEnum);
 			if(FAILED(hr)) 
 				continue;
 
@@ -227,8 +262,8 @@ int try_format(V4wState *s, int format)
 				if (pvi->bmiHeader.biBitCount!=biBitCount)
 					continue;
 
-		        pPin->Release();
-			    pEnum->Release();
+        pPin->Release();
+			  pEnum->Release();
 				return 0;
 			}
 		}
@@ -240,7 +275,7 @@ int try_format(V4wState *s, int format)
     return -1;
 }
 
-int try_format_size(V4wState *s, int format, int width, int height)
+int try_format_size(V4wState *s, int format, int width, int height, GUID *pPinCategory)
 {
     HRESULT hr=S_OK;
     IEnumPins *pEnum=0;
@@ -310,6 +345,11 @@ int try_format_size(V4wState *s, int format, int width, int height)
 			if(FAILED(hr)) 
 				continue;
 
+      GUID pCurrentPinCategory;
+      GetPinCategory(pPin, &pCurrentPinCategory);
+      if (*pPinCategory!=pCurrentPinCategory)
+        continue;
+
 			AM_MEDIA_TYPE *ppMediaTypes;
 			while(S_OK == ppEnum->Next(1, &ppMediaTypes, &ulFound2))
 			{
@@ -351,14 +391,24 @@ static int v4w_open_videodevice(V4wState *s)
 	CoInitialize(NULL);
 
 	// get a Graph
-	HRESULT hr=s->m_pGraph.CoCreateInstance(CLSID_FilterGraph);
+	//HRESULT hr=s->m_pGraph.CoCreateInstance(CLSID_FilterGraph);
+	HRESULT hr= CoCreateInstance (CLSID_FilterGraph,
+                          NULL,
+                          CLSCTX_INPROC_SERVER,
+                          IID_IGraphBuilder, //IID_IBaseFilter,
+                          (void **)&s->m_pGraph);
 	if(FAILED(hr))
 	{
 		return -1;
 	}
 
 	// get a CaptureGraphBuilder2
-	hr=s->m_pBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder2);
+	//hr=s->m_pBuilder.CoCreateInstance(CLSID_CaptureGraphBuilder2);
+	hr= CoCreateInstance (CLSID_CaptureGraphBuilder2,
+                          NULL,
+                          CLSCTX_INPROC_SERVER,
+                          IID_ICaptureGraphBuilder2, //IID_IBaseFilter,
+                          (void **)&s->m_pBuilder);
 	if(FAILED(hr))
 	{
 		return -2;
@@ -368,7 +418,8 @@ static int v4w_open_videodevice(V4wState *s)
 	s->m_pBuilder->SetFiltergraph(s->m_pGraph);
 
 	// get mediacontrol so we can start and stop the filter graph
-	hr=s->m_pGraph.QueryInterface(&(s->m_pControl));
+	//hr=s->m_pGraph.QueryInterface(&(s->m_pControl));
+	hr=s->m_pGraph->QueryInterface (IID_IMediaControl, (void **)&s->m_pControl);
 	if(FAILED(hr))
 	{
 		return -3;
@@ -434,17 +485,20 @@ static int v4w_open_videodevice(V4wState *s)
 	pEnumMoniker->Release();
 	pCreateDevEnum->Release();
 
-	if (try_format(s, s->pix_fmt)==0)
+
+  GUID pPinCategory;
+
+	if (try_format(s, s->pix_fmt, &pPinCategory)==0)
 		s->pix_fmt = s->pix_fmt;
-	else if (try_format(s,MS_YUV420P)==0)
+	else if (try_format(s,MS_YUV420P, &pPinCategory)==0)
 		s->pix_fmt = MS_YUV420P;
-	else if (try_format(s,MS_YUY2)==0)
+	else if (try_format(s,MS_YUY2, &pPinCategory)==0)
 		s->pix_fmt = MS_YUY2;
-	else if (try_format(s,MS_YUYV)==0)
+	else if (try_format(s,MS_YUYV, &pPinCategory)==0)
 		s->pix_fmt = MS_YUYV;
-	else if (try_format(s,MS_UYVY)==0)
+	else if (try_format(s,MS_UYVY, &pPinCategory)==0)
 		s->pix_fmt = MS_UYVY;
-	else if (try_format(s,MS_RGB24)==0)
+	else if (try_format(s,MS_RGB24, &pPinCategory)==0)
 		s->pix_fmt = MS_RGB24;
 	else
 	{
@@ -463,17 +517,17 @@ static int v4w_open_videodevice(V4wState *s)
 	else if (s->pix_fmt == MS_RGB24)
 		ms_message("Driver supports RGB24, using that format.");
 
-	if (try_format_size(s, s->pix_fmt, s->vsize.width, s->vsize.height)==0)
+	if (try_format_size(s, s->pix_fmt, s->vsize.width, s->vsize.height, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", s->vsize.width, s->vsize.height);
-	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QCIF_W, MS_VIDEO_SIZE_QCIF_H)==0)
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QCIF_W, MS_VIDEO_SIZE_QCIF_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QCIF_W, MS_VIDEO_SIZE_QCIF_H);
-	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_CIF_W, MS_VIDEO_SIZE_CIF_H)==0)
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_CIF_W, MS_VIDEO_SIZE_CIF_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_CIF_W, MS_VIDEO_SIZE_CIF_H);
-	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_4CIF_W, MS_VIDEO_SIZE_4CIF_H)==0)
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_4CIF_W, MS_VIDEO_SIZE_4CIF_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_4CIF_W, MS_VIDEO_SIZE_4CIF_H);
-	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_VGA_W, MS_VIDEO_SIZE_VGA_H)==0)
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_VGA_W, MS_VIDEO_SIZE_VGA_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_VGA_W, MS_VIDEO_SIZE_VGA_H);
-	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H)==0)
+	else if (try_format_size(s, s->pix_fmt, MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H, &pPinCategory)==0)
 		ms_message("Selected Size: %ix%i.", MS_VIDEO_SIZE_QVGA_W, MS_VIDEO_SIZE_QVGA_H);
 	else
 	{
@@ -573,7 +627,12 @@ static int v4w_open_videodevice(V4wState *s)
 
 
 	// get null renderer
-	hr=s->m_pNullRenderer.CoCreateInstance(CLSID_NullRenderer);
+	//hr=s->m_pNullRenderer.CoCreateInstance(CLSID_NullRenderer);
+	hr=CoCreateInstance (CLSID_NullRenderer,
+                          NULL,
+                          CLSCTX_INPROC_SERVER,
+                          IID_IBaseFilter,
+                          (void **)&s->m_pNullRenderer);
 	if(FAILED(hr))
 	{
 		return -16;
@@ -583,7 +642,7 @@ static int v4w_open_videodevice(V4wState *s)
 		s->m_pGraph->AddFilter(s->m_pNullRenderer, L"Null Renderer");
 	}
 
-	hr = s->m_pBuilder->RenderStream(&PIN_CATEGORY_PREVIEW,
+	hr = s->m_pBuilder->RenderStream(&pPinCategory,
 		&MEDIATYPE_Video, s->m_pDeviceFilter, s->m_pIDXFilter, s->m_pNullRenderer);
 	if (FAILED(hr))
 	{
@@ -692,6 +751,7 @@ static void v4w_init(MSFilter *f){
 	{
 		s->mire[idx]=NULL;
 	}
+	memset(s->nowebcamimage, 0, sizeof(s->nowebcamimage));
 	ms_mutex_init(&s->mutex,NULL);
 	s->start_time=0;
 	s->frame_count=-1;
@@ -709,22 +769,42 @@ static int _v4w_start(V4wState *s, void *arg)
 	i = v4w_open_videodevice(s);
 
 	if (s->rotregvalue==0){
-		//RemoveGraphFromRot(s->rotregvalue);		
-		if (s->m_pNullRenderer!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
-		if (s->m_pIDXFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
-		if (s->m_pDeviceFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
-		s->m_pBuilder=NULL;
-		s->m_pControl=NULL;
-		s->m_pIDXFilter=NULL;
+		//RemoveGraphFromRot(s->rotregvalue);
+		if (s->m_pGraph!=NULL)
+		{
+			if (s->m_pNullRenderer!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
+			if (s->m_pIDXFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
+			if (s->m_pDeviceFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
+		}
+
+		if (s->m_pNullRenderer)
+			s->m_pNullRenderer->Release();
+		if (s->m_pIDXFilter)
+			s->m_pIDXFilter->Release();
+		if (s->m_pDeviceFilter)
+			s->m_pDeviceFilter->Release();
+
+		if (s->m_pBuilder)
+			s->m_pBuilder->Release();
+		if (s->m_pControl)
+			s->m_pControl->Release();
+		if (s->m_pGraph)
+			s->m_pGraph->Release();
+
 		if (s->m_pDXFilter!=NULL)
 			s->m_pDXFilter->Release();
-		s->m_pDXFilter=NULL;
-		s->m_pGraph=NULL;
+
 		s->m_pNullRenderer=NULL;
+		s->m_pIDXFilter=NULL;
 		s->m_pDeviceFilter=NULL;
+		s->m_pBuilder=NULL;
+		s->m_pControl=NULL;
+		s->m_pGraph=NULL;
+		s->m_pDXFilter=NULL;
+
 		CoUninitialize();
 		s_callback = NULL;
 		flushq(&s->rq,0);
@@ -743,22 +823,42 @@ static int _v4w_stop(V4wState *s, void *arg){
 		{
 			ms_message("v4w: could not stop graph");
 		}
-		if (s->m_pNullRenderer!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
-		if (s->m_pIDXFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
-		if (s->m_pDeviceFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
-		//RemoveGraphFromRot(s->rotregvalue);
-		s->m_pBuilder=NULL;
-		s->m_pControl=NULL;
-		s->m_pIDXFilter=NULL;
+
+		if (s->m_pGraph!=NULL)
+		{
+			if (s->m_pNullRenderer!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
+			if (s->m_pIDXFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
+			if (s->m_pDeviceFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
+		}
+
+		if (s->m_pNullRenderer)
+			s->m_pNullRenderer->Release();
+		if (s->m_pIDXFilter)
+			s->m_pIDXFilter->Release();
+		if (s->m_pDeviceFilter)
+			s->m_pDeviceFilter->Release();
+
+		if (s->m_pBuilder)
+			s->m_pBuilder->Release();
+		if (s->m_pControl)
+			s->m_pControl->Release();
+		if (s->m_pGraph)
+			s->m_pGraph->Release();
+
 		if (s->m_pDXFilter!=NULL)
 			s->m_pDXFilter->Release();
-		s->m_pDXFilter=NULL;
-		s->m_pGraph=NULL;
+
 		s->m_pNullRenderer=NULL;
+		s->m_pIDXFilter=NULL;
 		s->m_pDeviceFilter=NULL;
+		s->m_pBuilder=NULL;
+		s->m_pControl=NULL;
+		s->m_pGraph=NULL;
+		s->m_pDXFilter=NULL;
+
 		CoUninitialize();
 		s_callback = NULL;
 		flushq(&s->rq,0);
@@ -799,22 +899,42 @@ static void v4w_uninit(MSFilter *f){
 		{
 			ms_message("v4w: could not stop graph");
 		}
-		if (s->m_pNullRenderer!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
-		if (s->m_pIDXFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
-		if (s->m_pDeviceFilter!=NULL)
-			s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
-		//RemoveGraphFromRot(s->rotregvalue);
-		s->m_pBuilder=NULL;
-		s->m_pControl=NULL;
-		s->m_pIDXFilter=NULL;
+
+		if (s->m_pGraph!=NULL)
+		{
+			if (s->m_pNullRenderer!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pNullRenderer);
+			if (s->m_pIDXFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pIDXFilter);
+			if (s->m_pDeviceFilter!=NULL)
+				s->m_pGraph->RemoveFilter(s->m_pDeviceFilter);
+		}
+
+		if (s->m_pNullRenderer)
+			s->m_pNullRenderer->Release();
+		if (s->m_pIDXFilter)
+			s->m_pIDXFilter->Release();
+		if (s->m_pDeviceFilter)
+			s->m_pDeviceFilter->Release();
+
+		if (s->m_pBuilder)
+			s->m_pBuilder->Release();
+		if (s->m_pControl)
+			s->m_pControl->Release();
+		if (s->m_pGraph)
+			s->m_pGraph->Release();
+
 		if (s->m_pDXFilter!=NULL)
 			s->m_pDXFilter->Release();
-		s->m_pDXFilter=NULL;
-		s->m_pGraph=NULL;
+
 		s->m_pNullRenderer=NULL;
+		s->m_pIDXFilter=NULL;
 		s->m_pDeviceFilter=NULL;
+		s->m_pBuilder=NULL;
+		s->m_pControl=NULL;
+		s->m_pGraph=NULL;
+		s->m_pDXFilter=NULL;
+
 		CoUninitialize();
 		s_callback = NULL;
 		flushq(&s->rq,0);
@@ -830,6 +950,10 @@ static mblk_t * v4w_make_nowebcam(V4wState *s){
 #else
 	int idx;
 	int count;
+	if(s->mire[0]==NULL &&  s->frame_ind==0 && s->nowebcamimage[0] != '\0')
+	  {
+	    s->mire[0] = ms_load_jpeg_as_yuv(s->nowebcamimage,&s->vsize);
+	  }
 	if (s->mire[0]==NULL && s->frame_ind==0){
 		/* load several images to fake a movie */
 		for (idx=0;idx<10;idx++)
@@ -870,6 +994,7 @@ static void v4w_postprocess(MSFilter * obj){
 	s->running=FALSE;
 	s->start_time=0;
 	s->frame_count=-1;
+	flushq(&s->rq,0);
 }
 
 static void v4w_process(MSFilter * obj){
@@ -895,8 +1020,10 @@ static void v4w_process(MSFilter * obj){
 			}
 		}else {
 			mblk_t *nowebcam = v4w_make_nowebcam(s);
-			if (nowebcam!=NULL)
+			if (nowebcam!=NULL){
 				om=dupmsg(nowebcam);
+				mblk_set_precious_flag(om,1);
+			}
 		}
 		ms_mutex_unlock(&s->mutex);
 		if (om!=NULL){
@@ -918,6 +1045,13 @@ static int v4w_set_fps(MSFilter *f, void *arg){
 	return 0;
 }
 
+
+static int v4w_set_pix_fmt(MSFilter *f,void *arg){
+	V4wState *s=(V4wState*)f->data;
+  s->pix_fmt=*((MSPixFmt*)arg);
+	return 0;
+}
+
 static int v4w_get_pix_fmt(MSFilter *f,void *arg){
 	V4wState *s=(V4wState*)f->data;
 	*((MSPixFmt*)arg) = (MSPixFmt)s->pix_fmt;
@@ -944,14 +1078,37 @@ static int v4w_set_device(MSFilter *f, void *arg){
 	return 0;
 }
 
+static int v4w_set_image(MSFilter *f, void *arg){
+	int idx;
+	V4wState *s=(V4wState*)f->data;
+	char *image = (char *)arg;
+	ms_mutex_lock(&s->mutex);
+	if (image!=NULL && image[0]!='\0')
+	  snprintf(s->nowebcamimage, sizeof(s->nowebcamimage), "%s", image);
+	else
+	  s->nowebcamimage[0] = '\0';
+	for (idx=0;idx<10;idx++)
+	{
+		if (s->mire[idx]==NULL)
+			break;
+		freemsg(s->mire[idx]);
+		s->mire[idx]=NULL;
+	}
+	s->frame_ind=0;
+	ms_mutex_unlock(&s->mutex);
+	return 0;
+}
+
 static MSFilterMethod methods[]={
 	{	MS_FILTER_SET_FPS	,	v4w_set_fps	},
+	{	MS_FILTER_SET_PIX_FMT	,	v4w_set_pix_fmt	},
 	{	MS_FILTER_GET_PIX_FMT	,	v4w_get_pix_fmt	},
 	{	MS_FILTER_SET_VIDEO_SIZE, v4w_set_vsize	},
 	{	MS_FILTER_GET_VIDEO_SIZE, v4w_get_vsize	},
 	{	MS_V4L_START			,	v4w_start		},
 	{	MS_V4L_STOP			,	v4w_stop		},
 	{	MS_V4L_SET_DEVICE	,	v4w_set_device		},
+	{       MS_V4L_SET_IMAGE        ,       v4w_set_image           },
 	{	0								,	NULL			}
 };
 
@@ -993,3 +1150,26 @@ MSFilterDesc ms_v4w_desc={
 #endif
 
 MS_FILTER_DESC_EXPORT(ms_v4w_desc)
+
+static MSFilter *vfw_create_reader(MSWebCam *obj){
+	return ms_filter_new_from_desc(&ms_v4w_desc);
+}
+
+static void vfw_detect(MSWebCamManager *obj);
+
+static void vfw_cam_init(MSWebCam *cam){
+	cam->name=ms_strdup("dx0");
+}
+
+MSWebCamDesc ms_v4w_cam_desc={
+	"DirectX Video Grabber",
+	&vfw_detect,
+	&vfw_cam_init,
+	&vfw_create_reader,
+	NULL
+};
+
+static void vfw_detect(MSWebCamManager *obj){
+	MSWebCam *cam=ms_web_cam_new(&ms_v4w_cam_desc);
+	ms_web_cam_manager_add_cam(obj,cam);
+}
