@@ -22,6 +22,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #include "mediastreamer2/mediastream.h"
+#ifdef VIDEO_ENABLED
+#include "mediastreamer2/msv4l.h"
+#endif
 
 #include <signal.h>
 #include <sys/types.h>
@@ -42,12 +45,19 @@ static void stop_handler(int signum)
 
 static bool_t parse_addr(const char *addr, char *ip, int len, int *port)
 {
-	char *semicolon;
+	const char *semicolon=NULL;
 	int iplen;
 	int slen;
-	
+	const char *p;
+
 	*port=0;
 	semicolon=strchr(addr,':');
+	for (p=addr+strlen(addr)-1;p>addr;p--){
+		if (*p==':') {
+			semicolon=p;
+			break;
+		}
+	}
 	if (semicolon==NULL) return FALSE;
 	iplen=semicolon-addr;
 	slen=MIN(iplen,len-1);
@@ -78,17 +88,11 @@ static void display_items(void *user_data, uint32_t csrc, rtcp_sdes_type_t t, co
 }
 
 static void parse_rtcp(mblk_t *m){
-	const report_block_t *rb;
 	do{
 		if (rtcp_is_RR(m)){
 			ms_message("Receiving RTCP RR");
 		}else if (rtcp_is_SR(m)){
 			ms_message("Receiving RTCP SR");
-			rb=rtcp_SR_get_report_block(m,0);
-			if (rb){
-				ms_message("interarrival jitter=%u",
-					report_block_get_interarrival_jitter(rb));
-			}
 		}else if (rtcp_is_SDES(m)){
 			ms_message("Receiving RTCP SDES");
 			rtcp_sdes_parse(m,display_items,NULL);
@@ -119,10 +123,6 @@ const char *usage="mediastream --local <port> --remote <ip:port> --payload <payl
 static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp, int jitter, bool_t ec, int bitrate);
 
 
-#ifdef VIDEO_ENABLED
-VAR_DECLSPEC PayloadType payload_type_x_snow;
-#endif
-
 int main(int argc, char * argv[])
 {
 	int i;
@@ -212,12 +212,13 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 #ifdef VIDEO_ENABLED
 		printf("Starting video stream.\n");
 		video=video_stream_new(localport, ms_is_ipv6(remote_ip));
+
 		video_stream_start(video,profile,
 					remote_ip,
-					remoteport,
+					remoteport,remoteport+1,
 					payload,
 					jitter,
-					"/dev/video0");
+					ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get()));
 		session=video->session;
 #else
 		printf("Error: video support not compiled.\n");
@@ -230,7 +231,7 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
         	for(n=0;n<100;++n){
 #ifdef WIN32
 			MSG msg;
-    		Sleep(10);
+			Sleep(10);
 			while (PeekMessage(&msg, NULL, 0, 0,1)){
         			TranslateMessage(&msg);
         			DispatchMessage(&msg);
@@ -241,7 +242,7 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 			ts.tv_nsec=10000000;
 			nanosleep(&ts,NULL);
 #endif
-#ifdef VIDEO_ENABLED
+#if defined(VIDEO_ENABLED)
 			if (video) video_stream_iterate(video);
 #endif
 		}
