@@ -45,19 +45,37 @@ static void init_random_number_generator(){
 	srandom(t.tv_usec+t.tv_sec);
 }
 
+
+#ifdef WIN32
+static bool_t win32_init_sockets(void){
+	WORD wVersionRequested;
+	WSADATA wsaData;
+	int i;
+	
+	wVersionRequested = MAKEWORD(1,1);
+	if( (i = WSAStartup(wVersionRequested,  &wsaData))!=0)
+	{
+		ortp_error("Unable to initialize windows socket api, reason: %d",i);
+		return FALSE;
+	}
+	return TRUE;
+}
+#endif
+
 /**
- *ortp_init:
- *
  *	Initialize the oRTP library. You should call this function first before using
  *	oRTP API.
 **/
- 
 void ortp_init()
 {
 	static bool_t initialized=FALSE;
 	if (initialized) return;
 	initialized=TRUE;
 
+#ifdef WIN32
+	win32_init_sockets();
+#endif
+
 	av_profile_init(&av_profile);
 	ortp_global_stats_reset();
 	init_random_number_generator();
@@ -66,8 +84,6 @@ void ortp_init()
 
 
 /**
- *ortp_scheduler_init:
- *
  *	Initialize the oRTP scheduler. You only have to do that if you intend to use the
  *	scheduled mode of the #RtpSession in your application.
  *	
@@ -94,8 +110,6 @@ void ortp_scheduler_init()
 
 
 /**
- *ortp_exit:
- *
  * Gracefully uninitialize the library, including shutdowning the scheduler if it was started.
  *	
 **/
@@ -108,14 +122,6 @@ void ortp_exit()
 	}
 }
 
-/**
- *ortp_get_scheduler:
- *
- *	Returns a pointer to the scheduler, NULL if it was not running.
- *	The application developer should have to call this function.
- *
- *Returns: a pointer to the scheduler.
-**/
 RtpScheduler * ortp_get_scheduler()
 {
 	if (__ortp_scheduler==NULL) ortp_error("Cannot use the scheduled mode: the scheduler is not "
@@ -127,8 +133,7 @@ RtpScheduler * ortp_get_scheduler()
 static FILE *__log_file=0;
 
 /**
- *ortp_set_log_file:
- *@file: a FILE pointer where to output the ortp logs.
+ *@param file a FILE pointer where to output the ortp logs.
  *
 **/
 void ortp_set_log_file(FILE *file)
@@ -141,8 +146,7 @@ static void __ortp_logv_out(OrtpLogLevel lev, const char *fmt, va_list args);
 OrtpLogFunc ortp_logv_out=__ortp_logv_out;
 
 /**
- *ortp_set_log_handler:
- *@func: your logging function, compatible with the OrtpLogFunc prototype.
+ *@param func: your logging function, compatible with the OrtpLogFunc prototype.
  *
 **/
 void ortp_set_log_handler(OrtpLogFunc func){
@@ -152,6 +156,10 @@ void ortp_set_log_handler(OrtpLogFunc func){
 
 unsigned int __ortp_log_mask=ORTP_WARNING|ORTP_ERROR|ORTP_FATAL;
 
+/**
+ * @ param levelmask a mask of ORTP_DEBUG, ORTP_MESSAGE, ORTP_WARNING, ORTP_ERROR
+ * ORTP_FATAL .
+**/
 void ortp_set_log_level_mask(int levelmask){
 	__ortp_log_mask=levelmask;
 }
@@ -161,15 +169,22 @@ static char * _strdup_vprintf(const char *fmt, va_list ap)
 	/* Guess we need no more than 100 bytes. */
 	int n, size = 200;
 	char *p,*np;
-	//va_list ap;
+#ifdef __linux
+	va_list cap;/*copy of our argument list: a va_list cannot be re-used (SIGSEGV on linux 64 bits)*/
+#endif
 	if ((p = (char *) ortp_malloc (size)) == NULL)
 		return NULL;
 	while (1)
 	{
 		/* Try to print in the allocated space. */
-		//va_start(ap, fmt);
+#ifdef __linux
+		va_copy(cap,ap);
+		n = vsnprintf (p, size, fmt, cap);
+		va_end(cap);
+#else
+		/*this works on 32 bits, luckily*/
 		n = vsnprintf (p, size, fmt, ap);
-		//va_end(ap);
+#endif
 		/* If that worked, return the string. */
 		if (n > -1 && n < size)
 			return p;
@@ -245,7 +260,9 @@ static void __ortp_logv_out(OrtpLogLevel lev, const char *fmt, va_list args){
 	ortp_free(msg);
 }
 
-
+/**
+ * Display global statistics (cumulative for all RtpSession)
+**/
 void ortp_global_stats_display()
 {
 	rtp_stats_display(&ortp_global_stats,"Global statistics");
@@ -254,57 +271,79 @@ void ortp_global_stats_display()
 #endif
 }
 
-
+/**
+ * Print RTP statistics.
+**/
 void rtp_stats_display(const rtp_stats_t *stats, const char *header)
 {
 #ifndef WIN32
-	ortp_log(ORTP_MESSAGE,
-	      "oRTP-stats:\n   %s :\n"
-	      " number of rtp packet sent=%lld\n"
-	      " number of rtp bytes sent=%lld bytes\n"
-	      " number of rtp packet received=%lld\n"
-	      " number of rtp bytes received=%lld bytes\n"
-	      " number of incoming rtp bytes successfully delivered to the application=%lld \n"
-	      " number of times the application queried a packet that didn't exist=%lld \n"
-		" number of rtp packet lost=%lld\n"
-	      " number of rtp packets received too late=%lld\n"
-	      " number of bad formatted rtp packets=%lld\n"
-	      " number of packet discarded because of queue overflow=%lld\n",
-	      header,
-	      (long long)stats->packet_sent,
-	      (long long)stats->sent,
-	      (long long)stats->packet_recv,
-	      (long long)stats->hw_recv,
-	      (long long)stats->recv,
-	      (long long)stats->unavaillable,
-		(long long)stats->cum_packet_loss,
-	      (long long)stats->outoftime,
-	      (long long)stats->bad,
-	      (long long)stats->discarded);
+  ortp_log(ORTP_MESSAGE,
+	   "oRTP-stats:\n   %s :",
+	   header);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet sent=%lld",
+	   (long long)stats->packet_sent);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp bytes sent=%lld bytes",
+	   (long long)stats->sent);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet received=%lld",
+	   (long long)stats->packet_recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp bytes received=%lld bytes",
+	   (long long)stats->hw_recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of incoming rtp bytes successfully delivered to the application=%lld ",
+	   (long long)stats->recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of times the application queried a packet that didn't exist=%lld ",
+	   (long long)stats->unavaillable);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet lost=%lld",
+	   (long long) stats->cum_packet_loss);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packets received too late=%lld",
+	   (long long)stats->outoftime);
+  ortp_log(ORTP_MESSAGE,
+	   " number of bad formatted rtp packets=%lld",
+	   (long long)stats->bad);
+  ortp_log(ORTP_MESSAGE,
+	   " number of packet discarded because of queue overflow=%lld",
+	   (long long)stats->discarded);
 #else
-	ortp_log(ORTP_MESSAGE,
-	      "oRTP-stats:\n   %s :\n"
-	      " number of rtp packet sent=%I64d\n"
-	      " number of rtp bytes sent=%I64d bytes\n"
-	      " number of rtp packet received=%I64d\n"
-	      " number of rtp bytes received=%I64d bytes\n"
-	      " number of incoming rtp bytes successfully delivered to the application=%I64d \n"
-	      " number of times the application queried a packet that didn't exist=%I64d \n"
-		" number of rtp packet lost=%I64d\n"
-	      " number of rtp packets received too late=%I64d\n"
-	      " number of bad formatted rtp packets=%I64d\n"
-	      " number of packet discarded because of queue overflow=%I64d\n",
-	      header,
-	      (uint64_t)stats->packet_sent,
-	      (uint64_t)stats->sent,
-	      (uint64_t)stats->packet_recv,
-	      (uint64_t)stats->hw_recv,
-	      (uint64_t)stats->recv,
-	      (uint64_t)stats->unavaillable,
-		(uint64_t) stats->cum_packet_loss,
-	      (uint64_t)stats->outoftime,
-	      (uint64_t)stats->bad,
-	      (uint64_t)stats->discarded);
+  ortp_log(ORTP_MESSAGE,
+	   "oRTP-stats:\n   %s :",
+	   header);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet sent=%I64d",
+	   (uint64_t)stats->packet_sent);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp bytes sent=%I64d bytes",
+	   (uint64_t)stats->sent);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet received=%I64d",
+	   (uint64_t)stats->packet_recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp bytes received=%I64d bytes",
+	   (uint64_t)stats->hw_recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of incoming rtp bytes successfully delivered to the application=%I64d ",
+	   (uint64_t)stats->recv);
+  ortp_log(ORTP_MESSAGE,
+	   " number of times the application queried a packet that didn't exist=%I64d ",
+	   (uint64_t)stats->unavaillable);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packet lost=%I64d",
+	   (uint64_t) stats->cum_packet_loss);
+  ortp_log(ORTP_MESSAGE,
+	   " number of rtp packets received too late=%I64d",
+	   (uint64_t)stats->outoftime);
+  ortp_log(ORTP_MESSAGE,
+		 " number of bad formatted rtp packets=%I64d",
+	   (uint64_t)stats->bad);
+  ortp_log(ORTP_MESSAGE,
+	   " number of packet discarded because of queue overflow=%I64d",
+	   (uint64_t)stats->discarded);
 #endif
 }
 
@@ -322,11 +361,6 @@ void rtp_stats_reset(rtp_stats_t *stats){
 
 
 /**
- *ortp_min_version_required:
- *@major: 
- *@minor: 
- *@micro:
- *
  * This function give the opportunity to programs to check if the libortp they link to
  * has the minimum version number they need.
  *
