@@ -21,65 +21,14 @@
 #include "ortp/payloadtype.h"
 #include "utils.h"
 
-/**
- *rtp_profile_clear_all:
- *@profile: an RTP profile (#RtpProfile object)
- *
- *	Initialize the profile to the empty profile (all payload type are unassigned).
- *
-**/
-
-/**
- *rtp_profile_set_name:
- *@profile: 	a rtp profile object (#RtpProfile)
- *@nm:			a string
- *
- *	Set a name to the rtp profile. (This is not required)
- *
-**/
-
-/**
- *rtp_profile_get_name:
- *@profile:		a rtp profile object (#RtpProfile)
- *
- *Returns:	the name of the rtp profile. May be NULL.
-**/
-
-/**
- *rtp_profile_set_payload:
- *@profile: an RTP profile (a #RtpProfile object)
- *@idx:	the payload type number
- *@pt:		the payload type description (a #PayloadType object )
- *
- *	Assign payload type number @index to payload type desribed in @pt for the RTP profile
- *	@profile.
- *
-**/
-
-/**
- *rtp_profile_get_payload:
- *@profile:	an RTP profile (a #RtpProfile object)
- *@index:	the payload type number
- *
- *	Gets the payload description of the payload type @index in the profile @profile.
- *
- *Returns: the payload description (a #PayloadType object)
-**/
-
-/**
- *rtp_profile_clear_payload:
- *@profile:	an RTP profile (a #RtpProfile object)
- *@index:	the payload type number
- *
- *	Set payload type number @index unassigned in profile @profile.
- *
-**/
-
 char *payload_type_get_rtpmap(PayloadType *pt)
 {
 	int len=(int)strlen(pt->mime_type)+15;
 	char *rtpmap=(char *) ortp_malloc(len);
-	snprintf(rtpmap,len,"%s/%i/1",pt->mime_type,pt->clock_rate);
+	if (pt->channels>0)
+		snprintf(rtpmap,len,"%s/%i/%i",pt->mime_type,pt->clock_rate,pt->channels);
+	else
+		 snprintf(rtpmap,len,"%s/%i",pt->mime_type,pt->clock_rate);
 	return rtpmap;
 }
 
@@ -96,8 +45,12 @@ PayloadType *payload_type_clone(PayloadType *payload)
 	PayloadType *newpayload=(PayloadType *)ortp_new0(PayloadType,1);
 	memcpy(newpayload,payload,sizeof(PayloadType));
 	newpayload->mime_type=ortp_strdup(payload->mime_type);
-	if (payload->recv_fmtp!=NULL) newpayload->recv_fmtp=ortp_strdup(payload->recv_fmtp);
-	if (payload->send_fmtp!=NULL) newpayload->send_fmtp=ortp_strdup(payload->send_fmtp);
+	if (payload->recv_fmtp!=NULL) {
+		newpayload->recv_fmtp=ortp_strdup(payload->recv_fmtp);
+	}
+	if (payload->send_fmtp!=NULL){
+		newpayload->send_fmtp=ortp_strdup(payload->send_fmtp);
+	}
 	newpayload->flags|=PAYLOAD_TYPE_ALLOCATED;
 	return newpayload;
 }
@@ -111,6 +64,11 @@ static bool_t canWrite(PayloadType *pt){
 	return TRUE;
 }
 
+/**
+ * Sets a recv parameters (fmtp) for the PayloadType.
+ * This method is provided for applications using RTP with SDP, but
+ * actually the ftmp information is not used for RTP processing.
+**/
 void payload_type_set_recv_fmtp(PayloadType *pt, const char *fmtp){
 	if (canWrite(pt)){
 		if (pt->recv_fmtp!=NULL) ortp_free(pt->recv_fmtp);
@@ -118,7 +76,12 @@ void payload_type_set_recv_fmtp(PayloadType *pt, const char *fmtp){
 		else pt->recv_fmtp=NULL;
 	}
 }
-	
+
+/**
+ * Sets a send parameters (fmtp) for the PayloadType.
+ * This method is provided for applications using RTP with SDP, but
+ * actually the ftmp information is not used for RTP processing.
+**/
 void payload_type_set_send_fmtp(PayloadType *pt, const char *fmtp){
 	if (canWrite(pt)){
 		if (pt->send_fmtp!=NULL) ortp_free(pt->send_fmtp);
@@ -127,6 +90,36 @@ void payload_type_set_send_fmtp(PayloadType *pt, const char *fmtp){
 	}
 }
 
+
+
+void payload_type_append_recv_fmtp(PayloadType *pt, const char *fmtp){
+	if (canWrite(pt)){
+		if (pt->recv_fmtp==NULL)
+			pt->recv_fmtp=ortp_strdup(fmtp);
+		else{
+			char *tmp=ortp_strdup_printf("%s;%s",pt->recv_fmtp,fmtp);
+			ortp_free(pt->recv_fmtp);
+			pt->recv_fmtp=tmp;
+		}
+	}
+}
+
+void payload_type_append_send_fmtp(PayloadType *pt, const char *fmtp){
+	if (canWrite(pt)){
+		if (pt->send_fmtp==NULL)
+			pt->send_fmtp=ortp_strdup(fmtp);
+		else{
+			char *tmp=ortp_strdup_printf("%s;%s",pt->send_fmtp,fmtp);
+			ortp_free(pt->send_fmtp);
+			pt->send_fmtp=tmp;
+		}
+	}
+}
+
+
+/**
+ * Frees a PayloadType.
+**/
 void payload_type_destroy(PayloadType *pt)
 {
 	if (pt->mime_type) ortp_free(pt->mime_type);
@@ -135,7 +128,17 @@ void payload_type_destroy(PayloadType *pt)
 	ortp_free(pt);
 }
 
-
+/**
+ * Parses a fmtp string such as "profile=0;level=10", finds the value matching
+ * parameter param_name, and writes it into result. 
+ * Despite fmtp strings are not used anywhere within oRTP, this function can
+ * be useful for people using RTP streams described from SDP.
+ * @param fmtp the fmtp line (format parameters)
+ * @param param_name the parameter to search for
+ * @param result the value given for the parameter (if found)
+ * @param result_len the size allocated to hold the result string
+ * @return TRUE if the parameter was found, else FALSE.
+**/ 
 bool_t fmtp_get_value(const char *fmtp, const char *param_name, char *result, size_t result_len){
 	const char *pos=strstr(fmtp,param_name);
 	if (pos){
@@ -171,7 +174,7 @@ int rtp_profile_get_payload_number_from_mime(RtpProfile *profile,const char *mim
 	return -1;
 }
 
-int rtp_profile_find_payload_number(RtpProfile*profile,const char *mime,int rate)
+int rtp_profile_find_payload_number(RtpProfile*profile,const char *mime,int rate,int channels)
 {
 	int i;
 	PayloadType *pt;
@@ -180,7 +183,11 @@ int rtp_profile_find_payload_number(RtpProfile*profile,const char *mime,int rate
 		pt=rtp_profile_get_payload(profile,i);
 		if (pt!=NULL)
 		{
-			if (strcasecmp(pt->mime_type,mime)==0 && pt->clock_rate==rate){
+			if (strcasecmp(pt->mime_type,mime)==0 &&
+			    pt->clock_rate==rate &&
+			    (pt->channels==channels || channels<=0 || pt->channels<=0)) {
+				/*we don't look at channels if it is undefined
+				ie a negative or zero value*/
 			
 				return i;
 			}
@@ -191,31 +198,48 @@ int rtp_profile_find_payload_number(RtpProfile*profile,const char *mime,int rate
 
 int rtp_profile_get_payload_number_from_rtpmap(RtpProfile *profile,const char *rtpmap)
 {
-	int clock_rate,ret;
-	char *p,*mime,*tmp,*c;
+	int clock_rate, channels, ret;
+	char* subtype = ortp_strdup( rtpmap );
+	char* rate_str = NULL;
+	char* chan_str = NULL;
 	
-	/* parse the rtpmap */
-	tmp=ortp_strdup(rtpmap);
-	p=strchr(tmp,'/');
-	if (p!=NULL){
-		mime=tmp;
-		*p='\0';
-		c=p+1;
-		p=strchr(c,'/');
-		if (p!=NULL) *p='\0';
-		clock_rate=atoi(c);
-	}else return -1;
 	
-	//printf("Searching for payload %s at freq %i",mime,clock_rate);
-	ret=rtp_profile_find_payload_number(profile,mime,clock_rate);
-	ortp_free(tmp);
+	/* find the slash after the subtype */
+	rate_str = strchr(subtype, '/');
+	if (rate_str && strlen(rate_str)>1) {
+		*rate_str = 0;
+		rate_str++;
+		
+		/* Look for another slash */
+		chan_str = strchr(rate_str, '/');
+		if (chan_str && strlen(chan_str)>1) {
+			*chan_str = 0;
+			chan_str++;
+		} else {
+			chan_str = NULL;
+		}
+	} else {
+		rate_str = NULL;
+	}
+	
+	// Use default clock rate if none given	
+	if (rate_str) clock_rate = atoi(rate_str);
+	else clock_rate = 8000;
+
+	// Use default number of channels if none given	
+	if (chan_str) channels = atoi(chan_str);
+	else channels = -1;
+
+	//printf("Searching for payload %s at freq %i with %i channels\n",subtype,clock_rate,ch1annels);
+	ret=rtp_profile_find_payload_number(profile,subtype,clock_rate,channels);
+	ortp_free(subtype);
 	return ret;
 }
 
-PayloadType * rtp_profile_find_payload(RtpProfile *prof,const char *mime,int rate)
+PayloadType * rtp_profile_find_payload(RtpProfile *prof,const char *mime,int rate,int channels)
 {
 	int i;
-	i=rtp_profile_find_payload_number(prof,mime,rate);
+	i=rtp_profile_find_payload_number(prof,mime,rate,channels);
 	if (i>=0) return rtp_profile_get_payload(prof,i);
 	return NULL;
 }
@@ -232,8 +256,7 @@ PayloadType * rtp_profile_get_payload_from_mime(RtpProfile *profile,const char *
 
 PayloadType * rtp_profile_get_payload_from_rtpmap(RtpProfile *profile, const char *rtpmap)
 {
-	int pt;
-	pt=rtp_profile_get_payload_number_from_rtpmap(profile,rtpmap);
+	int pt = rtp_profile_get_payload_number_from_rtpmap(profile,rtpmap);
 	if (pt==-1) return NULL;
 	else return rtp_profile_get_payload(profile,pt);
 }
@@ -251,6 +274,13 @@ RtpProfile * rtp_profile_new(const char *name)
 	return prof;
 }
 
+/**
+ *	Assign payload type number index to payload type desribed in pt for the RTP profile profile.
+ * @param profile a RTP profile
+ * @param idx the payload type number
+ * @param pt the payload type description
+ *
+**/
 void rtp_profile_set_payload(RtpProfile *prof, int idx, PayloadType *pt){
 	if (idx<0 || idx>=RTP_PROFILE_MAX_PAYLOADS) {
 		ortp_error("Bad index %i",idx);
@@ -259,6 +289,11 @@ void rtp_profile_set_payload(RtpProfile *prof, int idx, PayloadType *pt){
 	prof->payload[idx]=pt;
 }
 
+/**
+ * Initialize the profile to the empty profile (all payload type are unassigned).
+ *@param profile a RTP profile
+ *
+**/
 void rtp_profile_clear_all(RtpProfile *obj){
 	int i;
 	for (i=0;i<RTP_PROFILE_MAX_PAYLOADS;i++){
@@ -266,6 +301,13 @@ void rtp_profile_clear_all(RtpProfile *obj){
 	}
 }
 
+
+/**
+ * Set a name to the rtp profile. (This is not required)
+ * @param profile a rtp profile object
+ * @param nm a string
+ *
+**/
 void rtp_profile_set_name(RtpProfile *obj, const char *name){
 	if (obj->name!=NULL) ortp_free(obj->name);
 	obj->name=ortp_strdup(name);
