@@ -25,9 +25,9 @@
 
 #define SSRC_CHANGED_THRESHOLD 50
 
-void split_and_queue(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int *discarded)
+static void queue_packet(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int *discarded)
 {
-	mblk_t *mdata,*tmp;
+	mblk_t *tmp;
 	int header_size;
 	*discarded=0;
 	header_size=RTP_FIXED_HEADER_SIZE+ (4*rtp->cc);
@@ -37,13 +37,6 @@ void split_and_queue(queue_t *q, int maxrqsz, mblk_t *mp, rtp_header_t *rtp, int
 		freemsg(mp);
 		return;
 	}
-	/* creates a new mblk_t to be linked with the rtp header*/
-	mdata=dupb(mp);
-	
-	mp->b_wptr=mp->b_rptr+header_size;
-	mdata->b_rptr+=header_size;
-	/* link proto with data */
-	mp->b_cont=mdata;
 	/* and then add the packet to the queue */
 	
 	rtp_putq(q,mp);
@@ -174,7 +167,7 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 	
 	/* check for possible telephone events */
 	if (rtp->paytype==session->rcv.telephone_events_pt){
-		split_and_queue(&session->rtp.tev_rq,session->rtp.max_rq_size,mp,rtp,&i);
+		queue_packet(&session->rtp.tev_rq,session->rtp.max_rq_size,mp,rtp,&i);
 		stats->discarded+=i;
 		ortp_global_stats.discarded+=i;
 		return;
@@ -186,14 +179,9 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 		rtp_session_update_payload_type(session,rtp->paytype);
 	}
 	
+	jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts);
+
 	if (session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED) {
-		int32_t slide=0;
-		int32_t safe_delay=0;
-		jitter_control_new_packet(&session->rtp.jittctl,rtp->timestamp,local_str_ts,&slide,&safe_delay);
-		
-		session->rtp.rcv_diff_ts=session->rtp.hwrcv_diff_ts + slide - safe_delay;
-		ortp_debug("  rcv_diff_ts=%i", session->rtp.rcv_diff_ts);
-		
 		/* detect timestamp important jumps in the future, to workaround stupid rtp senders */
 		if (RTP_TIMESTAMP_IS_NEWER_THAN(rtp->timestamp,session->rtp.rcv_last_ts+session->rtp.ts_jump)){
 			ortp_debug("rtp_parse: timestamp jump ?");
@@ -216,10 +204,10 @@ void rtp_session_rtp_parse(RtpSession *session, mblk_t *mp, uint32_t local_str_t
 			ortp_global_stats.outoftime++;
 			return;
 		}
-		
 	}
 	
-	split_and_queue(&session->rtp.rq,session->rtp.max_rq_size,mp,rtp,&i);
+	queue_packet(&session->rtp.rq,session->rtp.max_rq_size,mp,rtp,&i);
 	stats->discarded+=i;
 	ortp_global_stats.discarded+=i;
 }
+
