@@ -215,7 +215,6 @@ rtp_session_init (RtpSession * session, int mode)
 {
 	JBParameters jbp;
 	memset (session, 0, sizeof (RtpSession));
-	session->rtp.max_rq_size = 100;/* maximum number of packet allowed to be queued */
 	session->mode = (RtpSessionMode) mode;
 	if ((mode == RTP_SESSION_RECVONLY) || (mode == RTP_SESSION_SENDRECV))
 	{
@@ -236,6 +235,12 @@ rtp_session_init (RtpSession * session, int mode)
 	rtp_session_set_profile (session, &av_profile); /*the default profile to work with */
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
+#ifndef WIN32
+	session->rtp.snd_socket_size=0;	/*use OS default value unless on windows where they are definitely too short*/
+	session->rtp.rcv_socket_size=0;
+#else
+	session->rtp.snd_socket_size=session->rtp.rcv_socket_size=65536;
+#endif
 	session->dscp=RTP_DEFAULT_DSCP;
 	session->multicast_ttl=RTP_DEFAULT_MULTICAST_TTL;
 	session->multicast_loopback=RTP_DEFAULT_MULTICAST_LOOPBACK;
@@ -261,6 +266,7 @@ rtp_session_init (RtpSession * session, int mode)
 	jbp.min_size=RTP_DEFAULT_JITTER_TIME;
 	jbp.nom_size=RTP_DEFAULT_JITTER_TIME;
 	jbp.max_size=-1;
+	jbp.max_packets= 100;/* maximum number of packet allowed to be queued */
 	jbp.adaptive=TRUE;
 	rtp_session_enable_jitter_buffer(session,TRUE);
 	rtp_session_set_jitter_buffer_params(session,&jbp);
@@ -269,6 +275,7 @@ rtp_session_init (RtpSession * session, int mode)
 	session->recv_buf_size = UDP_MAX_SIZE;
 	session->symmetric_rtp = FALSE;
 	session->permissive=FALSE;
+	msgb_allocator_init(&session->allocator);
 }
 
 
@@ -444,9 +451,9 @@ RtpProfile *rtp_session_get_recv_profile(RtpSession *session){
 }
 
 /**
- *	The default value is 65535 bytes, a big value which is working for everyone.
- *	However if your application can make assumption on the MTU, it can be interesting
- *	to set it to a lower value in order to save memory.
+ *	The default value is UDP_MAX_SIZE bytes, a value which is working for mostly everyone.
+ *	However if your application can make assumption on the sizes of received packet,
+ *	it can be interesting to set it to a lower value in order to save memory.
  *
  * @param session a rtp session
  * @param bufsize max size in bytes for receiving packets
@@ -455,6 +462,22 @@ void rtp_session_set_recv_buf_size(RtpSession *session, int bufsize){
 	session->recv_buf_size=bufsize;
 }
 
+/**
+ *	Set kernel send maximum buffer size for the rtp socket.
+ *	A value of zero defaults to the operating system default.
+**/
+void rtp_session_set_rtp_socket_send_buffer_size(RtpSession * session, unsigned int size){
+	session->rtp.snd_socket_size=size;
+}
+
+/**
+ *	Set kernel recv maximum buffer size for the rtp socket.
+ *	A value of zero defaults to the operating system default.
+**/
+void rtp_session_set_rtp_socket_recv_buffer_size(RtpSession * session, unsigned int size){
+	session->rtp.rcv_socket_size=size;
+}
+
 /**
  *	This function provides the way for an application to be informed of various events that
  *	may occur during a rtp session. @signal is a string identifying the event, and @cb is 
@@ -750,19 +773,9 @@ mblk_t * rtp_session_create_packet_in_place(RtpSession *session,uint8_t *buffer,
 	return mp;
 }
 
-/**
- *	Send the rtp datagram @mp to the destination set by rtp_session_set_remote_addr() 
- *	with timestamp @timestamp. For audio data, the timestamp is the number
- *	of the first sample resulting of the data transmitted. See rfc1889 for details.
- *  The packet (@mp) is freed once it is sended.
- *
- *@param session a rtp session.
- *@param mp a rtp packet presented as a mblk_t.
- *@param timestamp the timestamp of the data to be sent.
- * @return the number of bytes sent over the network.
-**/
+
 int
-rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
+__rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t packet_ts, uint32_t send_ts)
 {
 	rtp_header_t *rtp;
 	uint32_t packet_time;
@@ -773,13 +786,13 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 
 	if (session->flags & RTP_SESSION_SEND_NOT_STARTED)
 	{
-		session->rtp.snd_ts_offset = timestamp;
-        /* Set initial last_rcv_time to first send time. */
-        if ((session->flags & RTP_SESSION_RECV_NOT_STARTED)
-            || session->mode == RTP_SESSION_SENDONLY)
-        {
-            gettimeofday(&session->last_recv_time, NULL);
-        }
+		session->rtp.snd_ts_offset = send_ts;
+		/* Set initial last_rcv_time to first send time. */
+		if ((session->flags & RTP_SESSION_RECV_NOT_STARTED)
+		|| session->mode == RTP_SESSION_SENDONLY)
+		{
+		gettimeofday(&session->last_recv_time, NULL);
+		}
 		if (session->flags & RTP_SESSION_SCHEDULED)
 		{
 			session->rtp.snd_time_offset = sched->time_;
@@ -794,7 +807,7 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 	{
 		packet_time =
 			rtp_session_ts_to_time (session,
-				     timestamp -
+				     send_ts -
 				     session->rtp.snd_ts_offset) +
 					session->rtp.snd_time_offset;
 		/*ortp_message("rtp_session_send_with_ts: packet_time=%i time=%i",packet_time,sched->time_);*/
@@ -808,12 +821,17 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 		wait_point_unlock(&session->snd.wp);
 	}
 	
+	if(mp==NULL) {/*for people who just want to be blocked but
+		 do not want to send anything.*/
+		session->rtp.snd_last_ts = packet_ts;
+		return 0;
+	}
 
 	rtp=(rtp_header_t*)mp->b_rptr;
 	
 	packsize = msgdsize(mp) ;
 	
-	rtp->timestamp=timestamp;
+	rtp->timestamp=packet_ts;
 	if (session->snd.telephone_events_pt==rtp->paytype)
 	{
 		session->rtp.snd_seq++;
@@ -821,10 +839,11 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 	}
 	else
 		session->rtp.snd_seq=rtp->seq_number+1;
-	session->rtp.snd_last_ts = timestamp;
+	session->rtp.snd_last_ts = packet_ts;
 
 
 	ortp_global_stats.sent += packsize;
+	stream->sent_payload_bytes+=packsize-RTP_FIXED_HEADER_SIZE;
 	stream->stats.sent += packsize;
 	ortp_global_stats.packet_sent++;
 	stream->stats.packet_sent++;
@@ -838,6 +857,24 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, uint32_t timestamp)
 	return error;
 }
 
+/**
+ *	Send the rtp datagram @mp to the destination set by rtp_session_set_remote_addr() 
+ *	with timestamp @timestamp. For audio data, the timestamp is the number
+ *	of the first sample resulting of the data transmitted. See rfc1889 for details.
+ *  The packet (@mp) is freed once it is sended.
+ *
+ *@param session a rtp session.
+ *@param mp a rtp packet presented as a mblk_t.
+ *@param timestamp the timestamp of the data to be sent.
+ * @return the number of bytes sent over the network.
+**/
+
+int rtp_session_sendm_with_ts(RtpSession *session, mblk_t *packet, uint32_t timestamp){
+	return __rtp_session_sendm_with_ts(session,packet,timestamp,timestamp);
+}
+
+
+
 
 /**
  *	Send a rtp datagram to the destination set by rtp_session_set_remote_addr() containing
@@ -886,6 +923,23 @@ static void payload_type_changed_notify(RtpSession *session, int paytype){
  *	as it want. The RtpSession takes care of synchronisation between the stream timestamp
  *	and the user timestamp given here.
  *
+ *	This function returns the entire packet (with header).
+ *
+ *	The behaviour of this function has changed since version 0.15.0. Previously the payload data could be 
+ *	accessed using  mblk_t::b_cont::b_rptr field of the returned mblk_t.
+ *	This is no more the case.
+ *	The convenient way of accessing the payload data is to use rtp_get_payload() :
+ *	@code
+ *	unsigned char *payload;
+ *	int payload_size;
+ *	payload_size=rtp_get_payload(mp,&payload);
+ *	@endcode
+ *	OR simply skip the header this way, the data is then comprised between mp->b_rptr and mp->b_wptr:
+ *	@code
+ *	rtp_get_payload(mp,&mp->b_rptr);
+ *	@endcode
+ *
+ *
  * @param session a rtp session.
  * @param user_ts a timestamp.
  *
@@ -958,18 +1012,14 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 		}
 		rtp = (rtp_header_t *) qfirst(q)->b_rptr;
 		session->rtp.rcv_ts_offset = rtp->timestamp;
-		/* remember the timestamp offset between the stream timestamp (random)
-		 * and the user timestamp, that very often starts at zero */
-		session->rtp.hwrcv_diff_ts = rtp->timestamp - user_ts;
-		/* remember the difference between the last received on the socket timestamp and the user timestamp */
-		session->rtp.rcv_diff_ts=session->rtp.hwrcv_diff_ts - session->rtp.jittctl.jitt_comp_ts;
 		session->rtp.rcv_last_ret_ts = user_ts;	/* just to have an init value */
 		session->rcv.ssrc = rtp->ssrc;
 		/* delete the recv synchronisation flag */
 		rtp_session_unset_flag (session, RTP_SESSION_RECV_SYNC);
 	}
+
 	/*calculate the stream timestamp from the user timestamp */
-	ts = user_ts + session->rtp.rcv_diff_ts;
+	ts = jitter_control_get_compensated_timestamp(&session->rtp.jittctl,user_ts);
 	if (session->rtp.jittctl.enabled==TRUE){
 		if (session->permissive)
 			mp = rtp_getq_permissive(&session->rtp.rq, ts,&rejected);
@@ -998,7 +1048,7 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 		{
 			payload_type_changed_notify(session, rtp->paytype);
 		}
-		/* patch the packet so that it has a timestamp compensated by the 
+		/* update the packet's timestamp so that it corrected by the 
 		adaptive jitter buffer mechanism */
 		if (session->rtp.jittctl.adaptive){
 			uint32_t changed_ts;
@@ -1006,9 +1056,9 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 			timestamps*/
 			if (packet_ts!=session->rtp.rcv_last_ts)
 				jitter_control_update_corrective_slide(&session->rtp.jittctl);
-			changed_ts=packet_ts-session->rtp.jittctl.corrective_slide;
+			changed_ts=packet_ts+session->rtp.jittctl.corrective_slide;
 			rtp->timestamp=changed_ts;
-			/*printf("Returned packet has timestamp %u, with clock slide compensated it is %u\n",packet_ts,rtp->timestamp);*/
+			/*ortp_debug("Returned packet has timestamp %u, with clock slide compensated it is %u",packet_ts,rtp->timestamp);*/
 		}
 		session->rtp.rcv_last_ts = packet_ts;
 		if (!(session->flags & RTP_SESSION_FIRST_PACKET_DELIVERED)){
@@ -1048,38 +1098,6 @@ rtp_session_recvm_with_ts (RtpSession * session, uint32_t user_ts)
 }
 
 
-static int msg_to_buf (mblk_t * mp, uint8_t *buffer, int len)
-{
-	int rlen = len;
-	mblk_t *m, *mprev;
-	int mlen;
-	m = mp->b_cont;
-	mprev = mp;
-	while (m != NULL)
-	{
-		mlen = (int) (m->b_wptr - m->b_rptr);
-		if (mlen <= rlen)
-		{
-			mblk_t *consumed = m;
-			memcpy (buffer, m->b_rptr, mlen);
-			/* go to next mblk_t */
-			mprev->b_cont = m->b_cont;
-			m = m->b_cont;
-			consumed->b_cont = NULL;
-			freeb (consumed);
-			buffer += mlen;
-			rlen -= mlen;
-		}
-		else
-		{		/*if mlen>rlen */
-			memcpy (buffer, m->b_rptr, rlen);
-			m->b_rptr += rlen;
-			return len;
-		}
-	}
-	return len - rlen;
-}
-
 /**
  *	NOTE: use of this function is discouraged when sending payloads other than
  *	pcm/pcmu/pcma/adpcm types.
@@ -1120,125 +1138,40 @@ static int msg_to_buf (mblk_t * mp, uint8_t *buffer, int len)
  *
 **/
 int rtp_session_recv_with_ts (RtpSession * session, uint8_t * buffer,
-			       int len, uint32_t ts, int * have_more)
-{
-	mblk_t *mp;
-	int rlen = len;
-	int wlen, mlen;
-	uint32_t ts_int = 0;	/*the length of the data returned in the user supplied buffer, in TIMESTAMP UNIT */
-	PayloadType *payload;
-	RtpStream *stream=&session->rtp;
-
-	*have_more = 0;
-
-	mp = rtp_session_recvm_with_ts (session, ts);
-	payload =rtp_profile_get_payload (session->rcv.profile,
-					 session->rcv.pt);
-	if (payload==NULL){
-		ortp_warning("rtp_session_recv_with_ts: unable to recv an unsupported payload (%i)",session->rcv.pt);
-		if (mp!=NULL) freemsg(mp);
-		return -1;
+			       int len, uint32_t ts, int * have_more){
+	mblk_t *mp=NULL;
+	int plen,blen=0;
+	*have_more=0;
+	while(1){
+		if (session->pending){
+			mp=session->pending;
+			session->pending=NULL;
+		}else {
+			mp=rtp_session_recvm_with_ts(session,ts);
+			if (mp!=NULL) rtp_get_payload(mp,&mp->b_rptr);
+		}
+		if (mp){
+			plen=mp->b_wptr-mp->b_rptr;
+			if (plen<=len){
+				memcpy(buffer,mp->b_rptr,plen);
+				buffer+=plen;
+				blen+=plen;
+				len-=plen;
+				freemsg(mp);
+				mp=NULL;
+			}else{
+				memcpy(buffer,mp->b_rptr,len);
+				mp->b_rptr+=len;
+				buffer+=len;
+				blen+=len;
+				len=0;
+				session->pending=mp;
+				*have_more=1;
+				break;
+			}
+		}else break;
 	}
-	if (!(session->flags & RTP_SESSION_RECV_SYNC))
-	{
-		//ortp_debug("ts=%i   rcv_last_ret_ts=%i",ts,session->rtp.rcv_last_ret_ts);
-		if (RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN
-		    (ts, session->rtp.rcv_last_ret_ts))
-		{
-			/* the user has missed some data previously, so we are going to give him now. */
-			/* we must tell him to call the function once again with the same timestamp
-			 * by setting *have_more=1 */
-			*have_more = 1;
-		}
-		if (payload->type == PAYLOAD_AUDIO_CONTINUOUS)
-		{
-			ts_int = (len * payload->bits_per_sample) >> 3;
-			session->rtp.rcv_last_ret_ts += ts_int;
-			//ortp_debug("ts_int=%i",ts_int);
-		}
-		else
-			ts_int = 0;
-	}
-	else return 0;
-
-	/* try to fill the user buffer */
-	while (1)
-	{
-
-		if (mp != NULL)
-		{
-			mlen = msgdsize (mp->b_cont);
-			wlen = msg_to_buf (mp, buffer, rlen);
-			buffer += wlen;
-			rlen -= wlen;
-			ortp_debug("mlen=%i wlen=%i rlen=%i", mlen, wlen,
-				   rlen);
-			/* do we fill all the buffer ? */
-			if (rlen > 0)
-			{
-				/* we did not fill all the buffer */
-				freemsg (mp);
-				/* if we have continuous audio, try to get other packets to fill the buffer,
-				 * ie continue the loop */
-				//ortp_debug("User buffer not filled entirely");
-				if (ts_int > 0)
-				{
-					ts = session->rtp.rcv_last_ret_ts;
-					ortp_debug("Need more: will ask for %i.", ts);
-				}
-				else
-					return len - rlen;
-			}
-			else if (mlen > wlen)
-			{
-				int unread =
-					mlen - wlen + (int) (mp->b_wptr -
-						       mp->b_rptr);
-				/* not enough space in the user supplied buffer */
-				/* we re-enqueue the msg with its updated read pointers for next time */
-				ortp_debug ("Re-enqueuing packet.");
-				rtp_putq (&session->rtp.rq, mp);
-				/* quite ugly: I change the stats ... */
-				ortp_global_stats.recv -= unread;
-				stream->stats.recv -= unread;
-				return len;
-			}
-			else
-			{
-				/* the entire packet was written to the user buffer */
-				freemsg (mp);
-				return len;
-			}
-		}
-		else
-		{
-			/* fill with a zero pattern (silence) */
-			if (payload->pattern_length != 0)
-			{
-				int i = 0, j = 0;
-				while (i < rlen)
-				{
-					buffer[i] = payload->zero_pattern[j];
-					i++;
-					j++;
-					if (j <= payload->pattern_length)
-						j = 0;
-				}
-				return len;
-			}
-			*have_more = 0;
-			return 0;
-		}
-		mp = rtp_session_recvm_with_ts (session, ts);
-		payload = rtp_profile_get_payload (session->rcv.profile,
-						 session->rcv.pt);
-		if (payload==NULL){
-			ortp_warning("rtp_session_recv_with_ts: unable to recv an unsupported payload.");
-			if (mp!=NULL) freemsg(mp);
-			return -1;
-		}
-	}
-	return -1;
+	return blen;
 }
 /**
  *	When the rtp session is scheduled and has started to send packets, this function
@@ -1321,6 +1254,10 @@ void rtp_session_release_sockets(RtpSession *session){
 	if (session->rtcp.socket>=0) close_socket (session->rtcp.socket);
 	session->rtp.socket=-1;
 	session->rtcp.socket=-1;
+	if (session->rtp.tr!=NULL)
+	  ortp_free(session->rtp.tr);
+	if (session->rtcp.tr!=NULL)
+	  ortp_free(session->rtcp.tr);
 	session->rtp.tr = 0;
 	session->rtcp.tr = 0;
 
@@ -1383,6 +1320,7 @@ void rtp_session_uninit (RtpSession * session)
 	if (session->sd!=NULL) freemsg(session->sd);
 
 	session->signal_tables = o_list_free(session->signal_tables);
+	msgb_allocator_uninit(&session->allocator);
 }
 
 /**
@@ -1416,12 +1354,12 @@ void rtp_session_reset (RtpSession * session)
 	session->rtp.rcv_time_offset = 0;
 	session->rtp.rcv_ts_offset = 0;
 	session->rtp.rcv_query_ts_offset = 0;
-	session->rtp.rcv_diff_ts = 0;
 	session->rtp.rcv_last_ts = 0;
 	session->rtp.rcv_last_app_ts = 0;
 	session->rtp.hwrcv_extseq = 0;
 	session->rtp.hwrcv_since_last_SR=0;
 	session->rtp.snd_seq = 0;
+	session->rtp.sent_payload_bytes=0;
 	rtp_session_clear_send_error_code(session);
 	rtp_session_clear_recv_error_code(session);
 	rtp_stats_reset(&session->rtp.stats);
@@ -1559,6 +1497,31 @@ void rtp_add_csrc(mblk_t *mp, uint32_t csrc)
 	hdr->cc++;
 }
 
+/**
+ * Get a pointer to the beginning of the payload data of the RTP packet.
+ * @param packet a RTP packet represented as a mblk_t
+ * @param start a pointer to the beginning of the payload data, pointing inside the packet.
+ * @return the length of the payload data.
+**/
+int rtp_get_payload(mblk_t *packet, unsigned char **start){
+	unsigned char *tmp;
+	int header_len=RTP_FIXED_HEADER_SIZE+(rtp_get_cc(packet)*4);
+	tmp=packet->b_rptr+header_len;
+	if (tmp>packet->b_wptr){
+		if (packet->b_cont!=NULL){
+			tmp=packet->b_cont->b_rptr+(header_len- (packet->b_wptr-packet->b_rptr));
+			if (tmp<=packet->b_cont->b_wptr){
+				*start=tmp;
+				return packet->b_cont->b_wptr-tmp;
+			}
+		}
+		ortp_warning("Invalid RTP packet");
+		return -1;
+	}
+	*start=tmp;
+	return packet->b_wptr-tmp;
+}
+
 
 /**
  *  Gets last time a valid RTP or RTCP packet was received.
