@@ -44,7 +44,7 @@ static bool_t try_connect(int fd, const struct sockaddr *dest, socklen_t addrlen
 	return TRUE;
 }
 
-static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_family){
+static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_family, bool_t reuse_addr){
 	int err;
 	int optval = 1;
 	ortp_socket_t sock=-1;
@@ -71,12 +71,13 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 		sock = socket(res->ai_family, res->ai_socktype, 0);
 		if (sock < 0)
 		  continue;
-                
-		err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
-				  (SOCKET_OPTION_VALUE)&optval, sizeof (optval));
-		if (err < 0)
-		{
-			ortp_warning ("Fail to set rtp address reusable: %s.", getSocketError());
+		if (reuse_addr){
+			err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
+					(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
+			if (err < 0)
+			{
+				ortp_warning ("Fail to set rtp address reusable: %s.", getSocketError());
+			}
 		}
 
 		*sock_family=res->ai_family;
@@ -142,12 +143,13 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 	sock = socket (PF_INET, SOCK_DGRAM, 0);
 	
 	if (sock<0) return -1;
-	
-	err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
-			  (SOCKET_OPTION_VALUE)&optval, sizeof (optval));
-	if (err < 0)
-	{
-		ortp_warning ("Fail to set rtp address reusable: %s.",getSocketError());
+	if (reuse_addr){
+		err = setsockopt (sock, SOL_SOCKET, SO_REUSEADDR,
+				(SOCKET_OPTION_VALUE)&optval, sizeof (optval));
+		if (err < 0)
+		{
+			ortp_warning ("Fail to set rtp address reusable: %s.",getSocketError());
+		}
 	}
 
 	err = bind (sock,
@@ -161,26 +163,47 @@ static ortp_socket_t create_and_bind(const char *addr, int port, int *sock_famil
 		return -1;
 	}
 #endif
-	if (sock>=0)
-	{
-#ifdef WIN32
-		/* increase RTP buffer on windows */
-		int bufsize = 32768;
-		err = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&bufsize, sizeof(bufsize)); 
-		if (err == -1) {
-			ortp_warning ("Fail to increase buffer size for socket (port %i): %s.", port, getSocketError());
-		}
-		bufsize = 32768;
-		err = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&bufsize, sizeof(bufsize)); 
-		if (err == -1) {
-			ortp_warning ("Fail to increase buffer size for socket (port %i): %s.", port, getSocketError());
-		}
-#endif
+	if (sock>=0){
 		set_non_blocking_socket (sock);
 	}
 	return sock;
 }
 
+static void set_socket_sizes(int sock, unsigned int sndbufsz, unsigned int rcvbufsz){
+	int err;
+	bool_t done=FALSE;
+	if (sndbufsz>0){
+#ifdef SO_SNDBUFFORCE
+		err = setsockopt(sock, SOL_SOCKET, SO_SNDBUFFORCE, (void *)&sndbufsz, sizeof(sndbufsz)); 
+		if (err == -1) {
+			ortp_error("Fail to increase socket's send buffer size with SO_SNDBUFFORCE: %s.", getSocketError());
+		}else done=TRUE;
+#endif
+		if (!done){
+			err = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&sndbufsz, sizeof(sndbufsz)); 
+			if (err == -1) {
+				ortp_error("Fail to increase socket's send buffer size with SO_SNDBUF: %s.", getSocketError());
+			}
+		}
+	}
+	done=FALSE;
+	if (rcvbufsz>0){
+#ifdef SO_RCVBUFFORCE
+		err = setsockopt(sock, SOL_SOCKET, SO_RCVBUFFORCE, (void *)&rcvbufsz, sizeof(rcvbufsz)); 
+		if (err == -1) {
+			ortp_error("Fail to increase socket's recv buffer size with SO_RCVBUFFORCE: %s.", getSocketError());
+		}
+#endif
+		if (!done){
+			err = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&rcvbufsz, sizeof(rcvbufsz)); 
+			if (err == -1) {
+				ortp_error("Fail to increase socket's recv buffer size with SO_RCVBUF: %s.", getSocketError());
+			}
+		}
+		
+	}
+}
+
 static ortp_socket_t create_and_bind_random(const char *localip, int *sock_family, int *port){
 	int retry;
 	ortp_socket_t sock = -1;
@@ -192,7 +215,8 @@ static ortp_socket_t create_and_bind_random(const char *localip, int *sock_famil
 			localport = (rand () + 5000) & 0xfffe;
 		}
 		while ((localport < 5000) || (localport > 0xffff));
-		sock = create_and_bind(localip, localport, sock_family);
+		/*do not set REUSEADDR in case of random allocation */
+		sock = create_and_bind(localip, localport, sock_family,FALSE);
 		if (sock>=0) {
 			*port=localport;
 			return sock;
@@ -228,15 +252,16 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 	}
 	/* try to bind the rtp port */
 	if (port>0)
-		sock=create_and_bind(addr,port,&sockfamily);
+		sock=create_and_bind(addr,port,&sockfamily,TRUE);
 	else
 		sock=create_and_bind_random(addr,&sockfamily,&port);
 	if (sock>=0){
+		set_socket_sizes(sock,session->rtp.snd_socket_size,session->rtp.rcv_socket_size);
 		session->rtp.sockfamily=sockfamily;
 		session->rtp.socket=sock;
 		session->rtp.loc_port=port;
 		/*try to bind rtcp port */
-		sock=create_and_bind(addr,port+1,&sockfamily);
+		sock=create_and_bind(addr,port+1,&sockfamily,TRUE);
 		if (sock>=0){
 			session->rtcp.sockfamily=sockfamily;
 			session->rtcp.socket=sock;
@@ -505,13 +530,12 @@ static char * ortp_inet_ntoa(struct sockaddr *addr, int addrlen, char *dest, int
  *	Returns: 0 on success.
 **/
 int
-rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
-{
-  return rtp_session_set_remote_addr_and_port (session, addr, port, port+1);
+rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port){
+	return rtp_session_set_remote_addr_full (session, addr, port, port+1);
 }
 
 /**
- *rtp_session_set_remote_addr_and_port:
+ *rtp_session_set_remote_addr_full:
  *@session:		a rtp session freshly created.
  *@addr:		a local IP address in the xxx.xxx.xxx.xxx form.
  *@rtp_port:		a local rtp port.
@@ -525,7 +549,7 @@ rtp_session_set_remote_addr (RtpSession * session, const char * addr, int port)
 **/
 
 int
-rtp_session_set_remote_addr_and_port (RtpSession * session, const char * addr, int rtp_port, int rtcp_port)
+rtp_session_set_remote_addr_full (RtpSession * session, const char * addr, int rtp_port, int rtcp_port)
 {
 	int err;
 #ifdef ORTP_INET6
@@ -637,6 +661,11 @@ rtp_session_set_remote_addr_and_port (RtpSession * session, const char * addr, i
 	return 0;
 }
 
+int
+rtp_session_set_remote_addr_and_port(RtpSession * session, const char * addr, int rtp_port, int rtcp_port){
+	return rtp_session_set_remote_addr_full(session,addr,rtp_port,rtcp_port);
+}
+
 void rtp_session_set_sockets(RtpSession *session, int rtpfd, int rtcpfd)
 {
 	if (rtpfd>=0) set_non_blocking_socket(rtpfd);
@@ -857,7 +886,7 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 		bool_t sock_connected=!!(session->flags & RTP_SOCKET_CONNECTED);
 
 		if (session->rtp.cached_mp==NULL)
-			 session->rtp.cached_mp = allocb (session->recv_buf_size, 0);
+			 session->rtp.cached_mp = msgb_allocator_alloc(&session->allocator,session->recv_buf_size);
 		mp=session->rtp.cached_mp;
 		bufsz=(int) (mp->b_datap->db_lim - mp->b_datap->db_base);
 		if (sock_connected){
@@ -882,7 +911,7 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 			}
 			/* then parse the message and put on queue */
 			mp->b_wptr+=error;
-			rtp_session_rtp_parse (session, mp, user_ts + session->rtp.hwrcv_diff_ts, (struct sockaddr*)&remaddr,addrlen);
+			rtp_session_rtp_parse (session, mp, user_ts, (struct sockaddr*)&remaddr,addrlen);
 			session->rtp.cached_mp=NULL;
 			/*for bandwidth measurements:*/
 			update_recv_bytes(session,error);
@@ -916,8 +945,7 @@ void rtp_session_notify_inc_rtcp(RtpSession *session, mblk_t *m){
 		d->packet=m;
 		rtp_session_dispatch_event(session,ev);
 	}
-	else
-	  freemsg(m);  /* avoid memory leak */
+	else freemsg(m);  /* avoid memory leak */
 }
 
 int
